-- See the Copyright notice at the end of this file.
--
class HTTP_CONNECTION
	--
	-- Handle one connection to the HTTP_SERVER. The real HTTP protocol is defined here.
	--

inherit
	CONNECTION
		redefine set_io
		end
	DISPOSABLE

creation {ANY}
	make

feature {LOOP_ITEM}
	continue is
		do
			ios.read_line
			state.call([ios.last_string])
			if method_handler.is_ready then
				method_handler.answer
			end
		end

feature {HTTP_SERVER}
	set_server (a_server: like server) is
		do
			server := a_server
			state := states.item(0)
		end

feature {SERVER}
	set_io (a_io: like ios) is
		do
			Precursor(a_io)
			a_io.unset_timeout
			a_io.when_disconnect(agent handle_disconnect)
		end

feature {}
	method_handler_factory: FUNCTION[TUPLE[STRING, STRING, STRING, OUTPUT_STREAM], HTTP_METHOD_HANDLER]
			-- The first argument is the method, the second is the URI, the third is the version; the fourth is
			-- the output stream to the client socket

	make (a_method_handler_factory: like method_handler_factory) is
		do
			method_handler_factory := a_method_handler_factory
		end

feature {}
	server: HTTP_SERVER

	handle_disconnect (a_io: like ios) is
		require
			a_io = ios
			done
		do
			server.connection_done(Current)
		end

feature {} -- The HTTP protocol (see RFC 2616)
	state: PROCEDURE[TUPLE[STRING]]

	states: FAST_ARRAY[PROCEDURE[TUPLE[STRING]]] is
		once
			Result := {FAST_ARRAY[PROCEDURE[TUPLE[STRING]]] << agent a_request_line, agent a_header, agent a_body >> }
		end

	method_handler: HTTP_METHOD_HANDLER

	a_request_line (line: STRING) is
		local
			i, j: INTEGER
			method, uri, version: STRING
		do
			io.put_string(line)
			io.put_new_line
			i := line.first_index_of(' ')
			j := line.index_of(' ', i + 1)
			method := line.substring(1, i - 1)
			uri := line.substring(i + 1, j - 1)
			version := line.substring(j + 1, line.count)
			io.put_string("%Tmethod: " + method + "%N")
			io.put_string("%Turi: " + uri + "%N")
			state := states.item(1)
			method_handler := method_handler_factory.item([method, uri, version, ios])
			if method_handler = Void then
				create {HTTP_NO_METHOD_HANDLER} method_handler.make(method, uri, version, ios)
			end
		end

	a_header (line: STRING) is
		do
			method_handler.add_header(line.twin)
		end

	a_body (line: STRING) is
		do
			method_handler.add_body(line.twin)
		end

feature {}
	dispose is
		do
			handle_disconnect(ios)
		end

end -- class HTTP_CONNECTION
--
-- ------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is part of the SmartEiffel standard library.
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2006: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
-- documentation files (the "Software"), to deal in the Software without restriction, including without
-- limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
-- the Software, and to permit persons to whom the Software is furnished to do so, subject to the following
-- conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or substantial
-- portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO
-- EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
-- AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
-- OR OTHER DEALINGS IN THE SOFTWARE.
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------
