-- See the Copyright notice at the end of this file.
--
deferred class GL_WIDGET

inherit
	SUB_WINDOW
		redefine make
		end

insert
	GRAPHIC
		undefine default_create
		end

feature {ANY}
	make (p: like parent) is
		do
			width := 1
			height := 1
			min_width := 1
			min_height := 1
			std_width := 1
			std_height := 1
			make_with_size(p, p.width, p.height)
		end

	make_with_size (p: like parent; w, h: INTEGER) is
		local
			default_layout: ROW_LAYOUT
		do
			create default_layout
			width := w
			height := h
			min_width := 1
			min_height := 1
			std_width := w
			std_height := h
			opengl_widget := basic_opengl_window_create(0, 0, w, h, p.widget, False)
			widget := basic_opengl_get_widget(opengl_widget)
			container_init
			p.child_attach(Current)
			vision.register(Current)
			layout := default_layout
			layout.set_container(Current)
			set_shrink(False)
			set_expand(False)
			init_gl
			when_expose(agent redraw_gl)
			set_geometry(0, 0, w, h)
			resize_gl(w, h)
		end

	select_myself_gl is
		local
			ok: BOOLEAN
		do
			ok := basic_opengl_select(opengl_widget)
		end

	display_gl is
		do
			basic_opengl_swap_buffers(opengl_widget)
		end

	init_gl is
		deferred
		end

	redraw_gl is
		deferred
		end

	resize_gl (w, h: INTEGER) is
		deferred
		end

	quit_gl is
		require
			opengl_widget.is_not_null
		do
			basic_opengl_window_destroy(opengl_widget)
		end

	post_redisplay is
		do
			basic_opengl_post_redisplay(opengl_widget, 0, 0, width, height)
		end

feature {}
	opengl_widget: POINTER

	basic_opengl_window_create (x, y, w, h: INTEGER; p: POINTER; decorate: BOOLEAN): POINTER is
		external "plug_in"
		alias "{
			location: "${sys}plugins/vision"
			module_name: "opengl"
			feature_name: "basic_opengl_window_create"
			}"
		end

	basic_opengl_get_widget (ogl_widget: POINTER): POINTER is
		external "plug_in"
		alias "{
			location: "${sys}plugins/vision"
			module_name: "opengl"
			feature_name: "basic_opengl_get_widget"
			}"
		end

	basic_opengl_select (ogl_widget: POINTER): BOOLEAN is
		external "plug_in"
		alias "{
			location: "${sys}plugins/vision"
			module_name: "opengl"
			feature_name: "basic_opengl_select"
			}"
		end

	basic_opengl_swap_buffers (ogl_widget: POINTER) is
		external "plug_in"
		alias "{
			location: "${sys}plugins/vision"
			module_name: "opengl"
			feature_name: "basic_opengl_swap_buffers"
			}"
		end

	basic_opengl_post_redisplay (ogl_widget: POINTER; x, y, w, h: INTEGER) is
		external "plug_in"
		alias "{
			location: "${sys}plugins/vision"
			module_name: "opengl"
			feature_name: "basic_opengl_post_redisplay"
			}"
		end

	basic_opengl_window_destroy (ogl_widget: POINTER) is
		external "plug_in"
		alias "{
			location: "${sys}plugins/vision"
			module_name: "opengl"
			feature_name: "basic_opengl_window_destroy"
			}"
		end

end -- class GL_WIDGET
--
-- ------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is part of the SmartEiffel standard library.
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2006: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
-- documentation files (the "Software"), to deal in the Software without restriction, including without
-- limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
-- the Software, and to permit persons to whom the Software is furnished to do so, subject to the following
-- conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or substantial
-- portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO
-- EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
-- AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
-- OR OTHER DEALINGS IN THE SOFTWARE.
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------
