-- See the Copyright notice at the end of this file.
--
class XML_OBJECT_LOADER[E_ -> LOADABLE_GL_OBJECT]

inherit
	XML_CALLBACKS
	OBJECT_LOADER[E_]

creation {ANY}
	make

feature {ANY}
	load_from_file (complete_file_name: STRING): E_ is
		require
			universal_texture_loader /= Void
		local
			tfr: TEXT_FILE_READ; parser: XML_PARSER
		do
			loaded_file := complete_file_name
			create tfr.connect_to(loaded_file)
			create coords.make(3)
			create coordt.make(3)
			create object.make_at_origin
			create parser.connect_to(tfr)
			parser.parse(Current)
			Result := object
		end

	with_attribute (attribute_name, attribute_value: STRING) is
		do
			if attribute_name.is_equal(fz_coordx) then
				coordx := attribute_value.to_real
			elseif attribute_name.is_equal(fz_coordy) then
				coordy := attribute_value.to_real
			elseif attribute_name.is_equal(fz_coordz) then
				coordz := attribute_value.to_real
			elseif attribute_name.is_equal(fz_red) then
				red := attribute_value.to_real
			elseif attribute_name.is_equal(fz_green) then
				green := attribute_value.to_real
			elseif attribute_name.is_equal(fz_blue) then
				blue := attribute_value.to_real
			elseif attribute_name.is_equal(fz_alpha) then
				alpha := attribute_value.to_real
			elseif attribute_name.is_equal(fz_file) then
				file := attribute_value
			elseif attribute_name.is_equal(fz_coords) then
				coords.put(attribute_value.to_real, coord_index)
			elseif attribute_name.is_equal(fz_coordt) then
				coordt.put(attribute_value.to_real, coord_index)
			elseif attribute_name.is_equal(fz_function) then
				function := attribute_value
			end
		end

	open_node (node_name: STRING) is
		do
			current_node := node_name
			if current_node.is_equal(fz_object) then
				index := 0
				coord_index := 0
				is_object := True
			elseif current_node.is_equal(fz_facet) and then is_object then
				is_object := False
				is_facet := True
			elseif current_node.is_equal(fz_vertex) and then is_facet then
				create current_vertex.make(coordx, coordy, coordz)
				if vertex1 = Void then
					vertex1 := current_vertex
				elseif vertex2 = Void then
					vertex2 := current_vertex
				elseif vertex3 = Void then
					vertex3 := current_vertex
				end
				is_facet := False
				is_vertex := True
			elseif current_node.is_equal(fz_color) and then is_vertex then
				current_vertex.set_color(create {GL_COLOR}.set_color(red, green, blue, alpha))
				is_vertex := False
				is_color := True
			elseif current_node.is_equal(fz_normal) and then is_vertex then
				current_vertex.set_normal(coordx, coordy, coordz)
				is_vertex := False
				is_normal := True
			elseif current_node.is_equal(fz_texture) and then is_facet then
				is_facet := False
				is_texture := True
			elseif current_node.is_equal(fz_coord) and then is_texture then
				is_texture := False
				is_coord := True
			elseif current_node.is_equal(fz_composition) and then is_texture then
				is_texture := False
				is_composition := True
			end
		end

	close_node (node_name: STRING) is
		local
			facet: FACET
		do
			if current_node.is_equal(fz_object) and then is_object then
				is_object := False
			elseif current_node.is_equal(fz_facet) and then is_facet then
				create facet.make(vertex1, vertex2, vertex3)
				if facet.is_translucent then
					object.add_translucent_facet(facet)
				else
					object.add_opaque_facet(facet)
				end
				if has_texture then
					facet.set_texture(universal_texture_loader.load_texture(file))
					vertex1.set_2d_texture_coord(coords.item(coords.lower), coordt.item(coordt.lower))
					vertex2.set_2d_texture_coord(coords.item(coords.lower + 1), coordt.item(coordt.lower + 1))
					vertex3.set_2d_texture_coord(coords.item(coords.lower + 2), coordt.item(coordt.lower + 2))
				end
				if has_function then
					inspect
						function
					when "blend" then
						facet.set_texture_composition_function(gl.const_blend)
					when "modulate" then
						facet.set_texture_composition_function(gl.const_modulate)
					when "replace" then
						facet.set_texture_composition_function(gl.const_replace)
					when "decal" then
						facet.set_texture_composition_function(gl.const_decal)
					end
				end
				index := index + 1
				vertex1 := Void
				vertex2 := Void
				vertex3 := Void
				has_texture := False
				coord_index := 0
				is_facet := False
				is_object := True
			elseif current_node.is_equal(fz_vertex) and then is_vertex then
				is_vertex := False
				is_facet := True
			elseif current_node.is_equal(fz_color) and then is_color then
				is_color := False
				is_vertex := True
			elseif current_node.is_equal(fz_normal) and then is_normal then
				is_normal := False
				is_vertex := True
			elseif current_node.is_equal(fz_texture) and then is_texture then
				has_texture := True
				is_texture := False
				is_facet := True
			elseif current_node.is_equal(fz_coord) and then is_coord then
				coord_index := coord_index + 1
				is_coord := False
				is_texture := True
			elseif current_node.is_equal(fz_composition) and then is_composition then
				is_composition := False
				is_texture := True
			end
		end

	open_close_node (node_name: STRING) is
		do
			open_node(node_name)
			close_node(node_name)
		end

	current_node: STRING

	data (a_data: STRING) is
		do
		end

	parse_error is
		do
			io.put_string("Error while loading file `")
			io.put_string(loaded_file)
			io.put_string("'%N")
		end

feature {}
	object: E_

	loaded_file, file: STRING

	index: INTEGER

	current_vertex, vertex1, vertex2, vertex3: VERTEX

	coordx, coordy, coordz: REAL

	coords, coordt: FAST_ARRAY[REAL]

	coord_index: INTEGER

	red, green, blue, alpha: REAL

	function: STRING

	fz_object: STRING is "object"

	fz_coordx: STRING is "x"

	fz_coordy: STRING is "y"

	fz_coordz: STRING is "z"

	fz_facet: STRING is "facet"

	fz_vertex: STRING is "vertex"

	fz_color: STRING is "color"

	fz_normal: STRING is "normal"

	fz_red: STRING is "r"

	fz_green: STRING is "g"

	fz_blue: STRING is "b"

	fz_alpha: STRING is "a"

	fz_texture: STRING is "texture"

	fz_file: STRING is "file"

	fz_coord: STRING is "coord"

	fz_coords: STRING is "s"

	fz_coordt: STRING is "t"

	fz_composition: STRING is "composition"

	fz_function: STRING is "function"

	is_object, is_facet, is_vertex, is_color, is_normal, is_texture, is_coord, is_composition: BOOLEAN

	has_texture, has_function: BOOLEAN

end -- class XML_OBJECT_LOADER
--
-- ------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is part of the SmartEiffel standard library.
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2006: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
-- documentation files (the "Software"), to deal in the Software without restriction, including without
-- limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
-- the Software, and to permit persons to whom the Software is furnished to do so, subject to the following
-- conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or substantial
-- portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO
-- EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
-- AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
-- OR OTHER DEALINGS IN THE SOFTWARE.
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------
