-- See the Copyright notice at the end of this file.
--
class XML_WORLD_LOADER[E_ -> LOADABLE_GL_OBJECT]

inherit
	XML_CALLBACKS
	WORLD_LOADER[E_]

creation {ANY}
	make

feature {ANY}
	load_from_file (world: WORLD; complete_file_name: STRING) is
		local
			tfr: TEXT_FILE_READ; parser: XML_PARSER
		do
			loaded_file := complete_file_name
			loaded_world := world
			current_light_id := gl.const_light0
			create tfr.connect_to(loaded_file)
			create parser.connect_to(tfr)
			parser.parse(Current)
		end

	with_attribute (attribute_name, attribute_value: STRING) is
		do
			if attribute_name.is_equal(fz_coordx) then
				coordx := attribute_value.to_real
			elseif attribute_name.is_equal(fz_coordy) then
				coordy := attribute_value.to_real
			elseif attribute_name.is_equal(fz_coordz) then
				coordz := attribute_value.to_real
			elseif attribute_name.is_equal(fz_file) then
				file := attribute_value
			elseif attribute_name.is_equal(fz_angleh) then
				angleh := attribute_value.to_real
			elseif attribute_name.is_equal(fz_anglep) then
				anglep := attribute_value.to_real
			elseif attribute_name.is_equal(fz_r) then
				r_value := attribute_value.to_real
			elseif attribute_name.is_equal(fz_type) then
				current_type := attribute_value
			elseif attribute_name.is_equal(fz_green) then
				green := attribute_value.to_real
			elseif attribute_name.is_equal(fz_blue) then
				blue := attribute_value.to_real
			elseif attribute_name.is_equal(fz_alpha) then
				alpha := attribute_value.to_real
			elseif attribute_name.is_equal(fz_start) then
				f_start := attribute_value.to_real
			elseif attribute_name.is_equal(fz_end) then
				f_end := attribute_value.to_real
			elseif attribute_name.is_equal(fz_density) then
				f_density := attribute_value.to_real
			elseif attribute_name.is_equal(fz_atmospheric) then
				if attribute_value.is_boolean then
					is_atmospheric := attribute_value.to_boolean
				else
					is_atmospheric := False
				end
			end
		end

	open_node (node_name: STRING) is
		do
			current_node := node_name
			if current_node.is_equal(fz_world) then
				coordx := 0
				coordy := 0
				coordz := 0
				is_world := True
			elseif current_node.is_equal(fz_camera) and then is_world then
				is_world := False
				is_camera := True
			elseif current_node.is_equal(fz_fog) and then is_world then
				inspect
					current_type
				when "linear" then
					create current_fog.make_linear(f_start, f_end)
				when "exp" then
					create current_fog.make_exp(f_start, f_end, f_density)
				when "exp2" then
					create current_fog.make_exp2(f_start, f_end, f_density)
				else
					io.put_string("Warning: no fog type in ")
					io.put_string(loaded_file)
					io.put_new_line
				end
				is_world := False
				is_fog := True
			elseif current_node.is_equal(fz_light) and then is_world then
				inspect
					current_type
				when "directional" then
					is_directional := True
					is_spot := False
				when "spot" then
					is_directional := False
					is_spot := True
				else
					io.put_string("Warning: no light type in ")
					io.put_string(loaded_file)
					io.put_new_line
					is_directional := False
					is_spot := False
				end
				create current_light.make(current_light_id)
				if current_light_id = gl.const_light0 then
					current_light_id := gl.const_light1
				elseif current_light_id = gl.const_light1 then
					current_light_id := gl.const_light2
				elseif current_light_id = gl.const_light2 then
					current_light_id := gl.const_light3
				elseif current_light_id = gl.const_light3 then
					current_light_id := gl.const_light4
				elseif current_light_id = gl.const_light4 then
					current_light_id := gl.const_light5
				elseif current_light_id = gl.const_light5 then
					current_light_id := gl.const_light6
				elseif current_light_id = gl.const_light6 then
					current_light_id := gl.const_light7
				else
					io.put_string("Warning: check number of lights in ")
					io.put_string(loaded_file)
					io.put_new_line
					current_light_id := current_light_id + 1
				end
				if is_spot then
					current_light.set_spot_position(0, 0, 0)
				end
				is_world := False
				is_light := True
			elseif current_node.is_equal(fz_object) and then is_world then
				is_world := False
				is_object := True
			elseif current_node.is_equal(fz_position) and then (is_object or else is_camera or else is_light) then
				if is_light and then is_spot then
					current_light.set_spot_position(coordx, coordy, coordz)
				end
				is_position := True
			elseif current_node.is_equal(fz_orientation) and then (is_object or else is_camera or else is_light) then
				is_orientation := True
			elseif current_node.is_equal(fz_direction) and then is_light then
				if is_directional then
					current_light.set_directional(coordx, coordy, coordz)
				elseif is_spot then
					current_light.set_spot_direction(coordx, coordy, coordz)
				end
				is_light := False
				is_direction := True
			elseif current_node.is_equal(fz_ambient) and then is_light then
				current_light.set_ambient(create {GL_COLOR}.set_color(r_value, green, blue, alpha))
				is_light := False
				is_ambient := True
			elseif current_node.is_equal(fz_diffuse) and then is_light then
				current_light.set_diffuse(create {GL_COLOR}.set_color(r_value, green, blue, alpha))
				is_light := False
				is_diffuse := True
			elseif current_node.is_equal(fz_specular) and then is_light then
				current_light.set_specular(create {GL_COLOR}.set_color(r_value, green, blue, alpha))
				is_light := False
				is_specular := True
			elseif current_node.is_equal(fz_color) and then is_fog then
				if current_fog /= Void then
					current_fog.set_color(create {GL_COLOR}.set_color(r_value, green, blue, alpha))
				end
				is_fog := False
				is_color := True
			end
		end

	close_node (node_name: STRING) is
		local
			camera: CAMERA; object: E_
		do
			if current_node.is_equal(fz_world) and then is_world then
				is_world := False
			elseif current_node.is_equal(fz_camera) and then is_camera then
				create camera.make_with_position(coordx, coordy, coordz)
				loaded_world.set_camera(camera)
				if has_orientation then
					camera.set_orientation(angleh, anglep, r_value)
				end
				camera.update_direction
				has_orientation := False
				is_camera := False
				is_world := True
			elseif current_node.is_equal(fz_fog) and then is_fog then
				if current_fog /= Void then
					loaded_world.set_fog(current_fog, is_atmospheric)
				end
				is_fog := False
				is_world := True
			elseif current_node.is_equal(fz_light) and then is_light then
				loaded_world.add_light(current_light)
				is_light := False
				is_world := True
			elseif current_node.is_equal(fz_object) and then is_object then
				object := universal_object_loader.load_object(file, universal_texture_loader)
				if object.is_translucent then
					loaded_world.add_translucent_object(object)
				else
					loaded_world.add_opaque_object(object)
				end
				object.set_position(coordx, coordy, coordz)
				if has_orientation then
					object.set_orientation(angleh, anglep, r_value)
				end
				has_orientation := False
				is_object := False
				is_world := True
			elseif current_node.is_equal(fz_position) and then is_position then
				is_position := False
			elseif current_node.is_equal(fz_orientation) and then is_orientation then
				has_orientation := True
				is_orientation := False
			elseif current_node.is_equal(fz_direction) and then is_direction then
				is_direction := False
				is_light := True
			elseif current_node.is_equal(fz_ambient) and then is_ambient then
				is_ambient := False
				is_light := True
			elseif current_node.is_equal(fz_diffuse) and then is_diffuse then
				is_diffuse := False
				is_light := True
			elseif current_node.is_equal(fz_specular) and then is_specular then
				is_specular := False
				is_light := True
			elseif current_node.is_equal(fz_color) and then is_color then
				is_color := False
				is_fog := True
			end
		end

	open_close_node (node_name: STRING) is
		do
			open_node(node_name)
			close_node(node_name)
		end

	current_node: STRING

	data (a_data: STRING) is
		do
		end

	parse_error is
		do
			io.put_string("Error while loading file `")
			io.put_string(loaded_file)
			io.put_string("'%N")
		end

feature {}
	loaded_file, file: STRING

	loaded_world: WORLD

	coordx, coordy, coordz: REAL

	green, blue, alpha: REAL

	angleh, anglep, r_value: REAL

	current_light: LIGHT

	current_light_id: INTEGER

	current_type: STRING

	f_start, f_end, f_density: REAL

	current_fog: FOG

	fz_world: STRING is "world"

	fz_camera: STRING is "camera"

	fz_fog: STRING is "fog"

	fz_start: STRING is "start"

	fz_end: STRING is "end"

	fz_density: STRING is "density"

	fz_atmospheric: STRING is "atmospheric"

	fz_light: STRING is "light"

	fz_type: STRING is "type"

	fz_directional: STRING is "directional"

	fz_spot: STRING is "spot"

	fz_direction: STRING is "direction"

	fz_ambient: STRING is "ambient"

	fz_diffuse: STRING is "diffuse"

	fz_specular: STRING is "specular"

	fz_object: STRING is "object"

	fz_file: STRING is "file"

	fz_position: STRING is "position"

	fz_coordx: STRING is "x"

	fz_coordy: STRING is "y"

	fz_coordz: STRING is "z"

	fz_orientation: STRING is "orientation"

	fz_angleh: STRING is "h"

	fz_anglep: STRING is "p"

	fz_r: STRING is "r"

	fz_green: STRING is "g"

	fz_blue: STRING is "b"

	fz_alpha: STRING is "a"

	fz_color: STRING is "color"

	is_world, is_camera, is_light, is_object, is_position, is_orientation, is_direction, is_ambient, is_diffuse, is_specular, is_fog, is_color: BOOLEAN

	has_orientation: BOOLEAN

	is_directional, is_spot, is_atmospheric: BOOLEAN

end -- class XML_WORLD_LOADER
--
-- ------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is part of the SmartEiffel standard library.
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2006: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
-- documentation files (the "Software"), to deal in the Software without restriction, including without
-- limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
-- the Software, and to permit persons to whom the Software is furnished to do so, subject to the following
-- conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or substantial
-- portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO
-- EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
-- AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
-- OR OTHER DEALINGS IN THE SOFTWARE.
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------
