-- See the Copyright notice at the end of this file.
--
deferred class XML_CALLBACKS
	--
	-- Used by XML_PARSER to implement XML callbacks
	--
	-- CAVEAT: expect those features to always receive the same STRING object with different values. You
	-- should take care of twinning the object if you want to keep it.
	--

feature {XML_PARSER}
	with_attribute (attribute_name: STRING; attribute_value: STRING) is
			-- To be called to add all the attributed of a node BEFORE calling `open_node'
		require
			attribute_name /= Void
			attribute_value /= Void
		deferred
		end

	open_node (node_name: STRING) is
		require
			node_name /= Void
		deferred
		ensure
			current_node.is_equal(node_name)
		end

	close_node (node_name: STRING) is
		require
			current_node.is_equal(node_name)
		deferred
		end

	open_close_node (node_name: STRING) is
		require
			node_name /= Void
		deferred
		end

	current_node: STRING is
		deferred
		end

	data (a_data: STRING) is
		require
			a_data /= Void
		deferred
		end

	parse_error is
		deferred
		end

end -- class XML_CALLBACKS
--
-- ------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is part of the SmartEiffel standard library.
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2006: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
-- documentation files (the "Software"), to deal in the Software without restriction, including without
-- limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
-- the Software, and to permit persons to whom the Software is furnished to do so, subject to the following
-- conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or substantial
-- portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO
-- EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
-- AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
-- OR OTHER DEALINGS IN THE SOFTWARE.
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------
