-- See the Copyright notice at the end of this file.
--
class XML_PARSER
	-- This parser is still rudimentary

creation {ANY}
	connect_to, make

feature {ANY}
	parse (a_callbacks: like callbacks) is
		local
			pn: like parse_node
		do
			callbacks := a_callbacks
			pn := parse_node
			if pn = Parse_error then
				callbacks.parse_error
			end
		end

	connect_to (a_stream: like stream) is
		require
			a_stream.is_connected
		do
			stream := a_stream
			stream.read_character -- be sure to have a character available
		end

feature {}
	Parse_again: INTEGER is 0

	Parse_done: INTEGER is 1

	Parse_error: INTEGER is -1

feature {}
	stream: INPUT_STREAM

	callbacks: XML_CALLBACKS

	parse_node: INTEGER is
			-- True if a node was successfully parsed
		local
			name: STRING; data, data_blanks: STRING; done, open, open_close: BOOLEAN
		do
			skip_blanks
			from
				data := ""
				data.clear_count
				data_blanks := ""
				data_blanks.clear_count
			until
				stream.end_of_input or else skip('<')
			loop
				if stream.last_character.is_separator then
					data_blanks.add_last(stream.last_character)
				else
					if not data_blanks.is_empty then
						data.append_string(data_blanks)
						data_blanks.clear_count
					end
					data.add_last(stream.last_character)
				end
				stream.read_character
			end
			if stream.end_of_input then
				Result := Parse_done
			else
				if not data.is_empty then
					callbacks.data(data)
				end
				if skip('/') then
					skip_blanks
					name := read_name
					if name = Void then
						callbacks.parse_error
						Result := Parse_error
					else
						skip_blanks
						if not skip('>') then
							callbacks.parse_error
							Result := Parse_error
						else
							if callbacks.current_node.is_equal(name) then
								callbacks.close_node(name)
								Result := Parse_done
							else
								callbacks.parse_error
								Result := Parse_error
							end
						end
					end
				else
					skip_blanks
					name := read_name
					if name = Void then
						callbacks.parse_error
						Result := Parse_error
					else
						skip_blanks
						if skip('>') then
							callbacks.open_node(name)
							Result := parse_children
						elseif skip2('/', '>') then
							callbacks.open_close_node(name)
							Result := Parse_again
						else
							from
								Result := parse_attribute
								if skip('>') then
									done := True
									open := True
								elseif skip2('/', '>') then
									done := True
									open_close := True
								else
									done := Result /= Parse_again
								end
							until
								done
							loop
								Result := parse_attribute
								if skip('>') then
									done := True
									open := True
								elseif skip2('/', '>') then
									done := True
									open_close := True
								else
									done := Result /= Parse_again
								end
							end
							if Result /= Parse_error then
								if open then
									callbacks.open_node(name)
									Result := parse_children
								elseif open_close then
									callbacks.open_close_node(name)
								end
							end
						end
					end
				end
			end
		ensure
			(
			<<Parse_again, Parse_done, Parse_error>>).has(Result)
		end

	read_name: STRING is
		local
			name: STRING
		do
			name := read_identifier
			if name /= Void and then not name.is_empty then
				Result := once ""
				Result.copy(name)
			end
		end

	parse_attribute: INTEGER is
		local
			a, attribute, value: STRING
		do
			skip_blanks
			a := read_identifier
			if a = Void or else a.is_empty then
				Result := Parse_done
			else
				attribute := once ""
				attribute.copy(a)
				skip_blanks
				if not skip('=') then
					callbacks.parse_error
					Result := Parse_error
				else
					skip_blanks
					value := read_string
					if value = Void then
						callbacks.parse_error
						Result := Parse_error
					else
						callbacks.with_attribute(attribute, value)
						skip_blanks
					end
				end
			end
		end

	parse_children: INTEGER is
		do
			from
				Result := parse_node
			until
				Result /= Parse_again
			loop
				Result := parse_node
			end
			if Result /= Parse_error then
				Result := Parse_again
			end
		ensure
			(
			<<Parse_again, Parse_done, Parse_error>>).has(Result)
		end

	skip_blanks is
		do
			from
			until
				stream.end_of_input or else not stream.last_character.is_separator
			loop
				stream.read_character
			end
		end

	skip (character: CHARACTER): BOOLEAN is
		do
			if not stream.end_of_input then
				if stream.last_character = character then
					Result := True
					stream.read_character
				end
			end
		end

	skip2 (char1, char2: CHARACTER): BOOLEAN is
		do
			Result := skip(char1) and then skip(char2)
		end

	read_identifier: STRING is
		local
			done: BOOLEAN
		do
			from
				if stream.end_of_input then
					done := True
				else
					Result := once ""
					Result.clear_count
				end
			until
				done
			loop
				if stream.end_of_input then
					done := True
				else
					inspect
						stream.last_character
					when '0' .. '9', 'A' .. 'Z', 'a' .. 'z', ':', '_', '-' then
						Result.extend(stream.last_character)
						stream.read_character
					else
						done := True
					end
				end
			end
		end

	read_string: STRING is
		local
			done: BOOLEAN; limit: CHARACTER
		do
			from
				if stream.end_of_input then
					done := True
				else
					limit := stream.last_character
					inspect
						limit
					when '"', '%'' then
						create Result.make_empty
						stream.read_character
					else
						done := True
					end
				end
			until
				done
			loop
				if stream.end_of_input then
					done := True
				else
					if stream.end_of_input then
						Result := Void
						done := True
					elseif stream.last_character = limit then
						stream.read_character
						done := True
					else
						Result.extend(stream.last_character)
						stream.read_character
					end
				end
			end
		end

feature {}
	make is
			-- Create a not connected parser
		do
		end

end -- class XML_PARSER
--
-- ------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is part of the SmartEiffel standard library.
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2006: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
-- documentation files (the "Software"), to deal in the Software without restriction, including without
-- limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
-- the Software, and to permit persons to whom the Software is furnished to do so, subject to the following
-- conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or substantial
-- portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO
-- EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
-- AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
-- OR OTHER DEALINGS IN THE SOFTWARE.
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------
