-- See the Copyright notice at the end of this file.
--
class XML_TREE
	--
	-- DOM-like representation of an XML tree
	--

inherit
	XML_CALLBACKS

creation {ANY}
	make

feature {ANY}
	root: XML_NODE
			-- The root of the tree

feature {}
	attributes: DICTIONARY[STRING, STRING]

	open_nodes: STACK[XML_NODE]

feature {XML_PARSER}
	with_attribute (attribute_name: STRING; attribute_value: STRING) is
		do
			attributes.put(attribute_name, attribute_value)
		end

	open_node (node_name: STRING) is
		local
			node: XML_NODE; i: INTEGER
		do
			create node.make(node_name)
			from
				i := attributes.lower
			until
				i > attributes.upper
			loop
				node.set_attribute(attributes.key(i), attributes.item(i))
				i := i + 1
			end
			attributes.clear_count
			open_nodes.push(node)
		end

	close_node (node_name: STRING) is
		local
			node: XML_NODE
		do
			node := open_nodes.top
			open_nodes.pop
			if open_nodes.is_empty then
				root := node
			else
				open_nodes.top.add_child(node)
			end
		end

	open_close_node (node_name: STRING) is
		do
			open_node(node_name)
			close_node(node_name)
		end

	current_node: STRING is
		do
			Result := open_nodes.top.name
		end

	data (a_data: STRING) is
		do
			open_nodes.top.set_data(a_data)
		end

	parse_error is
		do
			not_yet_implemented
		end

feature {}
	make (in: INPUT_STREAM) is
			-- read the xml tree in the given input stream
		require
			in.is_connected
		do
			create {HASHED_DICTIONARY[STRING, STRING]} attributes.make
			parser.connect_to(in)
			parser.parse(Current)
		end

	parser: XML_PARSER is
		once
			create Result.make
		end

end -- class XML_TREE
--
-- ------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is part of the SmartEiffel standard library.
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2006: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
-- documentation files (the "Software"), to deal in the Software without restriction, including without
-- limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
-- the Software, and to permit persons to whom the Software is furnished to do so, subject to the following
-- conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or substantial
-- portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO
-- EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
-- AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
-- OR OTHER DEALINGS IN THE SOFTWARE.
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------
