/*
-- ------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2005: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
-- documentation files (the "Software"), to deal in the Software without restriction, including without
-- limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
-- the Software, and to permit persons to whom the Software is furnished to do so, subject to the following
-- conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or substantial
-- portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO
-- EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
-- AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
-- OR OTHER DEALINGS IN THE SOFTWARE.
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------
*/
#if defined __USE_POSIX || defined __unix__ || defined _POSIX_C_SOURCE

se_exec_data_t* basic_exec_execute(char*prog, char**args, int keep_env, int* in_fd, int* out_fd, int* err_fd) {
  se_exec_data_t* result = NULL;
  int id = fork();
  if (id == 0) {
    /* child */

    close(0);
    dup2(in_fd[0], 0);
    close(in_fd[1]);

    close(1);
    dup2(out_fd[1], 1);
    close(out_fd[0]);

    close(2);
    dup2(err_fd[1], 2);
    close(err_fd[0]);

    if (!keep_env) {
      clearenv();
    }
    execvp(prog, args); /* NO RETURN in child */
  }
  else if (id > 0) {
    /* father */
    result = (se_exec_data_t*)se_malloc(sizeof(se_exec_data_t));
    result->id = id;
    result->running = 1;
    close(in_fd[0]);
    close(out_fd[1]);
    close(err_fd[1]);
  }
  return result; /* ... in father only */
}

EIF_BOOLEAN basic_exec_is_finished(se_exec_data_t*data) {
  EIF_BOOLEAN result = (EIF_BOOLEAN)0;
  int status;
  if (data->running) {
    int id = waitpid(data->id, &status, WNOHANG);
    if (id != 0) {
      /* child is finished */
      result = (EIF_BOOLEAN)(id == data->id);
      data->status = WEXITSTATUS(status);
      data->running = 0;
    }
  }
  else{
    result = (EIF_BOOLEAN)1;
  }
  return result;
}

void basic_exec_wait(se_exec_data_t*data) {
  int status;
  if (data->running) {
    int id = waitpid(data->id, &status, 0);
    data->status = WEXITSTATUS(status);
    data->running = 0;
  }
}

int* basic_exec_create_pipe(void) {
  int* result = (int*)calloc(2, sizeof(int));
  pipe(result);
  return result;
}

EIF_INTEGER basic_exec_get_character (EIF_INTEGER fd) {
  EIF_INTEGER result = -1;
  char buf[1];
  ssize_t r = read(fd, buf, 1);
  if (r <= 0) {
    result = -1;
  }
  else {
    result = 0xff & ((EIF_INTEGER)(buf[0]));
  }
  return result;
}

void basic_exec_put_character(EIF_INTEGER fd, EIF_CHARACTER c) {
  char buf[1];
  buf[0] = c;
  write(fd, buf, 1);
}

#elif defined WIN32

se_exec_data_t* basic_exec_execute(char*prog, char**args, int keep_env, int* in_fd, int* out_fd, int* err_fd) {
  se_exec_data_t* result = (se_exec_data_t*)se_malloc(sizeof(se_exec_data_t));
  BOOL cp;
  LPCTSTR applicationName = prog;
  LPTSTR commandLine;
  LPVOID environment;
  LPVOID noEnv = { NULL };
  int i, l;
  char** c;

  l = strlen(prog) + 1; /* 1 for the trailing '\0' */
  for (c = args; *c != NULL; c++) {
    l += strlen(*c) + 1; /* 1 is the extra space */
  }
  commandLine = (LPTSTR)se_malloc(l * sizeof(char));
  sprintf(commandLine, "%s", prog);
  for (c = args; *c != NULL; c++) {
    sprintf(commandLine, "%s %s", commandLine, *c);
  }

  if (keep_env) {
    environment = NULL;
  }
  else {
    environment = noEnv;
  }

  cp = CreateProcess(applicationName, commandLine, NULL, NULL, FALSE, 0,
		     environment, NULL, &(result->startupInformation), &(result->processInformation));
  if (!cp) {
    free(result);
    result = NULL;
  }

  return result;
}

EIF_BOOLEAN basic_exec_is_finished(se_exec_data_t*data) {
  EIF_BOOLEAN result = (EIF_BOOLEAN)0;
  int status;
  if (data->running) {
    int id = waitpid(data->id, &status, WNOHANG);
    if (id != 0) {
      /* child is finished */
      result = (EIF_BOOLEAN)(id == data->id);
      data->status = WEXITSTATUS(status);
      data->running = 0;
    }
  }
  else{
    result = (EIF_BOOLEAN)1;
  }
  return result;
}

void basic_exec_wait(se_exec_data_t*data) {
  int status;
  if (data->running) {
    int id = waitpid(data->id, &status, 0);
    data->status = WEXITSTATUS(status);
    data->running = 0;
  }
}

int* basic_exec_create_pipe(void) {
  int* result = (int*)calloc(2, sizeof(int));
  pipe(result);
  return result;
}

EIF_INTEGER basic_exec_get_character (EIF_INTEGER fd) {
  EIF_INTEGER result = -1;
  char buf[1];
  ssize_t r = read(fd, buf, 1);
  if (r <= 0) {
    result = -1;
  }
  else {
    result = 0xff & ((EIF_INTEGER)(buf[0]));
  }
  return result;
}

void basic_exec_put_character (EIF_INTEGER fd, EIF_CHARACTER c) {
  char buf[1];
  buf[0] = c;
  write(fd, buf, 1);
}

#endif
