/*
-- ------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2005: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
-- documentation files (the "Software"), to deal in the Software without restriction, including without
-- limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
-- the Software, and to permit persons to whom the Software is furnished to do so, subject to the following
-- conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or substantial
-- portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO
-- EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
-- AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
-- OR OTHER DEALINGS IN THE SOFTWARE.
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------
*/

#if defined __USE_POSIX || defined __unix__ || defined _POSIX_C_SOURCE

/*#  include <unistd.h> done in base.h */
/*#  include <sys/types.h> done in base.h */
#include <sys/wait.h>

typedef struct se_exec_data se_exec_data_t;
struct se_exec_data {
  pid_t id;
  int running;
  int status;
};

#define basic_exec_id(data) (((se_exec_data_t*)data)->id)
#define basic_exec_status(data) (((se_exec_data_t*)data)->status)
#define basic_exec_kill(data, sig) do{kill((data)->id, (sig))}while(0)
#define basic_exec_get_in_descriptor(pipe) (((int*)pipe)[0])
#define basic_exec_get_out_descriptor(pipe) (((int*)pipe)[1])
#define basic_exec_flush(fd) fsync(fd)

#elif defined WIN32

typedef struct se_exec_data se_exec_data_t;
struct se_exec_data {
  STARTUPINFO startupInformation;
  PROCESS_INFORMATION processInformation;
};

#define basic_exec_id(data) (((se_exec_data_t*)data)->processInformation.dwProcessId)
#define basic_exec_status(data) (((se_exec_data_t*)data)->status)
#define basic_exec_kill(data, sig) do{kill((data)->id, (sig))}while(0)
#define basic_exec_get_in_descriptor(pipe) (((int*)pipe)[0])
#define basic_exec_get_out_descriptor(pipe) (((int*)pipe)[1])
#define basic_exec_flush(fd) ((void)0)

#else

#error "Unknown OS. Cannot use exec. Please drop an email to SmartEiffel@loria.fr"

#endif

extern se_exec_data_t* basic_exec_execute(char*prog, char**args, int keep_env, int* in_fd, int* out_fd, int* err_fd);
extern EIF_BOOLEAN basic_exec_is_finished(se_exec_data_t*data);
extern void basic_exec_wait(se_exec_data_t*data);
extern int* basic_exec_create_pipe(void);
extern EIF_INTEGER basic_exec_get_character (EIF_INTEGER fd);
extern void basic_exec_put_character(EIF_INTEGER fd, EIF_CHARACTER c);
