-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class CECIL_ENTRY

inherit
	VISITABLE
insert
	GLOBALS

creation {CECIL_FILE}
	make

feature {ANY}
	c_name: STRING
			-- The external C name used to call the Eiffel code.

	type_mark: TYPE_MARK
			-- The one which is written in the -cecil file (i.e. its `start_position' is in the -cecil file).

	feature_name: FEATURE_NAME
			-- Of the feature to be called (also the one of the -cecil file).

	type: TYPE
			-- Alias of `type_mark.type'. Static `type' of the target of the feature to be called.

	feature_stamp: FEATURE_STAMP
			-- The corresponding one of `type' and `feature_name'.

	accept (visitor: CECIL_ENTRY_VISITOR) is
		do
			visitor.visit_cecil_entry(Current)
		end

feature {CECIL_FILE}
	get_started (cecil_file: CECIL_FILE) is
			-- Called only once to initialize `type' and `feature_stamp'.
		require
			cecil_file /= Void
			type = Void
			feature_stamp = Void
		do
			type := type_mark.type
			if not type.valid_feature_name(feature_name) then
				error_handler.add_position(type_mark.start_position)
				error_handler.add_position(feature_name.start_position)
				error_handler.append("Unknown feature name ")
				error_handler.append(feature_name.to_string)
				error_handler.append(" in type ")
				error_handler.append(type.name.to_string)
				error_handler.append(".")
				error_handler.print_as_error
				error_handler.append("Error while loading features of cecil path file %"")
				error_handler.append(cecil_file.path)
				error_handler.append("%".")
				error_handler.print_as_fatal_error
			end
			feature_stamp := type.lookup(feature_name)
			on_echo(cecil_file)
		ensure
			type /= Void
			feature_stamp /= Void
		end

	collect is
		local
			af: ANONYMOUS_FEATURE
		do
			smart_eiffel.collect_cecil_entry(type, feature_stamp)
			af := feature_stamp.anonymous_feature(type)
			assignment_handler.from_external(type, af.arguments, af.result_type)
		end

	run_feature: RUN_FEATURE is
		do
			Result := feature_stamp.run_feature_for(type)
		end

	on_echo (cecil_file: CECIL_FILE) is
		do
			echo.put_string(once "%T{")
			echo.put_string(type.name.to_string)
			echo.put_string(once "}.")
			echo.put_string(feature_name.to_string)
			echo.put_string(once " (from -cecil %"")
			echo.put_string(cecil_file.path)
			echo.put_string(once "%").%N")
		end

feature {}
	make (cn: like c_name; tm: like type_mark; fn: like feature_name) is
		require
			cn /= Void
			tm /= Void
			fn /= Void
		do
			c_name := cn
			type_mark := tm
			feature_name := fn
		ensure
			c_name = cn
			type_mark = tm
			feature_name = fn
		end

invariant
	c_name /= Void

	type_mark /= Void

	feature_name /= Void

end -- class CECIL_ENTRY
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
