class EXTRACT_INTERNALS

inherit
	CODE_PRINTER
	INTROSPECTION_HANDLER
		redefine
			make
		end
	CREATION_CLAUSE_LIST_VISITOR
	CREATION_CLAUSE_VISITOR

insert
	EXTERNAL_TOOL

creation {ANY}
	make

feature {ANY}
	make is
		local
			plugin_factory: FAKE_PLUGIN_FACTORY
		do
			bootstrap
			create plugin_factory.make
			system_tools.set_plugin_factory(plugin_factory)
			smart_eiffel.compile(Current)
		end

	parse_arguments is
		local
			argi: INTEGER; arg: STRING
		do
			search_for_verbose_flag
			if not ace_file_mode then
				from
					argi := 1
				until
					argi > argument_count
				loop
					arg := argument(argi)
					if is_help_flag(arg) then
						argi := argi + 1
					elseif is_verbose_flag(arg) then
						argi := argi + 1
					else
						if root_class_name = Void then
							root_class_name := arg
							argi := argi + 1
						elseif root_procedure_name = Void then
							root_procedure_name := arg
							argi := argi + 1
						else
							argi := argi + 1
						end
					end
				end
			end
		end

	is_valid_argument_for_ace_mode (arg: STRING): BOOLEAN is
		do
		end

	compile is
		local
			live_types: FAST_ARRAY[LIVE_TYPE]
			i: INTEGER
			live_type: LIVE_TYPE
			helper_name: STRING
			class_file_name, cecil_file_name, header_name: STRING
			class_stream, cecil_stream: TEXT_FILE_WRITE
			tm: TYPE_MARK
			buffer: STRING
		do
			helper_name := ace.root_class_name.to_string.twin
			helper_name.append("_INTERNALS_HELPER")
			class_file_name := helper_name.as_lower
			cecil_file_name := class_file_name.twin
			header_name := cecil_file_name.twin
			class_file_name.append(".e")
			cecil_file_name.append(".se")
			header_name.append(".ih")
			create cecil_stream.connect_to(cecil_file_name)
			cecil_stream.put_string(header_name)
			cecil_stream.put_new_line
			create class_stream.connect_to(class_file_name)
			class_stream.put_string("expanded class ")
			class_stream.put_string(helper_name)
			class_stream.put_string("{


insert
	INTERNALS_HANDLER

feature
	storable_quit is
		do
			storable_quit_with_code(exit_success_code)
		end

	storable_quit_with_code (code: INTEGER) is
		do
			die_with_code(code)

}")
			live_types := smart_eiffel.live_type_map
			from
				i := live_types.lower
			until
				i > live_types.upper
			loop
				live_type := live_types.item(i)
				tm := live_type.canonical_type_mark
				if introspection_handler.is_introspected(tm) then
					put_live_fields(cecil_stream, live_type)
					class_stream.put_string(once "%T%T%Tif (")
					if tm.is_native_array then
						class_stream.put_string(once "create {NATIVE_ARRAY_INTERNALS[")
						class_stream.put_string(live_type.type.generic_list.first.name.to_string)
						class_stream.put_string(once "]}.make_blank(0))")
					else
						buffer := once ""
						buffer.clear_count
						if not put_create_call(buffer, tm) then
							std_error.put_string(once "INTERNALS_HANDLER cannot create ")
							std_error.put_string(tm.written_mark)
							std_error.put_new_line
							class_stream.put_string(once "Void)")
						else
							class_stream.put_string(buffer)
							class_stream.put_string(once ").to_internals")
						end
					end
					class_stream.put_string(once "{
 = Void then
			end

}")
				end
				i := i + 1
			end
			class_stream.put_string("{
		end

end -- class 
}")
			class_stream.put_string(helper_name)
			class_stream.put_new_line
			class_stream.disconnect
			cecil_stream.disconnect
		end

	valid_argument_for_ace_mode: STRING is "Do not use arguments in ace mode"

	use_short_mode: BOOLEAN is False

	usage: STRING is "extract_internals root_class"

feature {}
	put_live_fields(cecil_stream: OUTPUT_STREAM; live_type: LIVE_TYPE) is
		local
			wa: ARRAY[RUN_FEATURE_2]
			i, lt_id: INTEGER
			rf: RUN_FEATURE_2
			name, lt_name: STRING
		do
			echo.put_string(live_type.name.to_string)
			echo.put_new_line
			wa := live_type.writable_attributes
			if wa /= Void then
				lt_id := live_type.id
				lt_name := live_type.name.to_string
				from
					i := wa.lower
				until
					i > wa.upper
				loop
					rf := wa.item(i)
					name := rf.name.to_string
					cecil_stream.put_string(once "ih")
					cecil_stream.put_integer(lt_id)
					cecil_stream.put_string(name)
					cecil_stream.put_character('%T')
					cecil_stream.put_string(lt_name)
					cecil_stream.put_character('%T')
					cecil_stream.put_string(name)
					cecil_stream.put_new_line
					i := i + 1
				end
			end
		end

	put_create_call (output: STRING; type_mark: TYPE_MARK): BOOLEAN is
		require
			output /= Void
			type_mark.is_static
		local
			written_call: STRING
			create_count, fail_count: INTEGER
			cp_list: FAST_ARRAY[WEIGHTED_CREATION_PROCEDURE]
			creation_procedure: WEIGHTED_CREATION_PROCEDURE
		do
			Result := True
			written_call := resolved_creation(type_mark)
			if written_call /= Void then
				output.append(written_call)
			elseif type_mark.is_kernel_expanded then
				put_kernel_creation(output, type_mark)
			elseif is_pending(type_mark) then
				Result := False
			else
				set_pending(type_mark)
				if type_mark.is_expanded then
					-- Only cache expanded creations
					written_call := ""
				else
					written_call := output
				end
				fail_count := written_call.count
				written_call.append(once "create {")
				written_call.append(type_mark.written_mark)
				written_call.append(once "}.")
				create_count := written_call.count
				from
					cp_list := creation_procedures(type_mark)
					Result := False
				until
					Result or else cp_list.is_empty
				loop
					creation_procedure := cp_list.last
					cp_list.remove_last
 					written_call.keep_head(create_count)
 					written_call.append(creation_procedure.creation_name)
 					Result := put_arguments(written_call, creation_procedure.creation_procedure, type_mark.type)
				end
				if Result and written_call /= output then
					output.append(written_call)
					resolved_creations.force(written_call, type_mark.id)
				elseif written_call = output and not Result then
					output.keep_head(fail_count)
				end
			end
		end

	put_arguments (output: STRING; creation_feature: E_ROUTINE; type: TYPE): BOOLEAN is
		require
			output /= Void
			creation_feature /= Void
			type /= Void
		local
			args: FORMAL_ARG_LIST
			j: INTEGER
			arg_type_mark: TYPE_MARK
		do
			args := creation_feature.arguments
			Result := True
			if args /= Void then
				output.extend('(')
				arg_type_mark := args.type_mark(1).to_static(type)
				Result := put_argument(output, arg_type_mark)
				from
					j := 2
				until
					j > args.count or else not Result
				loop
					output.append(once ", ")
					arg_type_mark := args.type_mark(j).to_static(type)
					Result := put_argument(output, arg_type_mark)
					j := j + 1
				end
				output.extend(')')
			end
		end

	put_argument (output: STRING; arg_type_mark: TYPE_MARK): BOOLEAN is
		do
			Result := arg_type_mark.is_reference
			if Result then
				output.append(once "Void")
			else
				Result := put_create_call(output, arg_type_mark)
			end
		end

	put_kernel_creation (output: STRING; tm: TYPE_MARK) is
		require
			tm.is_kernel_expanded
		do
			if tm.is_boolean then
				output.append(once "False")
			elseif tm.is_character then
				output.append(once "'%%U'")
			elseif tm.is_integer then
				output.extend('{')
				output.append(tm.written_mark)
				output.append(once " 0}")
			elseif tm.is_real then
				output.extend('{')
				output.append(tm.written_mark)
				output.append(once " 0.0}")
			elseif tm.is_pointer then
				output.append(once "default_pointer")
			else
				output.append(once "(create {")
				output.append(tm.written_mark)
				output.append(once "})")
			end
		end

	creation_procedures (type_mark: TYPE_MARK): FAST_ARRAY[WEIGHTED_CREATION_PROCEDURE] is
		local
			ccl: CREATION_CLAUSE_LIST
			list: FAST_ARRAY[CREATION_CLAUSE]
			type: TYPE
			cn_internals_handler: CLASS_NAME
			creation_clause: CREATION_CLAUSE
			i, j: INTEGER
			procedure_list: FEATURE_NAME_LIST
			creation_name: FEATURE_NAME
			creation_feature: E_ROUTINE
			creation_feature_weight: WEIGHTED_CREATION_PROCEDURE
		do
			create Result.with_capacity(4)
			ccl := type_mark.class_text.creation_clause_list
			if ccl /= Void then
				list := ccl.list
				type := type_mark.type
				create cn_internals_handler.unknown_position(string_aliaser.hashed_string(as_internals_handler))
				from
					i := list.upper
				until
					i < list.lower
				loop
					creation_clause := list.item(i)
					if creation_clause.clients.gives_permission_to(cn_internals_handler) then
						procedure_list := creation_clause.procedure_list
						if procedure_list /= Void then
							from
								j := procedure_list.count
							until
								j < 1
							loop
								creation_name := procedure_list.item(j)
								if creation_name.to_string /= as_manifest_creation then
									creation_feature ::= type.lookup(creation_name).anonymous_feature(type)
									create creation_feature_weight.make(creation_feature, creation_name.to_string, type)
									Result.add_last(creation_feature_weight)
								end
								j := j - 1
							end
						end
					end
					i := i - 1
				end
				(create {REVERSE_COLLECTION_SORTER[WEIGHTED_CREATION_PROCEDURE]}).sort(Result)
			end
		end

	resolved_creations: FAST_ARRAY[STRING] is
		once
			create Result.with_capacity(4096)
		end

	creation_pending: FAST_ARRAY[BOOLEAN] is
		once
			create Result.with_capacity(4096)
		end

	resolved_creation (type_mark: TYPE_MARK): STRING is
		require
			type_mark.is_static
		local
			type_id: INTEGER
		do
			type_id := type_mark.id
			if creation_pending.valid_index(type_id) then
				Result := resolved_creations.item(type_id)
			end
		end

	is_pending (type_mark: TYPE_MARK): BOOLEAN is
		require
			type_mark.is_static
		local
			type_id: INTEGER
		do
			type_id := type_mark.id
			if creation_pending.valid_index(type_id) then
				Result := creation_pending.item(type_id)
			end
		end

	set_pending (type_mark: TYPE_MARK) is
		require
			type_mark.is_static
		local
			type_id: INTEGER
		do
			type_id := type_mark.id
			creation_pending.force(True, type_id)
		end

feature {CREATION_CLAUSE_LIST}
	visit_creation_clause_list (visited: CREATION_CLAUSE_LIST) is
		do
		end

feature {CREATION_CLAUSE}
	visit_creation_clause (visited: CREATION_CLAUSE) is
		do
		end

end -- class EXTRACT_INTERNALS
