class SAME_MSG
--
-- Check whether two *.msg file are the same (remove version numbers, Unix path for files, blank, 
-- tabs as well as non-meaningful line feed...).
--

insert
	ARGUMENTS
	
creation
	make
   
feature {}
   msg_file_1, msg_file_2: TEXT_FILE_READ
   
   msg_file_1_position, msg_file_2_position: INTEGER
   
   make is
      local
			c1, c2: CHARACTER count: INTEGER
      do
			create msg_file_1.connect_to(argument(1))
			create msg_file_2.connect_to(argument(2))
			if not (msg_file_1.is_connected and msg_file_2.is_connected) then
				error_exit
			end
			from
				c1 := standard_forth(msg_file_1)
				c2 := standard_forth(msg_file_2)
			until
				False
			loop
				if msg_file_1.end_of_input then
					if msg_file_2.end_of_input then
						successful_exit
					else
						error_exit
					end
				elseif msg_file_2.end_of_input then
					error_exit
				elseif c1 /= c2 then
					error_exit
				else
					inspect c1
					when '(' then
						c1 := inside_bracket_forth(msg_file_1)
						c2 := inside_bracket_forth(msg_file_2)
					when '/' then
						c1 := unix_path_forth(msg_file_1)
						c2 := unix_path_forth(msg_file_2)
					else
						c1 := standard_forth(msg_file_1)
						c2 := standard_forth(msg_file_2)
					end
				end
				count := count + 1
			end
      end
	
   error_exit is
      do
			std_output.put_string("%Nsame_msg: ")
			std_output.put_string(argument(1))
			std_output.put_string(" at: ")
			std_output.put_integer(msg_file_1_position)
			std_output.put_string(" and ")
			std_output.put_string(argument(2))
			std_output.put_string(" at: ")
			std_output.put_integer(msg_file_2_position)
			std_output.put_string("%N%N%T")
			std_output.put_string("diff ")
			std_output.put_string(argument(2))
			std_output.put_string(" ")
			std_output.put_string(argument(1))
			std_output.put_string("%N")
			exit(exit_failure_code)
      end
	
   successful_exit is
      do
			std_output.put_character('.')
			exit(exit_success_code)
      end
	
   exit (code: INTEGER) is
      do
			msg_file_1.disconnect
			msg_file_2.disconnect
			die_with_code(code)
      end
	
   standard_forth (m: TEXT_FILE_READ): CHARACTER is
      require
			not m.end_of_input
      local
			stop: BOOLEAN
      do
			from
			until
				stop
			loop
				m.read_character
				if m = msg_file_1 then
					msg_file_1_position := msg_file_1_position + 1
				else
					msg_file_2_position := msg_file_2_position + 1
				end
				if m.end_of_input then
					stop := True
				else
					Result := m.last_character
					inspect
						Result
					when ' ', '%T', '%N', '%C' then
					else
						stop := True
					end
				end
			end
      end
	
   inside_bracket_forth (m: TEXT_FILE_READ): CHARACTER is
      local
			stop: BOOLEAN
      do
			from
			until
				stop
			loop
				if m.end_of_input then
					stop := True
				elseif Result = ')' then
						stop := True
				else
					Result := standard_forth(m)
				end
			end
      end
	
   unix_path_forth (m: TEXT_FILE_READ): CHARACTER is
      local
			stop: BOOLEAN
      do
			from
			until
				stop
			loop
				if m.end_of_input then
					stop := True
				else
					Result := standard_forth(m)
					inspect Result
					when '.',' ' then
						stop := True
					else
					end
				end
			end
      end
	
end
