-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class SE
	--
	-- The `se' command. This command is a front-end to all the SmartEiffel tools.
	--

inherit
	JOB

insert
	ARGUMENTS
		redefine
			command_name
		end
	GLOBALS
	COMMAND_LINE_TOOLS

creation {ANY}
	make

feature {ANY}
	command_name: STRING is "se"

feature {}
	stack: LOOP_STACK
	process: PROCESS

	plugins: FAST_ARRAY[STRING]
	plugin_commands: FAST_ARRAY[STRING]

	make is
		local
			plugin: STRING
		do
			if system_tools /= Void then
			end
			create stack.make
			create plugins.with_capacity(8)
			create plugin_commands.with_capacity(8)
			load_plugins
			if argument_count < 1 then
				system_tools.bad_use_exit(argument(0), command_line_help_summary)
			else
				set_verbose
				plugin := argument(1)
				if plugins.has(plugin) then
					call_plugin(plugin)
				elseif plugin_commands.has(plugin) then
					call_plugin(plugins.item(plugin_commands.index_of(plugin)))
				elseif is_version_flag_no_exit(plugin) then
					print_all_version_numbers
				elseif is_help_flag(plugin) then
				else
					system_tools.bad_use_exit(argument(0), command_line_help_summary)
				end
			end
		end

	load_plugins is
		local
			i: ITERATOR[STRING]
		do
			from
				i := ini_parser.section_key_iterator(fz_conf_se_tools)
			until
				i.is_off
			loop
				plugins.add_last(i.item)
				plugin_commands.add_last(ini_parser.section_item(fz_conf_se_tools, i.item))
				i.next
			end
		end

	call_plugin (plugin: STRING) is
		require
			plugins.has(plugin)
		local
			i, status: INTEGER; cmd: STRING; arg: FAST_ARRAY[STRING]
		do
			i := plugins.index_of(plugin)
			cmd := plugin_commands.item(i)
			create arg.with_capacity(argument_count - 1)
			from
				i := 2
			until
				i > argument_count
			loop
				arg.add_last(argument(i))
				i := i + 1
			end
			status := do_plugin_call(cmd, arg)
			if status /= 0 then
				die_with_code(status)
			end
		end

	do_plugin_call (plugin: STRING; arguments: FAST_ARRAY[STRING]): INTEGER is
			-- synchronous call. Waits for the plugin to terminate before going on.
		local
			i: INTEGER; cmd: STRING
		do
			--|*** ugly hack. The good algorithm is below, but does not seem to work properly
			cmd := once ""
			cmd.clear_count
			system_tools.command_path_in(cmd, plugin)
			from
				i := arguments.lower
			until
				i > arguments.upper
			loop
				cmd.extend(' ')
				cmd.append(arguments.item(i))
				i := i + 1
			end
			Result := echo.system_call(cmd)

			--|*** does not work properly! (some output gets lost :-(
			--cmd := once ""
			--cmd.clear_count
			--system_tools.command_path_in(cmd, plugin)
			--create process.execute(cmd, arguments, True)
			--stack.add_job(Current)
			--stack.run
			--if process.is_connected then
			--	Result := process.status
			--else
			--	Result := -1
			--end
		end

	command_line_help_summary: STRING is
		local
			i, n, c: INTEGER
		once
			Result := "[
						  Usage: se <command> <options>

						  se is the front-end to all SmartEiffel tools. It must be followed by a command
						  and the arguments of that command.

						  To get help on a particular command, type

						     se <command> -help

						  The available commands are:

						  ]"
			from
				i := plugins.lower
				n := 0
			until
				i > plugins.upper
			loop
				c := plugins.item(i).count + 2
				if n + c > 78 then
					Result.extend('%N')
					n := 0
				end
				Result.append(plugins.item(i))
				if i < plugins.upper then
					Result.append(once ", ")
				else
					Result.append(once "%N%N")
				end
				n := n + c
				i := i + 1
			end
		end

	print_all_version_numbers is
		local
			arg: FAST_ARRAY[STRING]
			i, status: INTEGER
			plugin_name: STRING
		do
			plugin_name := command_name
			std_output.put_character('%N')
			std_output.put_string(plugin_name)
			std_output.put_spaces(command_name_length - plugin_name.count + 1)
			std_output.put_string(smart_eiffel.release_number)
			arg := {FAST_ARRAY[STRING]<<"-version">>}
			-- We use an environment variable rather than a new argument 
			-- (e.g. -short_version) so that a recent "se" degrades 
			-- gracefully when used with an old plugin.
			(create {SYSTEM}).set_environment_variable("SMART_EIFFEL_SHORT_VERSION", "1")
			from
				i := plugins.lower
			until
				i > plugins.upper
			loop
				plugin_name := plugins.item(i)
				std_output.put_string(plugin_name)
				std_output.put_spaces(command_name_length - plugin_name.count + 1)
				std_output.flush
				status := do_plugin_call(plugin_commands.item(i), arg)
				-- Do something if status /= 0 ?
				i := i + 1
			end
		end

	is_valid_argument_for_ace_mode (arg: STRING): BOOLEAN is
		do
			check
				False
			end
		end

	valid_argument_for_ace_mode: STRING is
		do
			check
				False
			end
		end

	set_verbose is
		local
			i: INTEGER; ok: BOOLEAN
		do
			from
				i := 2
			until
				ok or else i > argument_count
			loop
				inspect argument(i)
				when "-verbose", "--verbose", "/verbose" then
					echo.set_verbose
				else
				end
				i := i + 1
			end
		end

	command_name_length: INTEGER is
		local
			i: INTEGER
		once
			from
				i := plugins.lower
			until
				i > plugins.upper
			loop
				Result := Result.max(plugins.item(i).count)
				i := i + 1
			end
		ensure
			Result >=0
		end

feature {LOOP_ITEM}
	out_ready: BOOLEAN
	err_ready: BOOLEAN

	prepare (ready: READY_DESCRIPTION) is
		do
			ready.when_data(process.output)
			ready.when_data(process.error)
		end

	is_ready (ready: READY_DESCRIPTION): BOOLEAN is
		do
			out_ready := ready.is_data(process.output)
			err_ready := ready.is_data(process.error)
			Result := out_ready or else err_ready or else done
		end

	continue is
		do
			if not done then
				if out_ready then
					from
						process.output.read_line
					until
						process.output.end_of_input
					loop
						std_output.put_string(process.output.last_string)
						std_output.put_new_line
						std_output.flush
						process.output.read_line
					end
				end
				if err_ready then
					from
						process.error.read_line
					until
						process.error.end_of_input
					loop
						std_error.put_string(process.error.last_string)
						std_error.put_new_line
						std_error.flush
						process.error.read_line
					end
				end
			end
			if done then
				stack.break
			end
		end

	done: BOOLEAN is
		do
			Result := not process.is_connected or else process.is_finished
		end

	restart is
		do
			--|*** cannot restore postcondition (won't restart)
		end

invariant
	plugins.count = plugin_commands.count

end -- class SE
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
