-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class LOCAL_NAME1
	--
	-- A local name in some declaration list.
	--

inherit
	LOCAL_ARGUMENT1

insert
	LOCAL_NAME

creation {TOKEN_BUFFER, INTROSPECTION_HANDLER}
	make

feature {ANY}
	collect (type: TYPE): TYPE is
		do
			Result := result_type.resolve_in(type) --|*** Don't know why, but we have to do this during collect:
			live_reference_counter_increment
			--|*** (Dom. april 12th 2004) ??? ***
		end

	adapt_for (type: TYPE): like Current is
		do
			--|*** (PH 25/08/04) should be identical with 
			--|*** ARGUMENT_NAME1.adapt_for, so put it in LOCAL_ARGUMENT1.
			usage_warning_check(type) --|*** (PH 29/05/04) should be done earlier
			Result := Current
		end

	accept (visitor: LOCAL_NAME1_VISITOR) is
		do
			visitor.visit_local_name1(Current)
		end

feature {LOCAL_VAR_LIST}
	produce_c (type: TYPE): BOOLEAN is
			-- True if some C code must be produced for this local variable
			-- (i.e. the local is really used or this local variable is
			-- of some user expanded type with possibles side effects).
		local
			static_tm: TYPE_MARK
		do
			if live_reference_counter > 0 then
				Result := True
			else
				static_tm := result_type.to_static(type)
				if static_tm.is_user_expanded then
					Result := True
					--|*** Not perfect: we should also check that the corresponding creation
					--     procedure actually has some side-effect.
					--|*** (Dom. april 12th 2004)
				end
			end
		end

feature {LOCAL_VAR_LIST}
	c_declare (type: TYPE; volatile_flag: BOOLEAN) is
			-- C declaration of the local.
		local
			static_tm: TYPE_MARK
		do
			if produce_c(type) then
				static_tm := result_type.to_static(type)
				if volatile_flag then
					if static_tm.is_kernel_expanded then
						cpp.put_string(once "volatile ")
					end
				end
				tmp_string.clear_count
				static_tm.c_type_for_result_in(tmp_string)
				tmp_string.extend(' ')
				cpp.put_string(tmp_string)
				cpp.print_local(to_string)
				cpp.put_character('=')
				static_tm.c_initialize
				cpp.put_string(fz_00)
			end
		end

feature {DECLARATION_LIST}
	name_clash_check (type: TYPE) is
		do
			name_clash_check_(type, once "Conflict between local/feature name (VRLE).")
		end

feature {LOCAL_NAME2}
	parsing_reference_counter_increment is
		do
			parsing_reference_counter := parsing_reference_counter + 1
		end

	live_reference_counter_increment is
		do
			live_reference_counter := live_reference_counter + 1
		end

feature {LOCAL_NAME1}
	usage_warning_check (type: TYPE) is
		do
			if parsing_reference_counter = 0 and then not produce_c(type) then
				error_handler.append("Unused local variable.")
				error_handler.add_position(start_position)
				error_handler.print_as_warning
				-- To get this warning only once:
				parsing_reference_counter := 1
			end
		end

feature {}
	parsing_reference_counter: INTEGER
			-- How many LOCAL_NAME2 encountered during parsing for this local declaration.

	live_reference_counter: INTEGER
			-- How many actual usage encountered in the living code ?

	make (sp: POSITION; n: STRING) is
		require
			not sp.is_unknown
			not n.is_empty
		do
			start_position := sp
			hashed_string := string_aliaser.hashed_string(n)
			to_string := hashed_string.to_string
		ensure
			start_position = sp
			to_string.is_equal(n) and string_aliaser.registered_one(to_string)
		end

end -- class LOCAL_NAME1
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
