-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class OLD_MANIFEST_ARRAY
	--
	-- The old (now obsolete) manifest array notation:
	--                                                    << foo, bar >>
	--
	-- is still accepted when the type of elements can be computed with no ambiguity. 
	-- Expression OLD_MANIFEST_ARRAY are internally replaced with  the canonical corresponding 
	-- canonical MANIFEST_GENERIC.
	--
inherit
	EXPRESSION

creation {EIFFEL_PARSER}
	make

feature
	start_position: POSITION
			-- Of first character of the first < character.

	list: FAST_ARRAY[EXPRESSION]
	
	is_void: BOOLEAN is False

	is_current: BOOLEAN is False

	is_implicit_current: BOOLEAN is False

	is_writable: BOOLEAN is False

	is_manifest_string: BOOLEAN is False

	is_result: BOOLEAN is False

	extra_bracket_flag: BOOLEAN is True

	precedence: INTEGER is 2

	is_static: BOOLEAN is False

	specialize_in (type: TYPE): like Current is
		local
			i: INTEGER; exp1, exp2: EXPRESSION; l: like list
		do
			from
				i := list.upper
			until
				i < list.lower or else exp1 /= exp2
			loop
				exp1 := list.item(i)
				exp2 := exp1.specialize_in(type)
				i := i - 1
			end
			if exp1 = exp2 then
				Result := Current
			else
				from
					l := list.twin
					Result := twin
					Result.set_list(l)
					l.put(exp2, i + 1)
				until
					i < list.lower
				loop
					l.put(list.item(i).specialize_in(type), i)
					i := i - 1
				end
			end
		end

	has_been_specialized: BOOLEAN is
		local
			i: INTEGER
		do
			Result := True
			from
				i := list.upper
			until
				i < list.lower or else not Result
			loop
				Result := list.item(i).has_been_specialized
				i := i - 1
			end
		end

	specialize_thru (parent_type: TYPE; inherit_item: INHERIT_TEXT_ITEM; new_type: TYPE): like Current is
		local
			i: INTEGER; exp1, exp2: EXPRESSION; l: like list
		do
			from
				i := list.upper
			until
				i < list.lower or else exp1 /= exp2
			loop
				exp1 := list.item(i)
				exp2 := exp1.specialize_thru(parent_type, inherit_item, new_type)
				i := i - 1
			end
			if exp1 = exp2 then
				Result := Current
			else
				from
					l := list.twin
					Result := twin
					Result.set_list(l)
					l.put(exp2, i + 1)
				until
					i < list.lower
				loop
					l.put(list.item(i).specialize_thru(parent_type, inherit_item, new_type), i)
					i := i - 1
				end
			end
		end

	specialize_2 (type: TYPE): MANIFEST_GENERIC is
			-- Transformation into the canonical form.
		local
			i: INTEGER; exp1, exp2: EXPRESSION; l: like list
			array_type_mark: ARRAY_TYPE_MARK; items_type_mark: TYPE_MARK
			optional_list: FAST_ARRAY[EXPRESSION]; unknown_position: POSITION
		do
			-- First applying `specialize_2' for all items of the `list':
			from
				i := list.upper
			until
				i < list.lower or else exp1 /= exp2
			loop
				exp1 := list.item(i)
				exp2 := exp1.specialize_2(type)
				i := i - 1
			end
			if exp1 = exp2 then
				l := list
			else
				from
					l := list.twin
					l.put(exp2, i + 1)
				until
					i < list.lower
				loop
					l.put(list.item(i).specialize_2(type), i)
					i := i - 1
				end
			end
			-- Now computing `items_type_mark':
			from
				type_set_buffer.clear_count
				i := l.upper
			until
				i < l.lower
			loop
				type_set_buffer.add(l.item(i).resolve_in(type))
				i := i - 1
			end
			check
				type_set_buffer.count > 0
			end
			if type_set_buffer.count = 1 then
				items_type_mark := type_set_buffer.item(1).canonical_type_mark
			else
				items_type_mark := smallest_ancestor_from_type_set_buffer
				if items_type_mark = Void then
					error_handler.add_position(start_position)
					error_handler.append("The old manifest ARRAY notation can only be used when the common %
												%type mark for all items of the manifest ARRAY exists, is not ambiguous %
												%and is easy to compute! By the way, it is not easy or possible to %
												%compute the most general type for the following set of types: {")
					from
						i := type_set_buffer.lower
					until
						i > type_set_buffer.upper
					loop
						error_handler.append(type_set_buffer.item(i).name.to_string)
						i := i + 1
						if i <= type_set_buffer.upper then
							error_handler.append(", ")
						end
					end
					error_handler.append("}. See the next error message.")
					error_handler.print_as_error
					error_handler.add_position(start_position)
					error_handler.append("This obsolete manifest ARRAY creation is no longer supported. %
												 %See our %"SmartEiffel/tutorial/manifest_notation.e%" %
                                     %in order to use the new notation.")
					error_handler.print_as_fatal_error
				end
			end
			check
				items_type_mark.is_static
			end
			-- Now building the canonical form:
			create array_type_mark.make(start_position, items_type_mark)
			create optional_list.with_capacity(1)
			optional_list.add_last(create {INTEGER_CONSTANT}.make(1, start_position))
			create Result.make(start_position, array_type_mark, optional_list, l, 0, unknown_position)
			Result.specialize_2_from_old_manifest_array(type)
		end

	use_current (type: TYPE): BOOLEAN is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	safety_check (type: TYPE) is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	resolve_in (type: TYPE): TYPE is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	declaration_type: TYPE is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	collect (type: TYPE): TYPE is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	adapt_for (type: TYPE): like Current is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end
	
	side_effect_free (type: TYPE): BOOLEAN is
		do
			Result := False
		end

	accept (visitor: OLD_MANIFEST_ARRAY_VISITOR) is
		do
			visitor.visit_old_manifest_array(Current)
		end

	non_void_no_dispatch_type (type: TYPE): TYPE is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	simplify (type: TYPE): like Current is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	compile_to_c (type: TYPE) is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	mapping_c_target (type, target_formal_type: TYPE) is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	mapping_c_arg (type: TYPE) is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	compile_target_to_jvm, compile_to_jvm (type: TYPE) is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	jvm_branch_if_false, jvm_branch_if_true (type: TYPE): INTEGER is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	jvm_assign_creation, jvm_assign (type: TYPE) is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	bracketed_pretty (indent_level: INTEGER) is
		do
			pretty_printer.put_character('(')
			pretty(indent_level)
			pretty_printer.put_character(')')
		end

	pretty (indent_level: INTEGER) is
		local
			i: INTEGER
		do
			pretty_printer.set_indent_level(indent_level)
			pretty_printer.put_string(once "<<")
			from
				i := list.lower
			until
				i > list.upper
			loop
				list.item(i).pretty(indent_level + 1)
				i := i + 1
				if i <= list.upper then
					pretty_printer.put_string(once ", ")
					if pretty_printer.column > pretty_printer.wrap_limit then
						pretty_printer.set_indent_level(indent_level + 1)
					end
				end
			end
			pretty_printer.put_string(once ">>")
		end

	pretty_target (indent_level: INTEGER) is
		do
			pretty_printer.put_character('(')
			pretty(indent_level)
			pretty_printer.put_character(')')
			pretty_printer.put_character('.')
		end

	short (type: TYPE) is
		do
			check
				False -- (Because we have already switched to the canonical form.)
			end
		end

	short_target (type: TYPE) is
		do
			bracketed_short(type)
			short_printer.put_dot
		end

feature {OLD_MANIFEST_ARRAY}
	set_list (l: like list) is
		do
			list := l
		end

feature {}
	make (sp: like start_position; l: like list) is
		require
			not sp.is_unknown
			l /= Void implies not l.is_empty
		do
			start_position := sp
			if l = Void then
				error_handler.add_position(sp)
				error_handler.append("Empty manifest array not allowed. (If you really need to do it, just replace it %
                                 %with something like:%Ncreate {ARRAY[ANY]}.make(1, 0)")
				error_handler.print_as_fatal_error
			end
			list := l
		ensure
			start_position = sp
			list = l
		end

	type_set_buffer: HASHED_SET[TYPE] is
		once
			create Result.make
		end

	smallest_ancestor_from_type_set_buffer: TYPE_MARK is
		require
			type_set_buffer.count >= 2
		local
			i: INTEGER; type, type_2: TYPE; expanded_flag: BOOLEAN
		do
			-- First loop to handle the simple situation where one TYPE of `type_set_buffer' can hold all 
			-- other types:
			from
				i := type_set_buffer.lower
			until
				i > type_set_buffer.upper
			loop
				type_2 := type_set_buffer.item(i)
				if type = Void then
					-- Let start with `type_2':
					type := type_2
				elseif type.can_be_assigned_to(type_2) then
					type := type_2
				elseif type_2.can_be_assigned_to(type) then
					-- We stick to `type'.
				else
					-- Not a simple situation.
					type := Void
					i := type_set_buffer.upper
				end
				i := i + 1
			end
			if type = Void then
				-- Not a simple situation, first looking for some expanded:
				from
					i := type_set_buffer.lower
				until
					expanded_flag or else i > type_set_buffer.upper
				loop
					expanded_flag := type_set_buffer.item(i).is_expanded
					i := i + 1
				end
				if not expanded_flag then
					-- Now trying to search for some TYPE with only one parent TYPE:
					from
						i := type_set_buffer.lower
					until
						type /= Void or else i > type_set_buffer.upper
					loop
						type := type_set_buffer.item(i).has_only_one_conformant_parent
						if type = Void then
							i := i + 1
						end
					end
					if type /= Void then
						type_set_buffer.remove(type_set_buffer.item(i))
						type_set_buffer.add(type)
						type := Void
						if type_set_buffer.count = 1 then
							type := type_set_buffer.item(1)
						else
							Result := smallest_ancestor_from_type_set_buffer
						end
					end
				end
			end
			if type /= Void then
				Result := type.canonical_type_mark
			end
		end

invariant

	list.count > 0
	
end -- class OLD_MANIFEST_ARRAY
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
