-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
deferred class ATTRIBUTE
	--
	-- For all sorts of attributes : constants (CST_ATT), unique
	--   (CST_ATT_UNIQUE) and instance variables (WRITABLE_ATTRIBUTE).
	--

inherit
	ANONYMOUS_FEATURE

feature {ANY}
	result_type: TYPE_MARK

	is_deferred: BOOLEAN is False

	is_attribute: BOOLEAN is True

	use_signature (other: ANONYMOUS_FEATURE) is
		do
			result_type := other.result_type
		end

	frozen obsolete_mark: MANIFEST_STRING is
			--|*** Now valid in ETL3
		do
		end

	frozen rescue_compound: INSTRUCTION is
		do
		end

	frozen set_rescue_compound (instruction: INSTRUCTION) is
		do
			check
				False
			end
		end

	frozen set_ensure_assertion (e_ensure: E_ENSURE) is
			--|*** Now valid in ETL3
		do
			check
				False
			end
		end

	frozen arguments: FORMAL_ARG_LIST is
		do
		end

feature {FEATURE_STAMP, PRECURSOR_CALL}
	specialize_2 (type: TYPE): like Current is
		local
			ra: like require_assertion; ea: like ensure_assertion
		do
			if ace.boost then
				Result := Current
			else
				if require_assertion /= Void then
					ra := require_assertion.specialize_2(type)
				end
				if ensure_assertion /= Void then
					ea := ensure_assertion.specialize_2(type)
				end
				if ra = require_assertion and then ea = ensure_assertion then
					Result := Current
				else
					Result := twin
					Result.set_require(ra)
					Result.set_ensure(ea)
				end
			end
		end

feature {FEATURE_STAMP, LIVE_TYPE, PRECURSOR_CALL}
	simplify (type: TYPE): ANONYMOUS_FEATURE is
		do
			Result := Current
		end

feature {}
	frozen try_to_undefine_aux (fn: FEATURE_NAME; bc: CLASS_TEXT): DEFERRED_ROUTINE is
		do
			error_handler.add_position(start_position)
			error_handler.add_position(fn.start_position)
			error_handler.append("An attribute cannot be undefined.")
			error_handler.print_as_error
			bc.fatal_undefine(fn)
		end

invariant
	no_arguments: arguments = Void

end -- class ATTRIBUTE
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
