-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class C_PLUGIN

inherit
	PLUGIN

creation {C_PLUGIN_FACTORY}
	make

feature {NATIVE_PLUG_IN}
	include (position: POSITION) is
		local
			ok: BOOLEAN; i: INTEGER; strings: FAST_ARRAY[STRING]
		do
			if not is_included then
				bd.compute_parent_directory_of(position.path)
				cwd.copy(bd.last_entry)
				strings := c_linker_options
				if strings /= Void then
					from
						i := strings.lower
					until
						i > strings.upper
					loop
						system_tools.append_c_linker_option(strings.item(i))
						i := i + 1
					end
				end
				strings := c_libraries
				if strings /= Void then
					from
						i := strings.lower
					until
						i > strings.upper
					loop
						system_tools.add_external_lib(strings.item(i))
						i := i + 1
					end
				end
				strings := c_library_paths
				if strings /= Void then
					from
						i := strings.lower
					until
						i > strings.upper
					loop
						system_tools.add_external_lib_path(absolute_path(strings.item(i)))
						i := i + 1
					end
				end
				strings := c_headers
				if strings /= Void then
					from
						i := strings.lower
					until
						i > strings.upper
					loop
						echo.tfr_connect(tfr, absolute_path(strings.item(i)))
						cpp.put_h_file(tfr)
						i := i + 1
					end
					ok := True
				end
				strings := c_sources
				if strings /= Void then
					from
						i := strings.lower
					until
						i > strings.upper
					loop
						echo.tfr_connect(tfr, absolute_path(strings.item(i)))
						cpp.put_c_file(tfr)
						i := i + 1
					end
					ok := True
				end
				if not ok then
					error_handler.append("No support found for this external %"plug-in%" (plugin: %"")
					error_handler.append(name)
					error_handler.append("%" at ")
					error_handler.append(path)
					error_handler.append("). No description file found.")
					error_handler.add_position(position)
					error_handler.print_as_fatal_error
				end
				is_included := True
			end
		end

feature {}
	is_included: BOOLEAN

	make (position: POSITION; a_name, a_path: STRING) is
		local
			plugin_path, b, c, entry, filepath: STRING
		do
			name := a_name
			path := a_path
			hash_code := a_name.hash_code.bit_xor(a_path.hash_code)
			c := once ""
			b := once ""
			entry := once ""
			plugin_path := once ""
			bd.compute_parent_directory_of(position.path)
			cwd.copy(bd.last_entry)
			--|*** Old plugin scheme (transitional)
			bd.compute_subdirectory_with(absolute_path(a_path), once "c")
			c.copy(bd.last_entry)
			b.copy(name)
			b.append(once ".h")
			bd.compute_file_path_with(c, b)
			b.copy(bd.last_entry)
			if (create {FILE_TOOLS}).is_readable(b) then
				add_c_header(b.twin)
			end
			b.copy(name)
			b.append(once ".c")
			bd.compute_file_path_with(c, b)
			b.copy(bd.last_entry)
			if (create {FILE_TOOLS}).is_readable(b) then
				add_c_source(b.twin)
			end
			-- New plugin scheme
			bd.compute_subdirectory_with(absolute_path(a_path), a_name)
			plugin_path.copy(bd.last_entry)
			bd.compute_subdirectory_with(plugin_path, once "c")
			b.copy(bd.last_entry)
			bd.connect_to(b)
			if bd.is_connected then
				from
					bd.read_entry
				until
					bd.end_of_input
				loop
					entry.copy(bd.last_entry)
					bd.compute_file_path_with(b, entry)
					filepath := bd.last_entry.twin
					inspect
						entry
					when "libs" then
						c_libraries := read(filepath)
					when "paths" then
						c_library_paths := read(filepath)
					when "linker_options" then
						c_linker_options := read(filepath)
					else
						if entry.has_suffix(once ".h") or else entry.has_suffix(once ".H") then
							add_c_header(filepath)
						elseif entry.has_suffix(once ".c") or else entry.has_suffix(once ".C") then
							add_c_source(filepath)
						end
					end
					bd.read_entry
				end
				bd.disconnect
			end
			-- sanity checks
			if c_sources = Void and then c_headers = Void then
				error_handler.append("The plugin ")
				error_handler.append(a_name)
				error_handler.append(" at ")
				error_handler.append(a_path)
				error_handler.append(" defined in ")
				error_handler.append(position.path)
				error_handler.append(" does not provide any .h or .c file!")
				error_handler.print_as_fatal_error
			end
		end

	add_c_source (a_c_source: STRING) is
		do
			if c_sources = Void then
				create c_sources.with_capacity(2)
			end
			echo.put_string(once "Including source ")
			echo.put_string(a_c_source)
			echo.put_new_line
			c_sources.add_last(a_c_source)
		end

	add_c_header (a_c_header: STRING) is
		do
			if c_headers = Void then
				create c_headers.with_capacity(2)
			end
			echo.put_string(once "Including header ")
			echo.put_string(a_c_header)
			echo.put_new_line
			c_headers.add_last(a_c_header)
		end

	c_sources: FAST_ARRAY[STRING]

	c_headers: FAST_ARRAY[STRING]

	c_libraries: FAST_ARRAY[STRING]

	c_library_paths: FAST_ARRAY[STRING]

	c_linker_options: FAST_ARRAY[STRING]

feature {}
	read (filename: STRING): FAST_ARRAY[STRING] is
		local
			b: STRING; strings: ITERATOR[STRING]
		do
			parser_buffer.load_file(filename)
			if parser_buffer.is_ready then
				config.a_inifile

				-- try for "system.flavor.compiler" entry
				b := once ""
				b.copy(system_tools.system_name)
				b.extend('.')
				b.append(system_tools.system_flavor)
				b.extend('.')
				b.append(system_tools.c_compiler)
				strings := config.section_key_iterator(b)

				-- nothing found, try "system.flavor" entry
				if strings = Void then
					b.copy(system_tools.system_name)
					b.extend('.')
					b.append(system_tools.system_flavor)
					strings := config.section_key_iterator(b)
				end

				-- nothing found, try "system.compiler" entry
				if strings = Void then
					b.copy(system_tools.system_name)
					b.extend('.')
					b.append(system_tools.c_compiler)
					strings := config.section_key_iterator(b)
				end

				-- nothing found, try "system" entry
				if strings = Void then
					b.copy(system_tools.system_name)
					strings := config.section_key_iterator(b)
				end
				if strings /= Void then
					from
						create Result.with_capacity(2)
					until
						strings.is_off
					loop
						Result.add_last(config.section_item(b, strings.item).twin)
						strings.next
					end
				end
				parser_buffer.release
			end
		end

	config: INI_PARSER is
		once
			create Result
		end

invariant
	c_headers /= Void implies not c_headers.is_empty

	c_sources /= Void implies not c_sources.is_empty

end -- class C_PLUGIN
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2005: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
