-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class C_PRETTY_PRINTER
	--
	-- Singleton in charge of handling C code pretty printing.
	-- This singleton is shared via the GLOBALS.`cpp' once function.
	--

inherit
	CODE_PRINTER

insert
	PLATFORM

feature {SMART_EIFFEL}
	compile is
		local
			gc_flag, scoop: BOOLEAN; i, run_count: INTEGER; lt: LIVE_TYPE
		do
			if nb_errors = 0 then
				check
					smart_eiffel.root_procedure /= Void
				end
				get_started
				check
					smart_eiffel.is_ready
				end
				swap_on_h
				scoop := smart_eiffel.scoop
				gc_flag := not gc_handler.is_off
				-- ---------------------------------------------------------
				if ace.boost then
					put_string_on_h(once "#define SE_BOOST 1%N")
				end
				if ace.sedb then
					put_string_on_h(once "#define SE_SEDB 1%N")
				end
				if gc_flag then
					put_string_on_h(once "#define SE_GC_LIB 1%N")
				end
				if exceptions_handler.used then
					put_string_on_h(once "#define SE_EXCEPTIONS 1%N")
				end
				if scoop then
					put_string_on_h(once "#define SE_SCOOP 1%N")
				end
				-- ---------------------------------------------------------
				if scoop then
					sys_runtime_h_and_c(once "scoop")
					sys_runtime_h_and_c(once "scoop_thread")
					system_tools.add_lib_scoop
				end
				-- ---------------------------------------------------------
				if ace.profile then
					sys_runtime_h_and_c(once "profile")
				end
				-- ---------------------------------------------------------
				from
					put_comment_line(once "C Header Pass 1 :")
					agent_pool.c_header_pass1
					i := live_type_map.upper
				until
					i < 0
				loop
					lt := live_type_map.item(i)
					if lt.at_run_time then
						run_count := run_count + 1
					end
					lt.c_header_pass1
					i := i - 1
				end
				-- ---------------------------------------------------------
				from
					put_comment_line(once "C Header Pass 2 :")
					i := live_type_map.upper
				until
					i < 0
				loop
					lt := live_type_map.item(i)
					lt.c_header_pass2
					i := i - 1
				end
				-- ---------------------------------------------------------
				from
					put_comment_line(once "C Header Pass 3 :")
					i := live_type_map.upper
				until
					i < 0
				loop
					lt := live_type_map.item(i)
					lt.c_header_pass3
					i := i - 1
				end
				-- ---------------------------------------------------------
				from
					put_comment_line(once "C Header Pass 4 :")
					i := live_type_map.upper
				until
					i < 0
				loop
					lt := live_type_map.item(i)
					lt.c_header_pass4
					i := i - 1
				end
				if not smart_eiffel.is_at_run_time(as_native_array_character) then
					-- Force definition of T9 and T7:
					put_string(once "typedef T3* T9;%N")
					if ace.no_check then
						put_c_function(once "void se_prinT9(FILE* file, T9*o)", once "fprintf(file, %"NATIVE_ARRAY[STRING]#%%p\n%",(void*)*o);")
					end
				end
				manifest_string_pool.c_define1(smart_eiffel.is_at_run_time(as_string))
				customize_runtime
				-- ---------------------------------------------------------
				if gc_flag then
					gc_handler.define1
				end
				-- ---------------------------------------------------------
				compile_routines
			   cecil_define
				agent_pool.customize_c_runtime_2
				-- ---------------------------------------------------------
				if ace.sedb then
					prepare_introspection
				end
				-- ---------------------------------------------------------
				split_c_file_padding_here
				manifest_string_pool.c_define2(smart_eiffel.is_at_run_time(as_string))
				address_of_pool.c_define
				if gc_flag then
					gc_handler.define2
				end
				define_main(smart_eiffel.root_procedure)
				split_c_file_padding_here
				manifest_generic_pool.c_define
				split_c_file_padding_here
				switch_collection.c_define
				split_c_file_padding_here
				if scoop then
					separate_tools.generate_scoop_functions(live_type_map)
				end
            assignment_test_pool.c_define_assignment_test_functions(live_type_map)
				debug
					echo.put_string(once "Very Final magic_count : ")
					echo.put_integer(smart_eiffel.magic_count)
					echo.put_character('%N')
				end
				write_make_file
				echo_information
			else
				error_handler.append("Cannot produce C code.")
				error_handler.print_as_error
			end
		end

feature {}
	live_type_map: FAST_ARRAY[LIVE_TYPE] is
		do
			Result := smart_eiffel.live_type_map
		end

	agent_switches: FAST_ARRAY[TYPE] is
		do
			Result := smart_eiffel.agent_switches
		end

	agent_creations: FAST_ARRAY[AGENT_CREATION] is
		do
			Result := smart_eiffel.agent_creations
		end

	class_invariants: FAST_ARRAY[LIVE_TYPE] is
		do
			Result := smart_eiffel.class_invariants
		end

	run_features: FAST_ARRAY[RUN_FEATURE] is
		do
			Result := smart_eiffel.run_features
		end

	class_text_dictionary: DICTIONARY[CLASS_TEXT, HASHED_STRING] is
		do
			Result := smart_eiffel.class_text_dictionary
		end

	prepare_introspection is
			-- Creates the introspection features
		require
			ace.no_check
		local
			i: INTEGER
		do
			swap_on_c
			from
				i := live_type_map.upper
			until
				i < 0
			loop
				live_type_map.item(i).prepare_introspection
				i := i - 1
			end
		end

	compile_routines is
			-- Try to give the best order to the C output.
		require
			smart_eiffel.root_procedure.set_is_root
		local
			lt, lt_string: LIVE_TYPE; ct: TYPE_MARK; deep, i: INTEGER; stop: BOOLEAN; ctn: STRING
			cn, current_class_name: CLASS_NAME
		do
			if ace.no_check then
				echo.put_string(once "Compiling routines for ")
				echo.put_integer(live_type_map.count)
				echo.put_string(once " run classes :%N")
				swap_on_c
				from
					i := live_type_map.upper
				until
					i < 0
				loop
					lt := live_type_map.item(i)
					cn := lt.class_text_name
					if cn /= current_class_name then
						current_class_name := cn
					end
					lt.compile_to_c(0)
					i := i - 1
				end
			else
				echo.put_string(once "Compiling/Sorting ")
				echo.put_integer(live_type_map.count)
				echo.put_string(once " live TYPEs:%N")
				swap_on_c
				from
					i := live_type_map.upper
				until
					i < 0
				loop
					lt := live_type_map.item(i)
					ct := lt.canonical_type_mark
					if ct.is_kernel_expanded then
						lt.compile_to_c(0)
					elseif ct.is_string then
						lt_string := lt
					end
					i := i - 1
				end
				from
					i := live_type_map.upper
				until
					i < 0
				loop
					lt := live_type_map.item(i)
					i := i - 1
				end
				from
					i := live_type_map.upper
				until
					i < 0
				loop
					lt := live_type_map.item(i)
					ctn := lt.class_text_name.to_string
					if as_native_array = ctn then
						lt.compile_to_c(0)
					end
					i := i - 1
				end
				if lt_string /= Void then
					if lt_string.at_run_time then
						lt_string.compile_to_c(0)
					end
				end
				from
					i := live_type_map.upper
				until
					i < 0
				loop
					lt := live_type_map.item(i)
					ct := lt.canonical_type_mark
					ctn := ct.class_text_name.to_string
					if as_array = ctn or else as_fixed_array = ctn then
						lt.compile_to_c(0)
					end
					i := i - 1
				end
				from
					i := live_type_map.upper
				until
					i < 0
				loop
					lt := live_type_map.item(i)
					if lt.is_generic then
						lt.compile_to_c(0)
					end
					i := i - 1
				end
				from
					-- General sorting:
					deep := 6
				until
					stop
				loop
					from
						stop := True
						i := live_type_map.upper
					until
						i < 0
					loop
						lt := live_type_map.item(i)
						if not lt.compile_to_c_done then
							lt.compile_to_c(deep)
							if not lt.compile_to_c_done then
								stop := False
							end
						end
						i := i - 1
					end
					deep := deep - 1
					check
						deep >= -1
					end
				end
			end
		end

	show_profile is
		require
			ace.profile
			on_c
		local
			n, i, s, a, sa: INTEGER
		do
			n := run_features.count
			if class_invariants /= Void then
				i := class_invariants.count
			end
			s := switch_collection.count
			if agent_creations /= Void then
				a := agent_creations.count
			end
			if agent_switches /= Void then
				sa := agent_switches.count
			end
			put_string(once "{%N%
                             %FILE *profile_file = fopen(%"profile.se%", %"w%");%N%
                             %if (profile_file!=NULL) {%N%
                             %int i;%N%
                             %se_profile_t sorted_profile[")
			put_integer(n)
			if i > 0 then
				put_string(once "];%Nse_profile_t sorted_inv_profile[")
				put_integer(i)
			end
			if s > 0 then
				put_string(once "];%Nse_profile_t sorted_switch_profile[")
				put_integer(s)
			end
			if a > 0 then
				put_string(once "];%Nse_profile_t sorted_agent_profile[")
				put_integer(a)
			end
			if sa > 0 then
				put_string(once "];%Nse_profile_t sorted_agent_switch_profile[")
				put_integer(sa)
			end
			put_string(once "];%Nse_profile_t sorted_all_profile[")
			put_integer(n+i+s+a+sa+3)

			-- sorting profiles

			put_string(once "];%Nmemcpy(sorted_profile, profile, ")
			put_integer(n)
			put_string(once "*sizeof(se_profile_t));%Nqsort(sorted_profile, ")
			put_integer(n)
			put_string(once ", sizeof(se_profile_t), profile_comparator);%N")
			put_string(once "memcpy(sorted_all_profile, profile, ")
			put_integer(n)
			put_string(once "*sizeof(se_profile_t));%N")
			if i > 0 then
				put_string(once "memcpy(sorted_inv_profile, inv_profile, ")
				put_integer(i)
				put_string(once "*sizeof(se_profile_t));%Nqsort(sorted_inv_profile, ")
				put_integer(i)
				put_string(once ", sizeof(se_profile_t), profile_comparator);%N")
				put_string(once "memcpy(sorted_all_profile+")
				put_integer(n)
				put_string(once ", inv_profile, ")
				put_integer(i)
				put_string(once "*sizeof(se_profile_t));%N")
			end
			if s > 0 then
				put_string(once "memcpy(sorted_switch_profile, switch_profile, ")
				put_integer(s)
				put_string(once "*sizeof(se_profile_t));%Nqsort(sorted_switch_profile, ")
				put_integer(s)
				put_string(once ", sizeof(se_profile_t), profile_comparator);%N")
				put_string(once "memcpy(sorted_all_profile+")
				put_integer(n+i)
				put_string(once ", switch_profile, ")
				put_integer(s)
				put_string(once "*sizeof(se_profile_t));%N")
			end
			if a > 0 then
				put_string(once "memcpy(sorted_agent_profile, agent_profile, ")
				put_integer(a)
				put_string(once "*sizeof(se_profile_t));%Nqsort(sorted_agent_profile, ")
				put_integer(a)
				put_string(once ", sizeof(se_profile_t), profile_comparator);%N")
				put_string(once "memcpy(sorted_all_profile+")
				put_integer(n+i+s)
				put_string(once ", agent_profile, ")
				put_integer(a)
				put_string(once "*sizeof(se_profile_t));%N")
			end
			if sa > 0 then
				put_string(once "memcpy(sorted_agent_switch_profile, agent_switch_profile, ")
				put_integer(sa)
				put_string(once "*sizeof(se_profile_t));%Nqsort(sorted_agent_switch_profile, ")
				put_integer(sa)
				put_string(once ", sizeof(se_profile_t), profile_comparator);%N")
				put_string(once "memcpy(sorted_all_profile+")
				put_integer(n+i+s+a)
				put_string(once ", agent_switch_profile, ")
				put_integer(sa)
				put_string(once "*sizeof(se_profile_t));%N")
			end
			put_string(once "sorted_all_profile[")
			put_integer(n+i+s+a+sa)
			put_string(once "]=root_profile;%N")
			put_string(once "sorted_all_profile[")
			put_integer(n+i+s+a+sa+1)
			put_string(once "]=runinit_profile;%N")
			put_string(once "sorted_all_profile[")
			put_integer(n+i+s+a+sa+2)
			put_string(once "]=atexit_profile;%N")
			put_string(once "qsort(sorted_all_profile, ")
			put_integer(n+i+s+a+sa+3)
			put_string(once ", sizeof(se_profile_t), profile_comparator);%N")

			-- summary

			put_string(once "fprintf(profile_file, %"===============================================================================\nSummary:\n%");%N")
			put_string(once "start_sumup_profile(profile_file);%Ni=")
			put_integer(n+i+s+a+sa+3)
			put_string(once ";while (i-->0) {%Nsumup_profile(profile_file, sorted_all_profile+i);%N}%N")

			-- detailed printing

			put_string(once "fprintf(profile_file, %"\n===============================================================================\nDetailed profile:\n%");%N")
			put_string(once "start_print_profile(profile_file);%N")
			put_string(once "print_profile(profile_file, &root_profile);%N")
			put_string(once "print_profile(profile_file, &runinit_profile);%N")
			put_string(once "print_profile(profile_file, &atexit_profile);%N")
			put_string(once "fprintf(profile_file, %"\n-------------------------------------------------------------------------------\n%");%Ni=")
			put_integer(n)
			put_string(once ";while (i-->0) {%Nprint_profile(profile_file, sorted_profile+i);%N}%N")
			if i > 0 then
				put_string(once "fprintf(profile_file, %"\n-------------------------------------------------------------------------------\n%");%Ni=")
				put_integer(i)
				put_string(once ";while (i-->0) {%Nprint_profile(profile_file, sorted_inv_profile+i);%N}%N")
			end
			if s > 0 then
				put_string(once "fprintf(profile_file, %"\n-------------------------------------------------------------------------------\n%");%Ni=")
				put_integer(s)
				put_string(once ";while (i-->0) {%Nprint_profile(profile_file, sorted_switch_profile+i);%N}%N")
			end
			if a > 0 then
				put_string(once "fprintf(profile_file, %"\n-------------------------------------------------------------------------------\n%");%Ni=")
				put_integer(a)
				put_string(once ";while (i-->0) {%Nprint_profile(profile_file, sorted_agent_profile+i);%N}%N")
			end
			if sa > 0 then
				put_string(once "fprintf(profile_file, %"\n-------------------------------------------------------------------------------\n%");%Ni=")
				put_integer(sa)
				put_string(once ";while (i-->0) {%Nprint_profile(profile_file, sorted_agent_switch_profile+i);%N}%N")
			end

			put_string(once "fprintf(profile_file, %"\n===============================================================================\n%");%N")
			put_string(once "fclose(profile_file);}%N")
			put_string(once "else fprintf(SE_ERR, %"Cannot open profile.se for writing.\n%");%N}%N")
		end

	define_extern_tables is
		require
			on_c
		local
			size: INTEGER
		do
			size := id_provider.max_id + 1
			macro_def(once "SE_MAXID", size)
			if smart_eiffel.generator_used then
				put_extern4(once "T7*g", size)
			end
			if smart_eiffel.generating_type_used then
				put_extern4(once "T7*t", size)
			end
			if ace.boost then
				if not gc_handler.is_off and then smart_eiffel.class_used(as_weak_reference) /= Void then
					initialize_size_table
				end
			else
				put_extern4(once "char*p", size)
				c_code.copy(once "void(*se_prinT[")
				size.append_in(c_code)
				c_code.append(once "])(FILE*,void**)")
				put_extern1(c_code)
				initialize_size_table
				if ace.sedb then
					c_code.copy(once "void*(*se_introspecT[")
					size.append_in(c_code)
					c_code.append(once "])(void*,char*,int*,int*)")
					put_extern1(c_code)
					c_code.copy(once "char* se_atT[")
					size.append_in(c_code)
					c_code.append(once "]")
					put_extern1(c_code)
				end
			end
			if ace.profile then
				put_extern4(once "se_profile_t profile", run_features.count)
				if switch_collection.count > 0 then
					put_extern4(once "se_profile_t switch_profile", switch_collection.count)
				end
				if agent_creations /= Void then
					put_extern4(once "se_profile_t agent_profile", agent_creations.count)
				end
				if agent_switches /= Void then
					put_extern4(once "se_profile_t agent_switch_profile", agent_switches.count)
				end
				if class_invariants /= Void then
					put_extern4(once "se_profile_t inv_profile", class_invariants.count)
				end
				put_extern1(once "se_profile_t root_profile")
				put_extern1(once "se_profile_t runinit_profile")
				put_extern1(once "se_profile_t atexit_profile")
				put_extern1(once "se_local_profile_t global_profile")
			end
		end

	initialize_size_table is
			-- initialize se_strucT
		require
			on_c
			not smart_eiffel.status.is_analyzing
		local
			i_ltm, i_c, nb_types: INTEGER; a_type: LIVE_TYPE; collection_sorter: COLLECTION_SORTER[LIVE_TYPE]
		do
			nb_types := id_provider.max_id + 1
			put_extern3(once "int se_strucT", nb_types)
			collection_sorter.sort(live_type_map)
			from
				i_ltm := 0
				i_c := 0
				c_code.clear_count
			until
				i_ltm > live_type_map.upper
			loop
				a_type := live_type_map.item(i_ltm)
				from
				until
					i_c >= a_type.id
				loop
					if c_code.count >= 100 then
						c_code.extend('%N')
						put_string(c_code)
						c_code.clear_count
					end
					c_code.append(once "0,")
					i_c := i_c + 1
				end
				check
					a_type.id = i_c
				end
				if c_code.count >= 100 then
					c_code.extend('%N')
					put_string(c_code)
					c_code.clear_count
				end
				if a_type.at_run_time then
					c_code.append(once "sizeof(T")
					i_c.append_in(c_code)
					c_code.append(once "),")
				else
					c_code.append(once "0,")
				end
				i_ltm := i_ltm + 1
				i_c := i_c + 1
			end
			-- last items are automatically zeroed by C compiler
			check
				c_code.last = ','
			end
			c_code.remove_the_last
			c_code.append(once "};%N")
			put_string(c_code)
		ensure
			on_c
		end

	initialize_path_table is
		require
			ace.no_check
			on_c
		local
			i, id: INTEGER; ct: CLASS_TEXT; lt: LIVE_TYPE
		do
			from
				put_string(once "p[0]=%"???%";%N")
				i := 1
			until
				i > class_text_dictionary.count
			loop
				ct := class_text_dictionary.item(i)
				id := ct.id
				if id > 0 then
					put_string(once "p[")
					put_integer(id)
					put_string(once "]=")
					put_string_c(ct.path)
					put_string(fz_00)
				end
				i := i + 1
			end
			from
				i := live_type_map.upper
			until
				i < 0
			loop
				lt := live_type_map.item(i)
				if lt.at_run_time then
					put_string(once "se_prinT[")
					put_integer(lt.id)
					put_string(once "]=((void(*)(FILE*,void**))se_prinT")
					put_integer(lt.id)
					put_string(once ");%N")
					if lt.is_generic then
						put_string(once "p[")
						put_integer(lt.id)
						put_string(once "]=p[")
						put_integer(lt.class_text.id)
						put_string(once "];%N")
					end
					if ace.sedb then
						put_string(once "se_introspecT[")
						put_integer(lt.id)
						put_string(once "]=((void*(*)(void*,char*,int*,int*))se_introspecT")
						put_integer(lt.id)
						put_string(once ");%N")
						lt.prepare_introspection2
					end
				end
				i := i - 1
			end
		ensure
			on_c
		end

	initialize_generator is
		require
			on_c
		local
			i, id: INTEGER; ct: CLASS_TEXT; lt: LIVE_TYPE; n: STRING
		do
			from
				i := 1
			until
				i > class_text_dictionary.count
			loop
				ct := class_text_dictionary.item(i)
				id := ct.id
				if id >= 0 then
					put_array1('g', id)
					put_character('=')
					n := ct.name.to_string
					if n = as_integer then
						n := as_integer_32
					end
					put_se_string(n)
					put_string(fz_00)
				end
				i := i + 1
			end
			from
				i := live_type_map.upper
			until
				i < 0
			loop
				lt := live_type_map.item(i)
				if lt.generator_used then
					ct := lt.class_text
					if ct.name.hashed_name /= lt.name then
						put_array1('g', lt.id)
						put_character('=')
						put_array1('g', ct.id)
						put_string(fz_00)
					end
				end
				i := i - 1
			end
		ensure
			on_c
		end

	initialize_generating_type is
		require
			on_c
		local
			i: INTEGER; lt: LIVE_TYPE; ct: CLASS_TEXT; rtm: STRING
		do
			from
				i := live_type_map.upper
			until
				i < 0
			loop
				lt := live_type_map.item(i)
				if lt.generating_type_used then
					put_array1('t', lt.id)
					put_character('=')
					ct := lt.class_text
					rtm := lt.name.to_string
					if ct.name.to_string /= rtm then
						put_se_string(rtm)
					else
						put_array1('g', lt.id)
					end
					put_string(fz_00)
				end
				i := i - 1
			end
		ensure
			on_c
		end

	initialize_profile is
		require
			ace.profile
			on_c
		local
			i: INTEGER; b: STRING; rf: RUN_FEATURE; lt: LIVE_TYPE; t: TYPE; ac: AGENT_CREATION
		do
			from
				b := once "......................................."
				i := run_features.upper
			until
				i < run_features.lower
			loop
				rf := run_features.item(i)
				put_string(once "init_profile(profile+")
				put_integer(i)
				put_character(',')
				b.copy(rf.type_of_current.name.to_string)
				b.extend('.')
				rf.name.complete_name_in(b)
				put_string_c(b)
				put_string(fz_14)
				i := i - 1
			end
			if class_invariants /= Void then
				from
					b := once ""
					i := class_invariants.upper
				until
					i < class_invariants.lower
				loop
					lt := class_invariants.item(i)
					put_string(once "init_profile(inv_profile+")
					put_integer(i)
					put_character(',')
					b.copy(once "invariant of ")
					b.append(lt.name.to_string)
					put_string_c(b)
					put_string(fz_14)
					i := i - 1
				end
			end
			if agent_creations /= Void then
				from
					b := once ""
					i := agent_creations.upper
				until
					i < agent_creations.lower
				loop
					ac := agent_creations.item(i)
					put_string(once "init_profile(agent_profile+")
					put_integer(i)
					put_character(',')
					b.clear_count
					pretty_printer.expression_in(b, ac)
					put_string_c(b)
					put_string(fz_14)
					i := i - 1
				end
			end
			if agent_switches /= Void then
				from
					b := once ""
					i := agent_switches.upper
				until
					i < agent_switches.lower
				loop
					t := agent_switches.item(i)
					put_string(once "init_profile(agent_switch_profile+")
					put_integer(i)
					put_character(',')
					b.copy(once "agents ")
					b.append(t.canonical_type_mark.written_mark)
					put_string_c(b)
					put_string(fz_14)
					i := i - 1
				end
			end
			switch_collection.initialize_switch_site_profiles
		end

	c_code: STRING is
		once
			create Result.make(128)
		end

feature {SMART_EIFFEL, LIVE_TYPE, MANIFEST_STRING_POOL, SWITCH_COLLECTION, GC_HANDLER, MANIFEST_GENERIC_POOL}
	split_c_file_padding_here is
			-- Must be called only if we are sure that we will output something after that call.
		do
			if ace.no_split then
				-- Single C file mode.
			else
				if out_c.character_count > 150_000 then
					split_c_file_now
				end
			end
		end

feature {NO_INVARIANT_WRAPPER}
	invariant_checks_disabled: BOOLEAN

	set_invariant_checks_disabled (i: like invariant_checks_disabled) is
		do
			invariant_checks_disabled := i
		ensure
			invariant_checks_disabled = i
		end

feature {LIVE_TYPE}
	split_c_file_now is
		require
			multi_c_file_mode: not ace.no_split
		do
			split_count := split_count + 1
			end_c_linkage(out_c)
			out_c.disconnect
			path_c_in(path_c, split_count)
			backup_tfw_connect(path_c)
			begin_c_linkage(out_c)
			add_first_include(path_h)
			if current_out /= out_h then
				current_out := out_c
			end
		end

feature {ANY}
	on_h: BOOLEAN is
		require
			smart_eiffel.is_ready
		do
			Result := current_out = out_h
		end

	on_c: BOOLEAN is
		require
			smart_eiffel.is_ready
		do
			Result := current_out = out_c
		end

	swap_on_c is
		do
			current_out := out_c
		ensure
			on_c
		end

	swap_on_h is
		do
			current_out := out_h
		ensure
			on_h
		end

	put_string_on_h (string: STRING) is
		do
			out_h.put_string(string)
		end

	put_extern1 (type_variable: STRING) is
		do
			out_h.put_string(fz_extern)
			out_h.put_string(type_variable)
			out_h.put_string(fz_00)
			--
			out_c.put_string(type_variable)
			out_c.put_string(fz_00)
		end

	put_extern2 (type_variable: STRING; initializer: CHARACTER) is
		do
			out_h.put_string(fz_extern)
			out_h.put_string(type_variable)
			out_h.put_string(fz_00)
			--
			out_c.put_string(type_variable)
			out_c.put_character('=')
			out_c.put_character(initializer)
			out_c.put_string(fz_00)
		end

	put_extern3 (type_variable: STRING; array_size: INTEGER) is
		do
			out_h.put_string(fz_extern)
			out_h.put_string(type_variable)
			out_h.put_string(once "[];%N")
			--
			out_c.put_string(type_variable)
			out_c.put_character('[')
			out_c.put_integer(array_size)
			out_c.put_string(once "]={")
		end

	put_extern4 (type_variable: STRING; array_size: INTEGER) is
		do
			out_h.put_string(fz_extern)
			out_h.put_string(type_variable)
			out_h.put_string(once "[];%N")
			--
			out_c.put_string(type_variable)
			out_c.put_character('[')
			out_c.put_integer(array_size)
			out_c.put_string(once "];%N")
		end

	put_extern5 (type_variable: STRING; initializer: STRING) is
		do
			out_h.put_string(fz_extern)
			out_h.put_string(type_variable)
			out_h.put_string(fz_00)
			--
			out_c.put_string(type_variable)
			out_c.put_character('=')
			out_c.put_string(initializer)
			out_c.put_string(fz_00)
		end

	put_extern6 (type_variable: STRING; initializer: INTEGER) is
		do
			out_h.put_string(fz_extern)
			out_h.put_string(type_variable)
			out_h.put_string(fz_00)
			--
			out_c.put_string(type_variable)
			out_c.put_character('=')
			out_c.put_integer(initializer)
			out_c.put_string(fz_00)
		end

	put_extern7 (type_variable: STRING) is
		do
			out_h.put_string(fz_extern)
			out_h.put_string(type_variable)
			out_h.put_string(fz_00)
			out_c.put_string(type_variable)
			out_c.put_character('=')
		end

	put_c_heading (heading: STRING) is
		do
			out_h.put_string(heading)
			out_h.put_string(once ";%N")
			out_c.put_string(heading)
			out_c.put_string(once "{%N")
		end

	put_c_function (heading, body: STRING) is
		require
			not heading.is_empty
			body /= Void
		do
			put_c_heading(heading)
			out_c.put_string(body)
			out_c.put_string(once "}%N")
		end

	put_string (c: STRING) is
		require
			smart_eiffel.is_ready
		do
			current_out.put_string(c)
			debug
				current_out.flush
			end
		end

	put_string_c (s: STRING) is
		require
			on_c
		do
			tmp_string.clear_count
			manifest_string_pool.string_to_c_code(s, tmp_string)
			out_c.put_string(tmp_string)
		end

	put_character (c: CHARACTER) is
		require
			smart_eiffel.is_ready
		do
			current_out.put_character(c)
			debug
				current_out.flush
			end
		end

	put_integer (i: INTEGER_32) is
		require
			smart_eiffel.is_ready
		   only_common_c_values: i > Minimum_integer_32
		do
			current_out.put_integer(i)
			debug
				current_out.flush
			end
		end

	put_real (r: REAL) is
		require
			smart_eiffel.is_ready
		do
			current_out.put_real(r)
		end

	put_array1 (array_name: CHARACTER; value: INTEGER) is
		do
			current_out.put_character(array_name)
			current_out.put_character('[')
			current_out.put_integer(value)
			current_out.put_character(']')
		end

	put_se_string (c_string: STRING) is
		do
			current_out.put_string(once "(T7*)se_string(")
			put_string_c(c_string)
			current_out.put_character(')')
		end

	put_position (p: POSITION) is
		require
			on_c
		local
			hex: STRING
		do
			hex := once "0x        "
			hex.copy(once "0x")
			p.mangling.to_hexadecimal_in(hex)
			out_c.put_string(hex)
			put_position_comment_on(out_c, p)
		end

	put_position_in_ds (p: POSITION) is
		require
			on_c
		do
			out_c.put_string(once "ds.p=")
			put_position(p)
			out_c.put_string(fz_00)
		end

	put_target_as_target is
			-- Produce C code to pass the current stacked target as
			-- a target of a new call : user expanded are passed with
			-- a pointer and class invariant code is produced.
		require
			smart_eiffel.is_ready
		local
			code, class_invariant_flag: INTEGER; rf: RUN_FEATURE; type: TYPE; target: EXPRESSION
		do
			code := stack_top.code
			inspect
				code
			when C_direct_call then
				type := stack_top.type
				target := stack_top.target
				rf := stack_top.run_feature
				target.mapping_c_target(type, rf.type_of_current)
			when C_check_id then
				target := stack_top.target
				rf := stack_top.run_feature
				check
					rf.type_of_current.is_reference
				end
				if ace.boost then
					type := stack_top.type
					target.mapping_c_target(type, rf.type_of_current)
				else
					class_invariant_flag := class_invariant_call_opening(rf.type_of_current, False)
					check_id(stack_top.type, target, rf.id)
					if class_invariant_flag > 0 then
						class_invariant_call_closing(class_invariant_flag, False)
					end
				end
			else
				common_put_target
			end
		end

	put_target_as_value is
			-- Produce C code for a simple access to the stacked target.
			-- User's expanded values are not given using a pointer.
			-- There is no C code to check the class invariant.
		require
			smart_eiffel.is_ready
		local
			code: INTEGER
		do
			code := stack_top.code
			inspect
				code
			when C_direct_call then
				stack_top.target.compile_to_c(stack_top.type)
			when C_check_id then
				stack_top.run_feature.current_type.mapping_cast
				stack_top.target.compile_to_c(stack_top.type)
			else
				common_put_target
			end
		end

	put_error0 (msg: STRING) is
			-- Print `msg' and then stop execution.
			-- Also print stack when not -boost.
		do
			if ace.boost then
				put_comment(msg)
				put_string("%Nse_signal_handler(14/*System_level_type_error*/);%N")
			else
				put_string(once "error0(")
				put_string_c(msg)
				put_string(once ",NULL);%N")
			end
		end

	put_position_comment (p: POSITION) is
		do
			put_position_comment_on(current_out, p)
		end

	put_comment (str: STRING) is
		do
			put_string(fz_open_c_comment)
			put_string(str)
			put_string(fz_close_c_comment)
		end

	put_comment_line (str: STRING) is
		do
			put_character('%N')
			put_comment(str)
			put_character('%N')
		end

	define_main (rf3: RUN_FEATURE_3) is
		do
			swap_on_c
			-- Declare eiffel_root_object :
			tmp_string.clear_count
			tmp_string.extend('T')
			rf3.live_type.id.append_in(tmp_string)
			tmp_string.append(once "*eiffel_root_object")
			put_extern5(tmp_string, fz_null)
			-- Save argv argc :
			put_extern1(once "int se_argc")
			put_extern1(once "char**se_argv")
			define_initialize_eiffel_runtime(rf3)
			if not ace.no_main then
				really_define_c_main(rf3)
			end
		end

	trace_boolean_expression (type: TYPE; e: EXPRESSION) is
			-- Produce a C boolean expression including trace code.
		require
			e.resolve_in(type).is_boolean
			ace.no_check
		do
			put_character('(')
			put_trace_or_sedb_expression(e.start_position)
			put_character(',')
			e.compile_to_c(type)
			put_character(')')
		end

feature {INTEGER_CONSTANT}
	put_integer_64 (i: INTEGER_64) is
		do
			current_out.put_integer(i)
		end

feature {}
	get_started is
		require
			smart_eiffel.status.is_safety_checking
		do
			smart_eiffel.status.set_generating
			echo.file_removing(path_make)
			top := -1
			if ace.no_split then
				c_connect(path_c)
			else
				backup_tfw_connect(path_c)
			end
			current_out := out_c
			begin_c_linkage(out_c)
			add_first_include(path_h)
			h_connect(path_h)
			current_out := out_h
			begin_c_linkage(out_h)
			put_banner(out_h)
			out_h.put_character('%N')
			sys_runtime_h_and_c(fz_base)
			current_out := out_c
		ensure
			on_c
		end

	cecil_define is
		local
			save_out_h: like out_h
		do
			save_out_h := out_h
			cecil_pool.c_define_users
			out_h := save_out_h
		end

	echo_information is
		do
			if sure_void_count > 0 then
				echo.put_string(once "Calls with a Void target : ")
				echo.put_integer(sure_void_count)
				echo.put_string(once " (yes it is dangerous).%N")
			end
			echo.print_count(once "Direct Call", direct_call_count)
			echo.print_count(once "Check Id Call", check_id_call_count)
			echo.print_count(once "Switched Call", switched_call_count)
			echo.print_count(once "Inlined Procedure", inlined_procedure_count)
			echo.print_count(once "Inlined Function", inlined_function_count)
			echo.print_count(once "Procedure", procedure_count)
			echo.print_count(once "Function", function_count)
			echo.print_count(once "Procedure without Current", real_procedure_count)
			echo.print_count(once "Function without Current", real_function_count)
			echo.print_count(once "Precursor routine", precursor_routine_count)
			echo.put_string(once "Internal stacks size used : ")
			echo.put_integer(context_stack.count)
			echo.put_character('%N')
		end

	customize_runtime is
		do
			if ace.no_check then
				sys_runtime_h_and_c(fz_no_check)
				if ace.sedb then
					put_extern2(once "int se_general_trace_switch", '0')
					sys_runtime_h_and_c(fz_sedb)
				end
			else
				sys_runtime_h_and_c(fz_boost)
			end
			exceptions_handler.customize_c_runtime
			gc_handler.customize_c_runtime
			agent_pool.customize_c_runtime_1
			if smart_eiffel.deep_twin_used then
				sys_runtime_h_and_c(as_deep_twin)
			end
			if ace.profile then
				system_tools.add_lib_profile
				system_tools.add_lib_math
			end
			smart_eiffel.customize_runtime
		end

feature {RUN_FEATURE, NATIVE}
	target_cannot_be_dropped: BOOLEAN is
			-- True when top target cannot be dropped because we are not sure that
			-- target is non Void or that target has no side effects. When Result is True,
			-- printed C code is : "(((void)(<target>))"
		require
			smart_eiffel.is_ready
		local
			target: EXPRESSION; type, type_of_target: TYPE
		do
			inspect
				stack_top.code
			when C_direct_call, C_check_id then
				target := stack_top.target
				type := stack_top.type
				type_of_target := target.resolve_in(type)
				Result := not target.side_effect_free(type)
				if Result then
					put_string(once "((/*UT*/(void)(")
					if type_of_target = Void then
						--|*** How can it be Void ?? (Dom march 29th2004) ***
						put_target_as_target
					elseif type_of_target.is_user_expanded then
						put_target_as_value
					else
						put_target_as_target
					end
					put_string(fz_13)
				end
			else
			end
		end

	arguments_cannot_be_dropped: BOOLEAN is
			-- True when arguments cannot be dropped.
			-- Printed C code looks like:
			--  "(((void)<exp1>), ((void)<exp2>), ...((void)<expN>)"
		local
			args: EFFECTIVE_ARG_LIST; arg: EXPRESSION; i: INTEGER
		do
			inspect
				stack_top.code
			when C_direct_call, C_check_id, C_create_instruction then
				args := stack_top.effective_arguments
				if args = Void then
					-- Nothing to generate.
				else
					from
						i := 1
					until
						i > args.count
					loop
						arg := args.expression(i)
						if not arg.side_effect_free(stack_top.type) then
							if not Result then
								Result := True
								put_string(once "(/*UA*/((void)(")
							else
								put_string(once ",((void)(")
							end
							put_ith_argument(i)
							put_string(once "))")
						end
						i := i + 1
					end
				end
			else
				-- Nothing to generate.
			end
		end

	cannot_drop_all: BOOLEAN is
			-- Result is True when something (target or one argument)
			-- cannot be dropped. Thus when something cannot be dropped,
			-- Result is True and C code is printed :
			--  "(((void)<exp1>), ((void)<exp2>), ...((void)<expN>)"
		do
			if target_cannot_be_dropped then
				Result := True
				put_character(',')
				if arguments_cannot_be_dropped then
					put_character(')')
				else
					put_character('0')
				end
			else
				Result := arguments_cannot_be_dropped
			end
		end

feature {ANY}
	put_arguments is
			-- Produce code to access to the whole effective arguments list.
		require
			smart_eiffel.is_ready
		local
			i, c: INTEGER; fal: FORMAL_ARG_LIST; rf: RUN_FEATURE
		do
			-- Compute arguments count in `c':
			rf := stack_top.run_feature
			fal := rf.arguments
			c := fal.count -- Put all arguments:
			from
				i := 1
			until
				i > c
			loop
				put_ith_argument(i)
				if i < c then
					put_character(',')
				end
				i := i + 1
			end
		end

	put_ith_argument (index: INTEGER) is
			-- Produce code to access to the ith argument.
		require
			smart_eiffel.is_ready
			index >= 1
		local
			code: INTEGER; rf: RUN_FEATURE; fal: FORMAL_ARG_LIST; switch: SWITCH; unwrap_buffer: STRING
			type: TYPE
		do
			code := stack_top.code
			inspect
				code
			when C_switch then
				rf := stack_top.run_feature
				type := stack_top.type
				fal := rf.arguments
				switch.put_ith_argument(stack_top.static_type, type, stack_top.formal_arguments, fal, index)
			when C_create_expression, C_scoop_wrapper then
				put_character('a')
				put_integer(index)
			when C_inside_twin then
				check
					index = 1
				end
				if stack_top.run_feature.current_type.is_reference then
					put_string(once "((T0*)C)")
				else
					put_string(once "*C")
				end
			when C_create_instruction, C_direct_call, C_check_id, C_precursor then
				fal := stack_top.run_feature.arguments
				type := stack_top.type
				stack_top.effective_arguments.compile_to_c_ith(type, fal, index)
			when C_scoop_unwrapper then
				unwrap_buffer := once "                "
				unwrap_buffer.copy(once "((Tw")
				stack_top.run_feature.id.append_in(unwrap_buffer)
				stack_top.run_feature.name.mapping_c_in(unwrap_buffer)
				unwrap_buffer.append(once "*)data)->_")
				unwrap_buffer.append(stack_top.run_feature.arguments.name(index).to_string)
				out_c.put_string(unwrap_buffer)
			end
		end

feature {NATIVE_BUILT_IN}
	put_c_inline_h is
		local
			code: STRING
		do
			code := get_inline_ms.to_string
			if not c_inline_h_mem.fast_has(code) then
				c_inline_h_mem.add_last(code)
				out_h.put_string(code)
				out_h.put_character('%N')
			end
		end

	put_c_inline_c is
		local
			code: MANIFEST_STRING
		do
			code := get_inline_ms
			out_c.put_string(code.to_string)
		end

	put_trace_switch is
			-- The {ANY}.trace_switch feature
		do
			if ace.sedb then
				put_string(once "se_general_trace_switch=(")
				put_ith_argument(1)
				put_string(fz_14)
			end
		end

	put_sedb_breakpoint is
		require
			on_c
		local
			e: EXPRESSION; p: POSITION
		do
			if ace.sedb then
				out_c.put_string(once "sedb_breakpoint(&ds,")
				e := stack_top.target
				if e /= Void then
					p := e.start_position
				end
				put_position(p)
				put_string(once ");%N")
			end
		end

	put_target_generating_type is
		local
			rc: LIVE_TYPE
			t: TYPE_MARK
		do
			t := stack_top.run_feature.current_type
			put_string(fz_cast_t0_star)
			put_character('(')
			put_character('t')
			put_character('[')
			if t.is_reference then
				rc := t.live_type
				if rc.is_tagged then
					put_character('(')
					put_target_as_value
					put_character(')')
					put_string(fz_arrow_id)
				else
					put_integer(rc.id)
				end
			else
				put_integer(t.id)
			end
			put_character(']')
			put_character(')')
		end

	put_target_generator is
		local
			rc: LIVE_TYPE
			t: TYPE_MARK
		do
			t := stack_top.run_feature.current_type
			put_string(fz_cast_t0_star)
			put_character('(')
			put_character('g')
			put_character('[')
			if t.is_reference then
				rc := t.live_type
				if rc.is_tagged then
					put_character('(')
					put_target_as_value
					put_character(')')
					put_string(fz_arrow_id)
				else
					put_integer(rc.id)
				end
			else
				put_integer(t.id)
			end
			put_character(']')
			put_character(')')
		end

	target_position_in_error_handler is
			-- Add the target position into the `error_handler'.
		local
			target: EXPRESSION
		do
			target := stack_top.target
			if target /= Void then
				error_handler.add_position(target.start_position)
			end
		end

	put_object_size (t: TYPE_MARK) is
		require
			t.is_static
		local
			tcbd: BOOLEAN
		do
			tcbd := target_cannot_be_dropped
			if tcbd then
				out_c.put_character(',')
			end
			out_c.put_string(once "sizeof(T")
			out_c.put_integer(t.id)
			out_c.put_character(')')
			if tcbd then
				out_c.put_character(')')
			end
		end

feature {CECIL_FILE}
	connect_cecil_out_h (user_path_h: STRING) is
		do
			create out_h.make
			echo.tfw_connect(out_h, user_path_h)
			system_tools.mandatory_sys_runtime(fz_base, 'h')
			put_file(tmp_file_read, out_h)
			out_h.put_string(once "extern void*eiffel_root_object;%N%N%
			  %typedef unsigned char* T9;%N%
			  %/* Available Eiffel routines via -cecil:%N*/%N")
		end

	disconnect_cecil_out_h is
		do
			out_h.disconnect
		end

feature {CECIL_POOL, RUN_FEATURE}
	push_scoop_unwrapper (type: TYPE; rf: RUN_FEATURE) is
		require
			type /= Void
			rf /= Void
		do
			stack_push(C_scoop_unwrapper)
			stack_top.set_type(type)
			stack_top.set_run_feature(rf)
		end

	push_scoop_wrapper (type: TYPE; rf: RUN_FEATURE) is
		require
			type /= Void
			rf /= Void
		do
			stack_push(C_scoop_wrapper)
			stack_top.set_type(type)
			stack_top.set_run_feature(rf)
		end

feature {RUN_FEATURE}
	push_introspect (type: TYPE; rf: RUN_FEATURE) is
		require
			type /= Void
			rf /= Void
		do
			stack_push(C_introspect)
			stack_top.set_type(type)
			stack_top.set_run_feature(rf)
		end

feature {MANIFEST_STRING_INSPECTOR}
	put_string_inspector (a_feature: STRING) is
		local
			live_type: LIVE_TYPE; rf: RUN_FEATURE
		do
			check
				a_feature = as_storage or else a_feature = as_count
			end
			live_type := smart_eiffel.type_string.live_type
			rf := live_type.at(live_type.type.name_from_string(a_feature))
			stack_push(C_string_inspector)
			stack_top.set_run_feature(rf)
			stack_top.set_type(rf.type_of_current)
			rf.mapping_c
			pop
		end

feature {CECIL_POOL}
	push_pop_cpc (up_rf: RUN_FEATURE; run_time_set: RUN_TIME_SET; type: TYPE; fs: FEATURE_STAMP; t: EXPRESSION
		args: EFFECTIVE_ARG_LIST) is
			--|*** The `up_rf' argument will be removed in the futur, so I have added the `fs'.
			--|*** (Dom. june 8th 2004) ***
		require
			run_time_set = up_rf.live_type.run_time_set
		local
			dyn_rf: RUN_FEATURE
		do
			if run_time_set.count = 0 then
				se_evobt(up_rf.result_type, type, t)
			elseif run_time_set.count = 1 then
				dyn_rf := run_time_set.first.dynamic_feature(up_rf)
				push_check(dyn_rf, type, t, args)
				dyn_rf.mapping_c
				pop
			elseif dispatch_special(type, fs, t) then
				dyn_rf := run_time_set.first.dynamic_feature(up_rf)
				push_check(dyn_rf, type, t, args)
				dyn_rf.mapping_c
				pop
			elseif once_routine_pool.no_dispatch_for(fs, up_rf.type_of_current) then
				--|*** Warning. This is a hack ***
				--| The goal is to avoid dynamic dispatch on precomputed
				--| once. This is a close friend of the hack in
				--| LIVE_TYPE.make_switch_sites <fm-08/09/2004>
				dyn_rf := run_time_set.first.dynamic_feature(up_rf)
				push_check(dyn_rf, type, t, args)
				dyn_rf.mapping_c
				pop
			else
				use_switch(up_rf, run_time_set, type, t, args)
			end
		end

feature {SWITCH}
	inside_switch_call (dyn_rf, static_rf: RUN_FEATURE) is
			-- Call the `dyn_rf' inside switching function for `static_rf'.
		obsolete "-old_lookup stuff (PH 24/04/2003)"
		require
			dyn_rf /= Void
			static_rf /= Void
			dyn_rf.live_type.dynamic_feature(static_rf) = dyn_rf
		local
			dyn_result, static_result: TYPE_MARK; msg: STRING; cast, no_code: BOOLEAN
		do
			-- Extra comment to debug C code:
			-- put_comment(dyn_rf.current_type.written_mark)
			--
			static_result := static_rf.result_type
			if static_result /= Void then
				dyn_result := dyn_rf.result_type
				if static_result.is_user_expanded then
					if dyn_result.is_user_expanded then
						if dyn_result.live_type /= static_result.live_type then
							no_code := True
						end
					end
				end
			end
			if no_code then
				msg := "Conversion from: "
				msg.append(dyn_result.written_mark)
				msg.append(once " to: ")
				msg.append(static_result.written_mark)
				msg.append(once " not yet implemented (sorry). Please report on SmartEiffel@loria.fr.)")
				put_error0(msg)
			else
				-- Opening section for Result:
				if static_result /= Void then
					if static_result.is_boolean then
						out_c.put_string(once "R=(T6)(")
					else
						out_c.put_string(fz_26)
					end
					if static_result.is_native_array or else dyn_result.is_none and then static_result.is_expanded then
						cast := True
						out_c.put_string(fz_b7)
						out_c.put_integer(static_result.id)
						out_c.put_character(')')
						out_c.put_character('(')
					end
				end
				stack_push(C_switch)
				stack_top.set_run_feature(dyn_rf)
				stack_top.set_type(dyn_rf.type_of_current)
				stack_top.set_static_run_feature(static_rf)
				dyn_rf.mapping_c
				pop
				-- Closing section for Result:
				if static_result /= Void then
					if cast then
						out_c.put_character(')')
						out_c.put_character(')')
					end
					out_c.put_string(fz_14)
				end
			end
		end

	new_inside_switch_call (dyn_rf: RUN_FEATURE; arg: FORMAL_ARG_LIST; ct, rt: TYPE) is
			-- Call the `dyn_rf' inside switching function for `static_rf'.
		require
			dyn_rf.arguments /= Void implies dyn_rf.arguments.count = arg.count
		local
			dyn_result, static_result: TYPE_MARK; msg: STRING; cast, no_code: BOOLEAN
		do
			-- Extra comment to debug C code:
			-- put_comment(dyn_rf.current_type.written_mark)
			--
			if rt /= Void then
				static_result := rt.canonical_type_mark
				dyn_result := dyn_rf.result_type
				if static_result.is_user_expanded then
					if dyn_result.is_user_expanded then
						if dyn_result.live_type /= static_result.live_type then
							no_code := True
						end
					end
				end
			end
			if no_code then
				msg := "Conversion from: "
				msg.append(dyn_result.written_mark)
				msg.append(once " to: ")
				msg.append(static_result.written_mark)
				msg.append(once " not yet implemented (sorry). Please report on SmartEiffel@loria.fr.)")
				put_error0(msg)
			else
				-- Opening section for Result:
				if static_result /= Void then
					if static_result.is_boolean then
						out_c.put_string(once "R=(T6)(")
					else
						out_c.put_string(fz_26)
					end
					if static_result.is_native_array or else dyn_result.is_none and then static_result.is_expanded then
						cast := True
						out_c.put_string(fz_b7)
						out_c.put_integer(static_result.id)
						out_c.put_character(')')
						out_c.put_character('(')
					end
				end
				stack_push(C_switch)
				stack_top.set_run_feature(dyn_rf)
				stack_top.set_type(dyn_rf.type_of_current)
				stack_top.set_formal_arguments(arg)
				stack_top.set_static_type(ct)
				dyn_rf.mapping_c
				pop
				-- Closing section for Result:
				if static_result /= Void then
					if cast then
						out_c.put_character(')')
						out_c.put_character(')')
					end
					out_c.put_string(fz_14)
				end
			end
		end

feature {LIVE_TYPE}
	push_create_expression (type: TYPE; fs: FEATURE_STAMP) is
		require
			type /= Void
			fs /= Void
		do
			stack_push(C_create_expression)
			stack_top.set_type(type)
			stack_top.set_run_feature(fs.run_feature_for(type))
			direct_call_count := direct_call_count + 1
		end

feature {CREATE_INSTRUCTION, LOCAL_VAR_LIST, ONCE_ROUTINE_POOL, RUN_FEATURE}
	push_create_instruction (type: TYPE; rf: RUN_FEATURE; args: EFFECTIVE_ARG_LIST) is
		require
			type /= Void
			rf /= Void
		do
			stack_push(C_create_instruction)
			stack_top.set_type(type)
			stack_top.set_run_feature(rf)
			stack_top.set_effective_arguments(args)
			direct_call_count := direct_call_count + 1
		end

feature {CREATE_INSTRUCTION}
	expanded_attributes (type: TYPE; rt: TYPE_MARK) is
			-- Produce C code to initialize expanded attribute of the new object juste created in variable "n".
		require
			smart_eiffel.is_ready
			rt.is_static
		local
			wa: ARRAY[RUN_FEATURE]; a: RUN_FEATURE; at: TYPE_MARK; i: INTEGER; rf3: RUN_FEATURE_3
		do
			wa := rt.live_type.writable_attributes
			if wa /= Void then
				from
					i := wa.upper
				until
					i = 0
				loop
					a := wa.item(i)
					at := a.result_type
					if at.is_user_expanded then
						rf3 := at.live_type.default_create_run_feature
						if rf3 /= Void then
							stack_push(C_expanded_initialize)
							stack_top.set_type(type)
							stack_top.set_string(Void)
							stack_top.set_target(Void)
							stack_top.set_run_feature(a)
							direct_call_count := direct_call_count + 1
							rf3.mapping_c
							pop
						end
					end
					i := i - 1
				end
			end
		end

feature {NATIVE}
	inside_twin (type: TYPE; cpy: RUN_FEATURE) is
		require
			type /= Void
			cpy /= Void
		do
			stack_push(C_inside_twin)
			stack_top.set_type(type)
			stack_top.set_run_feature(cpy)
			cpy.mapping_c
			pop
		end

feature {ASSERTION}
	check_assertion (type: TYPE; e: EXPRESSION; tag_name: STRING) is
			-- Produce a C boolean expression including trace code
			-- and assertion check.
		require
			e.resolve_in(type).is_boolean
		do
			put_trace_or_sedb_instruction(e.start_position)
			put_string(once "ac_")
			put_string(check_assertion_mode)
			put_character('(')
			e.compile_to_c(type)
			put_character(',')
			if tag_name /= Void then
				put_string_c(tag_name)
			else
				put_string(fz_null)
			end
			put_string(fz_14)
		end

feature {ASSERTION_LIST}
	set_check_assertion_mode (s: STRING) is
		require
			s /= Void
		do
			check_assertion_mode := s
		ensure
			check_assertion_mode = s
		end

feature {ANY}
	put_trace_or_sedb_expression (position: POSITION) is
			-- Writes on `out_c' an `sedb(...)' call or some `ds.p'
			-- update according to the ACE mode of the given `position'.
		require
			not ace.boost
			not position.is_unknown
		do
			if position.sedb_trace then
				out_c.put_string(once "sedb(&ds,")
			else
				out_c.put_string(once "(ds.p=")
			end
			put_position(position)
			out_c.put_character(')')
		end

	put_trace_or_sedb_instruction (position: POSITION) is
		require
			not ace.boost
			not position.is_unknown
		do
			put_trace_or_sedb_expression(position)
			out_c.put_string(fz_00)
		end

feature {ANY} -- Numbering of inspect variables :
	inspect_incr is
		do
			inspect_level := inspect_level + 1
		end

	inspect_decr is
		do
			inspect_level := inspect_level - 1
		end

	put_inspect is
		do
			put_character('z')
			put_integer(inspect_level)
		end

feature {ANY} -- Printing Current, local or argument :
	inline_level_incr is
		do
			inline_level := inline_level + 1
		end

	inline_level_decr is
		do
			inline_level := inline_level - 1
		end

	print_current is
		require
			on_c
		local
			level: INTEGER
		do
			out_c.put_character('C')
			level := inline_level
			if level > 0 then
				out_c.put_integer(level)
			end
		end

	print_argument (rank: INTEGER) is
		require
			on_c
		local
			code: INTEGER
		do
			code := ('a').code + inline_level
			out_c.put_character(code.to_character)
			out_c.put_integer(rank)
		end

	print_local (name: STRING) is
		require
			on_c
		local
			level: INTEGER
		do
			from
				level := inline_level + 1
			until
				level = 0
			loop
				out_c.put_character('_')
				level := level - 1
			end
			out_c.put_string(name)
		end

feature {E_LOOP}
	variant_check (type: TYPE; e: EXPRESSION) is
		require
			e /= Void
		do
			put_trace_or_sedb_instruction(e.start_position)
			put_string(once "v=ac_lvc(c++,v,")
			e.compile_to_c(type)
			put_string(fz_14)
		end

feature {RUN_FEATURE}
	current_class_invariant (current_type: TYPE_MARK) is
			-- Add some C code to check class invariant with Current at the end of a routine for `Current'.
		require
			current_type.is_static
		local
			lt: LIVE_TYPE
		do
			lt := need_invariant(current_type.type)
			if lt /= Void then
				if lt.is_reference then
					out_c.put_string(once "if(se_rci(caller,C))")
				end
				out_c.put_string(once "se_i")
				out_c.put_integer(lt.id)
				if ace.profile then
					out_c.put_string(once "(&ds,&local_profile,C);%N")
				else
					out_c.put_string(once "(&ds,C);%N")
				end
			end
		end

feature {ANY}
	class_invariant_call_opening (type_of_target: TYPE; extra_cast_flag: BOOLEAN): INTEGER is
			-- Add extra code to call the class invariant when `type_of_target' actually has an invariant.
			-- A 0 `Result' indicate that there is no invariant.
		require
			type_of_target /= Void
		local
			lt: LIVE_TYPE
		do
			lt := need_invariant(type_of_target)
			if lt /= Void then
				out_c.put_string(once "se_i")
				out_c.put_integer(lt.id)
				out_c.put_string(once "(&ds,")
				if ace.profile then
					out_c.put_string(once "&local_profile,")
				end
				if type_of_target.is_reference then
					if extra_cast_flag then
						out_c.put_string(once "((T")
						out_c.put_integer(type_of_target.live_type.id)
						out_c.put_string(once "*)(")
						Result := 3
					else
						Result := 1
					end
				else
					Result := 1
				end
			end
		end

	class_invariant_call_closing (integer_flag: INTEGER; semicolon_flag: BOOLEAN) is
		do
			inspect
				integer_flag
			when 1 then
				cpp.put_character(')')
			when 3 then
				cpp.put_string(once ")))")
			end
			if semicolon_flag then 
				cpp.put_character(';')
			end
		end
	
feature {ANY}
	macro_def (str: STRING; id: INTEGER) is
		do
			tmp_string.copy(once "#define ")
			tmp_string.append(str)
			tmp_string.extend(' ')
			id.append_in(tmp_string)
			tmp_string.extend('%N')
			out_h.put_string(tmp_string)
		end

feature {}
	write_make_file is
		local
			score: REAL; cmd: STRING
		do
			--
			end_c_linkage(out_h)
			out_h.disconnect
			end_c_linkage(out_c)
			out_c.disconnect
			--
			c_plus_plus_definitions
			--
			echo.tfw_connect(out_make, path_make)
			if ace.no_split then
				write_make_file_no_split
			else
				write_make_file_split
			end
			if not executable_is_up_to_date then
				cmd := system_tools.strip_executable
				if cmd /= Void then
					echo_make(cmd)
				end
			end
			out_make.disconnect
			if nb_errors > 0 then
				echo.file_removing(path_make)
			else
				echo.put_string(once "Type inference score: ")
				score := direct_call_count + check_id_call_count
				score := score / (score + switched_call_count) * 100.0
				echo.put_real_format(score, 2)
				echo.put_character('%%')
				echo.put_character('%N')
			end
		end

feature {CALL_SUPPORT, AGENT_CREATION}
	put_cpc (type: TYPE; fs: FEATURE_STAMP; run_feature: RUN_FEATURE; target: EXPRESSION
		arguments: EFFECTIVE_ARG_LIST) is
		local
			type_of_target: TYPE; dynamic_feature: RUN_FEATURE
		do
			type_of_target := target.resolve_in(type)
			dynamic_feature := type_of_target.live_type.dynamic_feature(run_feature)
			if type_of_target.is_expanded or else target.is_current or else target.is_manifest_string then
				push_direct(dynamic_feature, type, target, arguments)
				dynamic_feature.mapping_c
				pop
			else
				push_pop_cpc(dynamic_feature, type_of_target.live_type.run_time_set, type, fs, target, arguments)
			end
		end

feature {ANY}
	sys_runtime_h_and_c (name: STRING) is
			--|*** (Dom may 7th 2004) *** OBSOLETE (OLD PLUG_IN) ***
			-- Inline corresponding SmartEiffel/sys/runtime/`name'.[hc]
			-- file. (At least, one file should be found.)
		local
			ok: BOOLEAN
		do
			system_tools.sys_runtime(name, 'h')
			if tmp_file_read.is_connected then
				ok := True
				put_file(tmp_file_read, out_h)
			end
			system_tools.sys_runtime(name, 'c')
			if tmp_file_read.is_connected then
				ok := True
				put_file(tmp_file_read, out_c)
			end
			if not ok then
				error_handler.append("No support found in directory sys/runtime for %"")
				error_handler.append(name)
				error_handler.append("%" (i.e. file(s) %"")
				tmp_path.remove_the_last
				error_handler.append(tmp_path)
				error_handler.append("[hc]%" not found).")
				error_handler.print_as_fatal_error
			end
		end

feature {PLUGIN}
	put_c_file (tfr: TEXT_FILE_READ) is
		require
			not tfr.end_of_input
		do
			put_file(tfr, out_c)
		ensure
			not tfr.is_connected
		end

	put_h_file (tfr: TEXT_FILE_READ) is
		require
			not tfr.end_of_input
		do
			put_file(tfr, out_h)
		ensure
			not tfr.is_connected
		end

feature {ANY}
	put_recompilation_comment (key: INTEGER) is
		do
			out_c.put_string(fz_open_c_comment)
			out_c.put_integer(key)
			out_c.put_string(fz_close_c_comment)
		end

	put_recompilation_comment_in (buffer: STRING; key: INTEGER) is
		do
			buffer.append(fz_open_c_comment)
			key.append_in(buffer)
			buffer.append(fz_close_c_comment)
		end

feature {CALL, CREATE_EXPRESSION}
	put_xloc_exp (expanded_type: TYPE) is
		require
			expanded_type.is_user_expanded
		do
			out_c.put_string(once "xloc_exp")
			out_c.put_integer(expanded_type.live_type.id)
		end

feature {RUN_FEATURE, CLASS_INVARIANT}
	put_extra_local_expanded (extra_local_expanded_list: FAST_ARRAY[TYPE]) is
			-- Put needed `xloc_expXX' declarations of temporary expanded variables.
		require
			extra_local_expanded_list /= Void implies extra_local_expanded_list.count > 0
		local
			i: INTEGER; type: TYPE
		do
			if extra_local_expanded_list /= Void then
				from
					tmp_string.clear_count
					i := extra_local_expanded_list.upper
				until
					i < extra_local_expanded_list.lower
				loop
					type := extra_local_expanded_list.item(i)
					check
						type.is_user_expanded
					end
					type.canonical_type_mark.c_type_for_result_in(tmp_string)
					tmp_string.append(once " xloc_exp")
					type.live_type.id.append_in(tmp_string)
					tmp_string.append(fz_00)
					i := i - 1
				end
				out_c.put_string(tmp_string)
			end
		end

	se_tmp_open_separate (t: TYPE_MARK): INTEGER is
			-- The `Result' is the index of the `se_tmp' variable, because this
			-- variable may be used more than once. Result >= 0 implies that
			-- a temporary variable is used and thus that
			-- `se_tmp_close_separate' must be called. Result < 0 indicate that
			-- `t' is not registered.
		require
			t.is_separate
		local
			stop: BOOLEAN
		do
			-- 			from
			-- 				Result := se_tmp_list.upper
			-- 			until
			-- 				stop
			-- 			loop
			-- 				if Result < 0 then
			-- 					stop := True
			-- 				elseif t = se_tmp_list.item(Result) then
			-- 					stop := True
			-- 					se_tmp_list.put(Void, Result)
			-- 					out_c.put_string(once "(((void)(/*separate*/se_tmp")
			-- 					out_c.put_integer(Result)
			-- 					out_c.put_character('=')
			-- 					put_target_as_target
			-- 					out_c.put_string(once ")),")
			-- 				else
			-- 					Result := Result - 1
			-- 				end
			-- 			end
		end

	se_tmp_close_separate (se_tmp_idx: INTEGER) is
			-- Assume `se_tmp_idx' is the value obtained with
			-- `se_tmp_open_separate'.
		require
			se_tmp_idx >= 0
		do
			-- 			check -- Obtained with `se_tmp_open_separate':
			-- 				se_tmp_list.item(se_tmp_idx) = Void
			-- 			end
			-- 			out_c.put_character(')')
		end

feature {NATIVE}
	include_register (origin: POSITION; include_name: STRING) is
		do
			if include_memory = Void then
				create {HASHED_SET[STRING]} include_memory.make
			end
			if not include_memory.has(include_name) then
				include_memory.add(include_name)
				put_position_comment_on(out_h, origin)
				add_include_on(out_h, include_name)
				put_position_comment_on(out_h, origin)
			end
		end

feature {NATIVE_C_PLUS_PLUS}
	c_plus_plus_register (native: NATIVE_C_PLUS_PLUS) is
		do
			if c_plus_plus = Void then
				create c_plus_plus.with_capacity(64)
			end
			c_plus_plus.add_last(native)
		end

	add_include (include: STRING) is
		do
			add_include_on(out_h, include)
		end

	add_include_on (output: TEXT_FILE_WRITE; include: STRING) is
		do
			end_c_linkage(output)
			output.put_string(once "#include ")
			inspect
				include.first
			when '%"', '<' then
			else
				output.put_character('%"')
			end
			output.put_string(include)
			inspect
				include.last
			when '%"', '>' then
			else
				output.put_character('%"')
			end
			output.put_character('%N')
			begin_c_linkage(output)
		end

feature {RUN_FEATURE_2}
	attribute_read_removal (up_rf: RUN_FEATURE; run_time_set: RUN_TIME_SET): BOOLEAN is
		require
			smart_eiffel.is_ready
			run_time_set.count > 1
		local
			rf2a, rf2b: RUN_FEATURE_2; offseta, offsetb: INTEGER; rta, rtb: TYPE_MARK; i: INTEGER; lt: LIVE_TYPE
		do
			from
				Result := True
				i := run_time_set.count
			until
				not Result or else i = 0
			loop
				lt := run_time_set.item(i)
				rf2b ?= lt.dynamic_feature(up_rf)
				if rf2b /= Void then
					offsetb := lt.offset_of(rf2b)
				end
				if rf2a = Void then
					rf2a := rf2b
					offseta := offsetb
				end
				if rf2b = Void then
					Result := False
				else
					Result := offseta = offsetb
					if Result then
						rta := rf2a.result_type
						rtb := rf2b.result_type
						if rta.is_reference then
							Result := rtb.is_reference
						elseif rta.written_name = rtb.written_name then
						elseif rta.is_native_array then
							if rta.generic_list.first.is_reference then
								Result := rtb.generic_list.first.is_reference
							end
						else
							Result := False
						end
					end
				end
				i := i - 1
			end
		end

feature {ANY}
	c_set_dump_stack_top_for_in (t: TYPE; code_buffer, ds, comment: STRING) is
		do
			code_buffer.append(once "set_dump_stack_top(")
			if smart_eiffel.scoop then
				current_thread_for(t, code_buffer)
				code_buffer.extend(',')
			end
			code_buffer.append(ds)
			code_buffer.append(once ");/*")
			code_buffer.append(comment)
			code_buffer.append(once "*/%N")
		end

	c_set_dump_stack_top_for (t: TYPE; ds, comment: STRING) is
		require
			on_c
		local
			code: STRING
		do
			code := once ""
			code.clear_count
			c_set_dump_stack_top_for_in(t, code, ds, comment)
			out_c.put_string(code)
		end

	current_thread_for (t: TYPE; buffer: STRING) is
		require
			smart_eiffel.scoop
		do
			if t.is_separate then
				buffer.append(once "self")
			else
				buffer.append(once "se_current_subsystem_thread()")
			end
		end

feature {E_REQUIRE, ASSERTION_LIST}
	stop_recursive_assertion_opening (inside_feature_flag: BOOLEAN) is
		do
			-- Remove the `assertion_flag' stuff in boost'ed SCOOP guards
			if ace.no_check then --|*** should be require_check?
				if ace.flat_check then
					put_string(once "if(assertion_depth){%Nassertion_depth--;%N")
				else
					if inside_feature_flag then
						put_string(once "if(fd.assertion_flag){%Nfd.assertion_flag=0;%N")
					else
						put_string(once "if(ds.fd->assertion_flag){%Nds.fd->assertion_flag=0;%N")
					end
				end
			end
		end

	stop_recursive_assertion_closing (inside_feature_flag: BOOLEAN) is
		do
			-- Remove the `assertion_flag' stuff in boost'ed SCOOP guards
			if ace.no_check then
				if ace.flat_check then
					put_string(once "assertion_depth++;%N}%N")
				else
					if inside_feature_flag then
						put_string(once "fd.assertion_flag=1;%N}%N")
					else
						put_string(once "ds.fd->assertion_flag=1;%N}%N")
					end
				end
			end
		end

feature {E_OLD, RUN_FEATURE}
	c_frame_descriptor_format: STRING is
			-- The format to print `Current' and other locals.
		once
			create Result.make(512)
		end

	c_frame_descriptor_locals: STRING is
			-- To initialize field `locals' of `se_dump_stack'.
		once
			create Result.make(512)
		end

feature {}
	c_plus_plus: FAST_ARRAY[NATIVE_C_PLUS_PLUS]

	begin_c_linkage (output: TEXT_FILE_WRITE) is
			-- Begin wrap for C linkage
		do
			output.put_string(once "#ifdef __cplusplus%Nextern %"C%" {%N#endif%N")
		end

	end_c_linkage (output: TEXT_FILE_WRITE) is
			-- End wrap for C linkage
		do
			output.put_string(once "%N#ifdef __cplusplus%N}%N#endif%N")
		end

	c_plus_plus_definitions is
		local
			cpp_path_h, cpp_path_c: STRING; i: INTEGER; no_split_save: BOOLEAN
		do
			if c_plus_plus /= Void then
				no_split_save := ace.no_split
				ace.set_no_split(True)
				echo.put_string(once "C++ external definitions.%N")
				cpp_path_h := system_tools.path_h.twin
				cpp_path_h.remove_tail(2)
				cpp_path_h.append(once "_external_cpp")
				cpp_path_c := cpp_path_h.twin
				cpp_path_h.append(h_suffix)
				cpp_path_c.append(c_plus_plus_suffix)
				echo.tfw_connect(out_h, cpp_path_h)
				echo.tfw_connect(out_c, cpp_path_c)
				begin_c_linkage(out_c)
				begin_c_linkage(out_h)
				add_first_include(cpp_path_h)
				system_tools.add_c_plus_plus_file(cpp_path_c)
				sys_runtime_h_and_c(once "c_plus_plus")
				if c_plus_plus /= Void then
					from
						i := c_plus_plus.upper
					until
						i < 0
					loop
						c_plus_plus.item(i).c_plus_plus_definition
						i := i - 1
					end
				end
				end_c_linkage(out_c)
				end_c_linkage(out_h)
				out_h.disconnect
				out_c.disconnect
				ace.set_no_split(no_split_save)
			end
		end

	echo_make (cmd: STRING) is
		do
			out_make.put_string(cmd)
			out_make.put_character('%N')
		end

	out_c: TEXT_FILE_WRITE
			-- The current *.c output file.

	out_h: TEXT_FILE_WRITE
			-- The *.h output file.

	in_h: TEXT_FILE_READ
			-- The *.h, when copied in the symbols file

	current_out: TEXT_FILE_WRITE
			-- Is `out_c' or `out_h'.

	out_make: TEXT_FILE_WRITE is
			-- The *.make output file.
		once
			create Result.make
		end

	c_inline_h_mem: FAST_ARRAY[STRING] is
		once
			create Result.with_capacity(4)
		end

feature {VOID_PROC_CALL, VOID_CALL}
	se_evobt (return_type: TYPE_MARK; type: TYPE; target: EXPRESSION) is
		require
			target /= Void
			on_c
		local
			p: POSITION
		do
			sure_void_count := sure_void_count + 1
			if return_type /= Void then
				if return_type.is_reference then
					-- Because of a Borland C compiler bug we have to add
					-- this extra cast:
					out_c.put_string(once "(T0*)")
				end
				out_c.put_character('(')
			end
			if ace.no_check then
				out_c.put_string(once "se_evobt")
				out_c.put_character('(')
				target.compile_to_c(type)
				out_c.put_character(',')
				put_position(target.start_position)
				out_c.put_character(')')
			else
				out_c.put_string(once "/*se_evobt*/")
				p := target.start_position
				put_position_comment_on(out_c, p)
				target.compile_to_c(type)
				out_c.put_character(',')
				exceptions_handler.se_evobt
			end
			if return_type /= Void then
				out_c.put_character(',')
				if return_type.is_reference then
					return_type.c_initialize
				else
					out_c.put_character('M')
					out_c.put_integer(return_type.id)
				end
				out_c.put_character(')')
			end
			if return_type = Void then
				out_c.put_character(';')
				out_c.put_character('%N')
			end
		end

feature {}
	use_switch (up_rf: RUN_FEATURE; run_time_set: RUN_TIME_SET; type: TYPE; target: EXPRESSION
		args: EFFECTIVE_ARG_LIST) is
		require
			up_rf.live_type.run_time_set = run_time_set
			run_time_set.count > 1
			target /= Void
			on_c
		local
			rt: TYPE_MARK; switch: SWITCH
		do
			rt := up_rf.result_type
			switched_call_count := switched_call_count + 1
			out_c.put_string(switch.name(up_rf))
			out_c.put_character('(')
			if ace.profile then
				out_c.put_string(once "&local_profile,")
			end
			if ace.no_check then
				out_c.put_string(fz_85)
				put_position(target.start_position)
				out_c.put_character(',')
			end
			target.compile_to_c(type)
			if args /= Void then
				out_c.put_character(',')
				args.compile_to_c(type, up_rf.arguments)
			end
			put_character(')')
			if rt = Void then
				out_c.put_string(fz_00)
			end
		end

	define_initialize_eiffel_runtime (rf3: RUN_FEATURE_3) is
		require
			on_c
			rf3.is_root
		local
			no_check: BOOLEAN; lt: LIVE_TYPE; ct: TYPE_MARK; do_atexit: BOOLEAN; se_atexit_stamp: FEATURE_STAMP
			buffer: STRING
		do
			no_check := ace.no_check
			ct := rf3.current_type
			lt := rf3.live_type
			echo.put_string(once "Define initialize stuff.%N")
			define_extern_tables
			once_routine_pool.c_variables_for_precomputable_routines
			-- Define atexit() function, if useful (i.e. std_output is used, or SCOOP, which defines its own atexit methods):
			se_atexit_stamp := smart_eiffel.se_atexit_stamp
			do_atexit := se_atexit_stamp /= Void or else smart_eiffel.scoop or else ace.profile
			if do_atexit then
				put_c_heading(once "void se_atexit(void)")
				if ace.profile then
					out_c.put_string(once "se_local_profile_t local_profile, *parent_profile;%N")
				end
				if no_check then
					out_c.put_string(once "[
                 se_frame_descriptor fd={"<atexit wrapper>",0,0,"",1};
                 se_dump_stack ds;
                 ds.fd=&fd;
                 ds.p=0;
                 ds.caller=NULL;
                 ds.exception_origin=NULL;
                 ds.locals=NULL;

		           ]")
				end
				if ace.profile then
					out_c.put_string(once "parent_profile=&global_profile;%N")
					out_c.put_string(once "local_profile.profile=&atexit_profile;%N")
					out_c.put_string(once "start_profile(parent_profile, &local_profile);%N")
				end
				if se_atexit_stamp /= Void then
					-- Add the C code to call it knowing that the corresponding feature is a procedure which does not 
					-- use  `Current' (See {ANY}.se_atexit definition).
					if ace.sedb then
						-- (Calling Eiffel function flush would result in running sedb again whereas the user said s/he 
						-- wanted to exit.)
						out_c.put_string("if (sedb_status != SEDB_EXIT_MODE) ")
					end
					buffer := once "............................"
					buffer.clear_count
					buffer.extend('r')
					smart_eiffel.se_atexit_id.append_in(buffer)
					buffer.append(once "se_atexit(")
					if not ace.boost then
						buffer.append(once "&ds")
					end
					if ace.profile then
						if buffer.last /= '(' then
							buffer.extend(',')
						end
						buffer.append(once "&local_profile")
					end
					if not ace.boost then
						if buffer.last /= '(' then
							buffer.extend(',')
						end
						buffer.append(once "NULL/*Unused Target*/")
					end
					buffer.append(once ");%N")
					out_c.put_string(buffer)
				end
				if smart_eiffel.scoop then
					out_c.put_string(once "scoop_atexit();%N")
				end
				if ace.profile then
					out_c.put_string(once "stop_profile(parent_profile, &local_profile);%N")
				end
				if ace.profile then
					show_profile
				end
				out_c.put_string(once "}%N")
			end
			if (not ace.no_split) and then split_count > 10 then
				-- We are producing a quite large system, so just use a brand new file right-now:
				split_c_file_now
			end
			put_c_heading(once "void initialize_eiffel_runtime(int argc,char*argv[])")
			if smart_eiffel.scoop then
				out_c.put_string(once "init_scoop_thread();%N")
				out_c.put_string(once "se_init_separate_root(SE_SCOOP_THREAD_TYPE, %"<root>%");%N{%N")
			end
			if ace.profile then
				out_c.put_string(once "se_local_profile_t local_profile, *parent_profile;%N")
			end
			if no_check then
				if smart_eiffel.scoop then
					out_c.put_string(once "se_subsystem_t* self = se_current_subsystem_thread();%N")
				end
				out_c.put_string(once "[
                se_frame_descriptor irfd={"<runtime init>",0,0,"",1};
                se_dump_stack ds = {NULL,NULL,0,NULL,NULL};
                ds.fd=&irfd;

		     ]")
				rf3.c_set_dump_stack_top(once "&ds", fz_link)
			end
			out_c.put_string(once "se_argc=argc;%Nse_argv=argv;%N")
			if do_atexit then
				out_c.put_string(once "atexit(se_atexit);%N")
			end
			if ace.profile then
				out_c.put_string(once "parent_profile=&global_profile;%N")
				out_c.put_string(once "local_profile.profile=&runinit_profile;%N")
				out_c.put_string(once "start_profile(parent_profile, &local_profile);%N")
			end
			gc_handler.initialize_runtime
			exceptions_handler.initialize_runtime
			if no_check then
				initialize_path_table
			end
			if smart_eiffel.generator_used then
				initialize_generator
			end
			if smart_eiffel.generating_type_used then
				initialize_generating_type
			end
			if ace.profile then
				initialize_profile
			end
			if not exceptions_handler.used then
				if ace.no_check then
					out_c.put_string(once "[
               #ifdef SIGINT
               signal(SIGINT,se_signal_handler);
               #endif
               #ifdef SIGTERM
               signal(SIGTERM,se_signal_handler);
               #endif

				    ]")
				end
				out_c.put_string(once "[
               #ifdef SIGQUIT
               signal(SIGQUIT,se_signal_handler);
               #endif
               #ifdef SIGILL
               signal(SIGILL,se_signal_handler);
               #endif
               #ifdef SIGABRT
               signal(SIGABRT,se_signal_handler);
               #endif
               #ifdef SIGFPE
               signal(SIGFPE,se_signal_handler);
               #endif
               #ifdef SIGSEGV
               signal(SIGSEGV,se_signal_handler);
               #endif
               #ifdef SIGBUS
               signal(SIGBUS,se_signal_handler);
               #endif
               #ifdef SIGSYS
               signal(SIGSYS,se_signal_handler);
               #endif
               #ifdef SIGTRAP
               signal(SIGTRAP,se_signal_handler);
               #endif
               #ifdef SIGXCPU
               signal(SIGXCPU,se_signal_handler);
               #endif
               #ifdef SIGXFSZ
               signal(SIGXFSZ,se_signal_handler);
               #endif

				    ]")
			end
			manifest_string_pool.c_call_initialize
			once_routine_pool.c_code_for_precomputable_routines
			if ace.sedb then
				out_c.put_string(once "se_general_trace_switch=1;%N")
			end
			if not gc_handler.is_off then
				out_c.put_string(fz_48)
			end
			out_c.put_string(once "{%N")
			gc_handler.declare_allocate_n(lt)
			out_c.put_string(once "eiffel_root_object=n;%N}%N")
			if ace.no_check then
				rf3.c_set_dump_stack_top(fz_null, fz_unlink)
			end
			if ace.profile then
				out_c.put_string(once "stop_profile(parent_profile, &local_profile);%N")
			end
			out_c.put_string(once "}%N")
			if smart_eiffel.scoop then
				out_c.put_string(once "}%N")
			end
		ensure
			on_c
		end

	check_assertion_mode: STRING

	inspect_level: INTEGER

	inline_level: INTEGER

	check_id (type: TYPE; e: EXPRESSION; id: INTEGER) is
			--|*** VIRER id ???? ***
			-- Produce a C expression checking that `e' is not Void and
			-- that `e' is really of type `id'.
			-- The result of the C expression is the pointer to the
			-- corresponding Object.
		require
			e.resolve_in(type).is_reference
			id > 0
		do
			if ace.no_check then
				put_string(once "((T")
				put_integer(id)
				put_string(once "*)ci(")
				put_integer(id)
				put_character(',')
				e.compile_to_c(type)
				put_character(',')
				put_position(e.start_position)
				put_string(fz_13)
				check_id_call_count := check_id_call_count + 1
			else
				e.compile_to_c(type)
				direct_call_count := direct_call_count + 1
			end
		end

	tmp_string: STRING is
		once
			create Result.make(256)
		end

	need_invariant (type_of_target: TYPE): LIVE_TYPE is
			-- Give the corresponding LIVE_TYPE when `type_of_target' actually needs some class invariant checking.
		require
			type_of_target /= Void
		do
			if type_of_target.class_text.invariant_check and then not invariant_checks_disabled then
				Result := type_of_target.live_type
				if Result /= Void and then Result.at_run_time and then Result.class_invariant /= Void then
					-- Let `Result'unchanged.
				else
					Result := Void
				end
			end
		end

	split_count: INTEGER
			-- Number of *.c files.

	path_h: STRING is
		once
			Result := system_tools.path_h
		ensure
			Result.has_suffix(h_suffix)
		end

	path_c: STRING is
		once
			if ace.no_split then
				Result := path_h.twin
				Result.remove_the_last
				Result.extend('c')
			else
				split_count := 1
				create Result.make(path_h.count + 4)
				path_c_in(Result, split_count)
			end
		ensure
			Result.has_suffix(c_suffix) or else Result.has_suffix(once ".d")
		end

	path_make: STRING is
		once
			Result := path_h.twin
			Result.remove_tail(2)
			Result.append(system_tools.make_suffix)
		ensure
			Result.has_suffix(system_tools.make_suffix)
		end

	add_first_include (the_first_include: STRING) is
		do
			put_banner(out_c)
			add_include_on(out_c, the_first_include)
		end

	put_banner (output: TEXT_FILE_WRITE) is
		require
			output.is_connected
		do
			output.put_string(once "/*%NANSI C code generated by ")
			output.put_string(smart_eiffel.copyright)
			output.put_string(once "C Compiler options used: ")
			output.put_string(system_tools.c_compiler_options)
			output.put_string(once "%N*/%N")
		end

	c_code_saved: BOOLEAN

	get_inline_ms: MANIFEST_STRING is
		local
			e: EXPRESSION
		do
			e := stack_top.effective_arguments.expression(1)
			if e.is_manifest_string then
				Result ?= e
			end
			if Result = Void then
				--|*** PH(18/04/04) This validity test has to be made in "specialize_in"
				--|*** Yes, but we need the actual argument too... (Dom may 28th 2004) ***
				error_handler.add_position(e.start_position)
				fatal_error("Bad usage of C inlining (argument is not a manifest string).")
			end
		end

	h_connect (h_path: STRING) is
		do
			if out_h = Void then
            create {TEXT_FILE_WRITE} out_h.make
			end
			out_h.connect_to(h_path)
		end

	c_connect (c_path: STRING) is
		do
			if out_c = Void then
            create {TEXT_FILE_WRITE} out_c.make
			end
			out_c.connect_to(c_path)
			if out_c.is_connected then
				echo.put_string(once "Writing %"")
				echo.put_string(c_path)
				echo.put_string(once "%" file.%N")
			else
				echo.w_put_string(once "Cannot write file %"")
				echo.w_put_string(c_path)
				echo.w_put_string(fz_b0)
				die_with_code(exit_failure_code)
			end
		ensure
			out_c.is_connected
		end

	backup_tfw_connect (c_path: STRING) is
		require
			not ace.no_split
		do
			tmp_path.copy(c_path)
			tmp_path.remove_tail(2)
			tmp_path.append(system_tools.object_suffix)
			if (create {FILE_TOOLS}).is_readable(tmp_path) then
				if (create {FILE_TOOLS}).is_readable(c_path) then
					c_path.put('d', c_path.count)
				else
					echo.file_removing(tmp_path)
				end
			end
			c_connect(c_path)
		end

	path_c_in (str: STRING; number: INTEGER) is
		do
			str.copy(path_h)
			str.remove_tail(2)
			number.append_in(str)
			str.extend('.')
			str.extend('c')
		end

	write_make_file_split is
		require
			not ace.no_split
		local
			i: INTEGER; no_change, recompile: BOOLEAN; executable_name, c_name: STRING; file_tools: FILE_TOOLS
		do
			if ace.precompile_header then
				echo_make(system_tools.split_mode_h_precompiler_command(path_h))
			elseif system_tools.precompiled_header_suffix /= Void then
				tmp_string.copy(path_h)
				tmp_string.remove_tail(2)
				tmp_string.append(system_tools.precompiled_header_suffix)
				echo.file_removing(tmp_string)
			end
			from
				no_change := True
				i := split_count
			until
				i = 0
			loop
				recompile := True
				path_c_in(tmp_path, i)
				tmp_string.copy(tmp_path)
				tmp_string.put('d', tmp_string.count)
				if (create {FILE_TOOLS}).is_readable(tmp_string) then
					if file_tools.same_files(tmp_path, tmp_string) then
						echo.put_string(fz_01)
						echo.put_string(tmp_path)
						echo.put_string(once "%" not changed.%N")
						echo.file_removing(tmp_string)
						recompile := False
					else
						echo.file_renaming(tmp_string, tmp_path)
						tmp_path.remove_tail(2)
						tmp_path.append(system_tools.object_suffix)
						echo.file_removing(tmp_path)
					end
				end
				if recompile then
					c_name := once "...This is a local once buffer..."
					c_name.copy(path_h)
					c_name.remove_tail(2)
					i.append_in(c_name)
					c_name.extend('.')
					c_name.extend('c')
					echo_make(system_tools.split_mode_c_compiler_command(c_name))
					no_change := False
				end
				i := i - 1
			end
			executable_name := ace.executable_name
			if no_change and then executable_name /= Void then
				no_change := (create {FILE_TOOLS}).is_readable(executable_name)
			else
				no_change := False
			end
			if no_change and then not system_tools.is_linking_mandatory then
				executable_is_up_to_date := True
				echo.put_string(once "Executable is up-to-date %
			    %(no C compilation, no linking done).%N")
			else
				tmp_string.copy(path_h)
				tmp_string.remove_tail(2)
				echo_make(system_tools.split_mode_linker_command(tmp_string, split_count))
			end
		end

	executable_is_up_to_date: BOOLEAN
			-- When the executable seems to be already correct (no C
			-- compilation and no linking is to be done).

	write_make_file_no_split is
		require
			ace.no_split
		do
			echo_make(system_tools.no_split_mode_command(path_c))
		end

	common_put_target is
		local
			rf: RUN_FEATURE; e: EXPRESSION; ct: TYPE_MARK; c_name: STRING; unwrap_buffer: STRING
			mem_id, class_invariant_flag: INTEGER
		do
			inspect
				stack_top.code
			when C_inside_twin then
				rf := stack_top.run_feature
				ct := rf.current_type
				if ct.is_reference then
					out_c.put_character('(')
					ct.mapping_cast
					out_c.put_character('R')
					out_c.put_character(')')
				else
					out_c.put_character('&')
					out_c.put_character('R')
				end
			when C_create_instruction, C_create_expression then
				rf := stack_top.run_feature
				if rf.current_type.is_user_expanded then
					if not rf.current_type.is_dummy_expanded then
						out_c.put_character('&')
					end
				end
				out_c.put_character('n')
			when C_switch then
				rf := stack_top.run_feature
				class_invariant_flag := class_invariant_call_opening(rf.type_of_current, False)
				out_c.put_string(once "((T")
				out_c.put_integer(rf.id)
				out_c.put_string(once "*)C)")
				if class_invariant_flag > 0 then
					class_invariant_call_closing(class_invariant_flag, False)
				end
			when C_expanded_initialize then
				rf := stack_top.run_feature
				if not rf.current_type.is_dummy_expanded then
					out_c.put_character('&')
				end
				e := stack_top.target
				if e /= Void then
					e.compile_to_c(stack_top.type)
				else
					c_name := stack_top.string
					if c_name /= Void then
						out_c.put_string(c_name)
					else
						out_c.put_string(once "n->_")
						out_c.put_string(rf.name.to_string)
					end
				end
			when C_precursor then
				out_c.put_character('C')
			when C_scoop_wrapper then
				out_c.put_string(once "C->ref")
			when C_scoop_unwrapper then
				mem_id := stack_top.run_feature.id
				unwrap_buffer := once "                "
				unwrap_buffer.copy(once "((T")
				mem_id.append_in(unwrap_buffer)
				unwrap_buffer.append(once "*)((Tw")
				mem_id.append_in(unwrap_buffer)
				stack_top.run_feature.name.mapping_c_in(unwrap_buffer)
				unwrap_buffer.append(once "*)data)->C)")
				out_c.put_string(unwrap_buffer)
			when C_string_inspector then
				put_inspect
			when C_introspect then
				out_c.put_string(once "*C")
			end
		end

	put_file (tfr: TEXT_FILE_READ; output: TEXT_FILE_WRITE) is
		require
			not tfr.end_of_input
		do
			split_c_file_padding_here
			from
				tfr.read_character
			until
				tfr.end_of_input
			loop
				output.put_character(tfr.last_character)
				tfr.read_character
			end
			tfr.disconnect
		ensure
			not tfr.is_connected
		end

	really_define_c_main (rf3: RUN_FEATURE_3) is
		require
			not ace.no_main
		local
			id, class_invariant_flag: INTEGER; type: TYPE; lt: LIVE_TYPE; ct: TYPE_MARK
		do
			echo.put_string(once "Define C main function.%N")
			type := rf3.type_of_current
			ct := rf3.current_type
			lt := rf3.live_type
			id := lt.id
			system_tools.put_c_main_function_type(out_c)
			out_c.put_string(once " main(int argc,char*argv[]){%N")
			if ace.profile then
				out_c.put_string(once "se_local_profile_t local_profile, master_profile;%N")
				out_c.put_string(once "master_profile.profile=NULL;%N")
				out_c.put_string(once "init_profile(&root_profile, %"<root>%");%N")
				out_c.put_string(once "init_profile(&runinit_profile, %"<runinit>%");%N")
				out_c.put_string(once "init_profile(&atexit_profile, %"<atexit>%");%N")
				out_c.put_string(once "global_profile.profile=&root_profile;%N")
				out_c.put_string(once "get_profiler_started(&master_profile);%N")
				out_c.put_string(once "start_profile(&master_profile, &global_profile);%N")
			end
			if not gc_handler.is_off then
				out_c.put_string(once "stack_bottom=((void**)(void*)(&argc));%N")
			end
			out_c.put_string(once "initialize_eiffel_runtime(argc,argv);%N")
			if ace.profile then
				out_c.put_string(once "local_profile=global_profile;%N")
			end
			out_c.put_string(once "{T")
			out_c.put_integer(id)
			out_c.put_string(once "*n=eiffel_root_object;%N")
			if smart_eiffel.scoop then
				out_c.put_string(once "se_subsystem_t* self;%N")
				out_c.put_string(once "for (self = &(se_thread_system->super); self->num!=0; self=self->next) ;%N")
			end
			if ace.no_check then
				out_c.put_string(once "se_frame_descriptor root={%"<system root>%",1,0,%"")
				tmp_string.clear_count
				ct.c_frame_descriptor_in(tmp_string)
				out_c.put_string(tmp_string)
				out_c.put_string(once "%",1};%N%
			       %se_dump_stack ds;%N%
			       %ds.fd=&root;%N%
			       %ds.current=((void**)(&n));%N")
				put_position_in_ds(rf3.start_position)
				out_c.put_string(once "ds.caller=NULL;%Nds.exception_origin=NULL;%Nds.locals=NULL;%N")
				rf3.c_set_dump_stack_top(once "&ds", fz_link)
			end
			expanded_attributes(type, ct)
			push_create_instruction(type, rf3, Void)
			if smart_eiffel.scoop then
				out_c.put_string(once "se_subsystem_t* self=se_current_subsystem_thread();%N")
			end
			rf3.mapping_c_root
			pop
			class_invariant_flag := class_invariant_call_opening(lt.type, True)
			if class_invariant_flag > 0 then
				out_c.put_character('n')
				class_invariant_call_closing(class_invariant_flag, True)
			end
			gc_handler.gc_info_before_exit
			if ace.no_check then
				rf3.c_set_dump_stack_top(once "NULL", fz_unlink)
			end
			if ace.profile then
				out_c.put_string(once "global_profile=local_profile;%N")
				out_c.put_string(once "stop_profile(&master_profile, &global_profile);%N")
			end
			system_tools.put_c_main_function_exit(out_c)
		end

	fz_base: STRING is "base"

	include_memory: SET[STRING]

	put_position_comment_on (output: TEXT_FILE_WRITE; p: POSITION) is
		local
			i: INTEGER; path: STRING; stop: BOOLEAN
		do
			if p.is_unknown then
				output.put_string(once "/*unknown position*/")
			else
				output.put_string(fz_open_c_comment)
				output.put_character('l')
				output.put_integer(p.line)
				output.put_character('c')
				output.put_integer(p.column)
				path := p.path
				from
					i := path.count
				until
					stop
				loop
					if i <= 1 then
						output.put_character(' ')
						stop := True
					else
						inspect
							path.item(i)
						when '/', '\' then
							stop := True
						else
							i := i - 1
						end
					end
				end
				from
				until
					i > path.count
				loop
					output.put_character(path.item(i))
					i := i + 1
				end
				output.put_string(fz_close_c_comment)
			end
		end

end -- class C_PRETTY_PRINTER
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
