-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
deferred class CODE_CONTEXT

inherit
	ANY
insert
	GLOBALS
	CODE_CONTEXT_CONSTANTS

feature {}
	run_feature_memory: RUN_FEATURE

	type_memory: TYPE

	target_memory: EXPRESSION

	effective_arguments_memory: EFFECTIVE_ARG_LIST

	formal_arguments_memory: FORMAL_ARG_LIST

	static_type_memory: TYPE

	static_run_feature_memory: RUN_FEATURE

	string_memory: STRING

feature {CODE_PRINTER} -- Consultation:
	valid: BOOLEAN

	code: INTEGER is
		deferred
		end

	reserve is
		require
			not_valid: validate
		do
			run_feature_memory := Void
			type_memory := Void
			target_memory := Void
			effective_arguments_memory := Void
			formal_arguments_memory := Void
			static_type_memory := Void
			static_run_feature_memory := Void
			string_memory := Void
		ensure
			valid
		end

	free is
		require
			valid: invalidate
		do
		ensure
			not valid
		end

	run_feature: like run_feature_memory is
		require
			valid and then valid_run_feature
		do
			Result := run_feature_memory
		end

	type: like type_memory is
		require
			valid
		do
			Result := type_memory
		end

	target: like target_memory is
		require
			valid and then valid_target
		do
			Result := target_memory
		end

	effective_arguments: like effective_arguments_memory is
		require
			valid and then valid_effective_arguments
		do
			Result := effective_arguments_memory
		end

	formal_arguments: like formal_arguments_memory is
		require
			valid and then valid_formal_arguments
		do
			Result := formal_arguments_memory
		end

	static_type: like static_type_memory is
		require
			valid and then valid_static_type
		do
			Result := static_type_memory
		end

	static_run_feature: like static_run_feature_memory is
		require
			valid and then valid_static_run_feature
		do
			Result := static_run_feature_memory
		end

	string: like string_memory is
		require
			valid and then valid_string
		do
			Result := string_memory
		end

feature {CODE_PRINTER} -- The good old "assertion-sets-its-own-flag" trick
	frozen validate: BOOLEAN is
		do
			Result := not valid
			valid := True
		end

	frozen invalidate: BOOLEAN is
		do
			Result := valid
			valid := False
		end

feature {CODE_PRINTER} -- Modification:
	set_run_feature (a_run_feature: like run_feature) is
		require
			valid and then valid_run_feature
			run_feature = Void
		do
			run_feature_memory := a_run_feature
		ensure
			run_feature = a_run_feature
		end

	set_type (a_type: like type) is
		require
			valid
			type = Void
		do
			type_memory := a_type
		ensure
			type = a_type
		end

	set_target (a_target: like target) is
		require
			valid and then valid_target
			target = Void
		do
			target_memory := a_target
		ensure
			target = a_target
		end

	set_effective_arguments (a_effective_arguments: like effective_arguments) is
		require
			valid and then valid_effective_arguments
			effective_arguments = Void
		do
			effective_arguments_memory := a_effective_arguments
		ensure
			effective_arguments = a_effective_arguments
		end

	set_formal_arguments (a_formal_arguments: like formal_arguments) is
		require
			valid and then valid_formal_arguments
			formal_arguments = Void
		do
			formal_arguments_memory := a_formal_arguments
		ensure
			formal_arguments = a_formal_arguments
		end

	set_static_type (a_static_type: like static_type) is
		require
			valid and then valid_static_type
			static_type = Void
		do
			static_type_memory := a_static_type
		ensure
			static_type = a_static_type
		end

	set_static_run_feature (a_static_run_feature: like static_run_feature) is
		require
			valid and then valid_static_run_feature
			static_run_feature = Void
		do
			static_run_feature_memory := a_static_run_feature
		ensure
			static_run_feature = a_static_run_feature
		end

	set_string (a_string: like string) is
		require
			valid and then valid_string
			string = Void
		do
			string_memory := a_string
		ensure
			string = a_string
		end

feature {CODE_PRINTER} -- Validation:
	valid_run_feature: BOOLEAN is
		deferred
		end

	valid_target: BOOLEAN is
		deferred
		end

	valid_effective_arguments: BOOLEAN is
		deferred
		end

	valid_formal_arguments: BOOLEAN is
		deferred
		end

	valid_static_type: BOOLEAN is
		deferred
		end

	valid_static_run_feature: BOOLEAN is
		deferred
		end

	valid_string: BOOLEAN is
		deferred
		end

invariant
	not valid_run_feature implies run_feature_memory = Void

	not valid_target implies target_memory = Void

	not valid_effective_arguments implies effective_arguments_memory = Void

	not valid_formal_arguments implies formal_arguments_memory = Void

	not valid_static_type implies static_type_memory = Void

	not valid_static_run_feature implies static_run_feature_memory = Void

	not valid_string implies string_memory = Void

end -- class CODE_CONTEXT
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
