-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class EIFFELDOC_COMMENT_WRITER
	--
	-- Produces a nicely formatted comment, using a very light wiki syntax.
	--
	-- '''Note:''' HTML tags are not supported, neither is the special <nowiki> tag.
	--
	-- See also EIFFELDOC_COMMENT_STATE and its children.
	--

inherit
	IN_OUT_VISITOR

insert
	GLOBALS

creation {EIFFELDOC_CONTEXT}
	make

feature {EIFFELDOC_CONTEXT}
	write (comment: COMMENT; for_feature: ANONYMOUS_FEATURE; summary: BOOLEAN) is
		local
			i: INTEGER; done: BOOLEAN; c: STRING
		do
			c := once ""
			from
				c.clear_count
				i := comment.list.lower
			until
				done or else i > comment.list.upper
			loop
				if i > comment.list.lower then
					c.extend('%N')
				end
				done := append_comment_line_in(comment.list.item(i), c, summary)
				i := i + 1
			end
			put_comment_line(c, for_feature)
		end

feature {}
	append_comment_line_in (line, buffer: STRING; summary: BOOLEAN): BOOLEAN is
		require
			buffer /= Void
		local
			i, s: INTEGER; c, p: CHARACTER
		do
			if summary then
				from
					s := 1
					i := 1
					p := '%U'
					comment_open_braces.clear_count
				until
					s = 0 or else i > line.count or else s = 3 and then i = line.count
				loop
					c := line.item(i)
					inspect
						s
					when 1 then
						inspect
							c
						when ' ', '%N', '%F', '%R', '%T' then
							buffer.extend(c)
							s := 2
						when '(', '[', '{', '<' then
							buffer.extend(c)
							if (once " %N%F%R%T([{<%"").has(p) then
								comment_open_braces.add_last(c)
							end
						when '%%' then
							s := 4
						when '"' then
							if comment_open_braces.count > 0 and then comment_open_braces.last = '"' then
								-- closing '"'
								buffer.extend(c)
								comment_open_braces.remove_last
							else
								-- opening '"'
								buffer.extend(c)
								if (once " %N%F%R%T([{<%"").has(p) then
									comment_open_braces.add_last(c)
								end
							end
						when '>' then
							if comment_open_braces.count > 0 and then comment_open_braces.last = '<' then
								comment_open_braces.remove_last
							end
							buffer.extend(c)
						when '}' then
							if comment_open_braces.count > 0 and then comment_open_braces.last = '<' then
								comment_open_braces.remove_last
							end
							buffer.extend(c)
						when ']' then
							if comment_open_braces.count > 0 and then comment_open_braces.last = '<' then
								comment_open_braces.remove_last
							end
							buffer.extend(c)
						when ')' then
							if comment_open_braces.count > 0 and then comment_open_braces.last = '<' then
								comment_open_braces.remove_last
							end
							buffer.extend(c)
						when '.', '!', '?' then
							buffer.extend(c)
							if not (once ".?!").has(p) then
								s := 3
							end
						else
							buffer.extend(c)
						end
					when 2 then
						inspect
							c
						when ' ', '%N', '%F', '%R', '%T' then
							buffer.extend(c)
						when '(', '[', '{', '<' then
							buffer.extend(c)
							comment_open_braces.add_last(c)
							s := 1
						when '%%' then
							s := 4
						when '"' then
							if comment_open_braces.count = 0 or else comment_open_braces.last /= '"' then
								comment_open_braces.add_last(c)
							end
							buffer.extend(c)
							s := 1
						else
							buffer.extend(c)
							s := 1
						end
					when 3 then
						inspect
							c
						when ' ', '%N', '%F', '%R', '%T' then
							if comment_open_braces.count = 0 then
								s := 0
							else
								buffer.extend(c)
							end
						when '%%' then
							s := 4
						when '"' then
							buffer.extend(c)
							if comment_open_braces.count > 0 and then comment_open_braces.last = '"' then
								comment_open_braces.remove_last
								if comment_open_braces.count = 0 then
									s := 0
								end
							else
								s := 1
							end
						when '>' then
							buffer.extend(c)
							if comment_open_braces.count > 0 and then comment_open_braces.last = '<' then
								comment_open_braces.remove_last
								if comment_open_braces.count = 0 then
									s := 0
								end
							else
								s := 1
							end
						when '}' then
							buffer.extend(c)
							if comment_open_braces.count > 0 and then comment_open_braces.last = '{' then
								comment_open_braces.remove_last
								if comment_open_braces.count = 0 then
									s := 0
								end
							else
								s := 1
							end
						when ']' then
							buffer.extend(c)
							if comment_open_braces.count > 0 and then comment_open_braces.last = '[' then
								comment_open_braces.remove_last
								if comment_open_braces.count = 0 then
									s := 0
								end
							else
								s := 1
							end
						when ')' then
							buffer.extend(c)
							if comment_open_braces.count > 0 and then comment_open_braces.last = '(' then
								comment_open_braces.remove_last
								if comment_open_braces.count = 0 then
									s := 0
								end
							else
								s := 1
							end
						when '.', '!', '?' then
							buffer.extend(c)
						else
							buffer.extend(c)
							s := 1
						end
					when 4 then
						inspect
							c
						when 'N' then
							buffer.extend('%N')
						when 'R' then
							buffer.extend('%R')
						when 'F' then
							buffer.extend('%F')
						when '%%' then
							buffer.extend('%%')
						when '.', '!', '?', '(', ')', '[', ']', '{', '}', '"' then
							-- allows to bypass the end-of-sentence checks
							buffer.extend(c)
						when 'U' then
							-- discarded
						else
							buffer.extend('%%')
							buffer.extend(c)
						end
						s := 1
					end
					p := c
					i := i + 1
				end
				if s = 3 or else s = 0 then
					Result := True
				end
			else
				buffer.append(line)
			end
		end

	comment_open_braces: FAST_ARRAY[CHARACTER] is
		once
			create Result.make(0)
		end

feature {}
	put_comment_line (line: STRING; for_feature: ANONYMOUS_FEATURE) is
		local
			offset, next_offset, i, lower: INTEGER; state, top_state: EIFFELDOC_COMMENT_STATE
			all_states: FAST_ARRAY[EIFFELDOC_COMMENT_STATE]
		do
			-- looks like the State design pattern, but in fact it is more like a Chain of Responsibility.

			all_states := context.all_states
			lower := all_states.lower
			from
				offset := context.beginning_of_line.handle(line, offset, for_feature, states)
			until
				offset > line.count
			loop
				from
					i := lower
					next_offset := offset
					if top_state /= Void and then top_state.handle_first and then top_state.can_handle(line, offset) then
						next_offset := top_state.handle(line, offset, for_feature, states)
						if next_offset /= offset and then not states.is_empty then
							top_state := states.top
						end
					end
				invariant
					next_offset = offset
				until
					next_offset > offset
				loop
					state := all_states.item(i)
					if (state /= top_state or else not top_state.handle_first) and then state.can_handle(line, offset) then
						next_offset := state.handle(line, offset, for_feature, states)
						if next_offset /= offset and then not states.is_empty then
							top_state := states.top
						end
					end
					i := i + 1
				end
				offset := next_offset
			end
			from
				-- The stack should be empty for correctly formatted text, but people are not always that nice.
				-- The wiki syntax must be kept flexible.
			variant
				states.count
			until
				states.is_empty
			loop
				if states.top.can_handle(line, offset) then
					offset := states.top.handle(line, offset, for_feature, states)
				else
					states.pop
				end
			end
		end

feature {}
	make (a_context: like context) is
		do
			context := a_context
			create states.make
		end

	context: EIFFELDOC_CONTEXT

	states: STACK[EIFFELDOC_COMMENT_STATE]

end -- class EIFFELDOC_COMMENT_WRITER
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2005: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
