-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class EIFFELDOC_OPTIONS
	-- Arguments that modify the behaviour of Eiffeldoc

insert
	ARGUMENTS
	GLOBALS

creation {EIFFELDOC}
	make

feature {ANY}
	title: STRING

	home_address: STRING

	home_image_address: STRING

	doc_css: STRING

	nav_css: STRING

	class_css: STRING

	arrow_address: STRING

	separator_address: STRING

	depends: BOOLEAN

	wiki_prefix: STRING

feature {}
	real_nav_css: STRING is "eiffeldoc_nav.css"

	real_doc_css: STRING is "eiffeldoc_doc.css"

	real_class_css: STRING is "eiffeldoc_class.css"

feature {EIFFELDOC}
	navbar_html (filename: STRING): HTML_OUTPUT_STREAM is
		require
			filename /= Void
		local
			f: TEXT_FILE_WRITE
		do
			create f.connect_to(filename)
			create Result.connect_to(f)
			Result.header
			Result.put_stylesheet(real_nav_css)
			Result.body
		ensure
			Result.in_a_body
		end

	generate_stylesheets is
		do
			generate_css(doc_css, real_doc_css)
			generate_css(nav_css, real_nav_css)
			generate_css(class_css, real_class_css)
		end

feature {EIFFELDOC, EIFFELDOC_SHORTER, EIFFELDOC_SHORTER_SOURCEDOC}
	doc_html (filename: STRING): HTML_OUTPUT_STREAM is
		require
			filename /= Void
		local
			f: TEXT_FILE_WRITE
		do
			create f.connect_to(filename)
			create Result.connect_to(f)
			Result.header
			Result.put_stylesheet(real_doc_css)
		ensure
			Result.in_header
		end

feature {EIFFELDOC_SHORTER_CLASSDOC}
	class_html (filename: STRING): HTML_OUTPUT_STREAM is
		require
			filename /= Void
		local
			f: TEXT_FILE_WRITE
		do
			create f.connect_to(filename)
			create Result.connect_to(f)
			Result.header
			Result.put_stylesheet(real_class_css)
		ensure
			Result.in_header
		end

feature {EIFFELDOC}
	parse_argument (i: INTEGER): INTEGER is
			-- Returns the number of arguments it used (0 if the argument was not matched)
		require
			i.in_range(1, argument_count)
		local
			arg: STRING
		do
			arg := argument(i)
			if eiffeldoc.flag_match(once "title", arg) then
				if i + 1 <= argument_count then
					title := argument(i + 1)
					Result := 2
				end
			elseif eiffeldoc.flag_match(once "home_address", arg) then
				if i + 1 <= argument_count then
					home_address := argument(i + 1)
					Result := 2
				end
			elseif eiffeldoc.flag_match(once "home_image_address", arg) then
				if i + 1 <= argument_count then
					home_image_address := argument(i + 1)
					Result := 2
				end
			elseif eiffeldoc.flag_match(once "nav_css", arg) then
				if i + 1 <= argument_count then
					nav_css := argument(i + 1)
					Result := 2
				end
			elseif eiffeldoc.flag_match(once "doc_css", arg) then
				if i + 1 <= argument_count then
					doc_css := argument(i + 1)
					Result := 2
				end
			elseif eiffeldoc.flag_match(once "class_css", arg) then
				if i + 1 <= argument_count then
					class_css := argument(i + 1)
					Result := 2
				end
			elseif eiffeldoc.flag_match(once "arrow_address", arg) then
				if i + 1 <= argument_count then
					arrow_address := argument(i + 1)
					Result := 2
				end
			elseif eiffeldoc.flag_match(once "separator_address", arg) then
				if i + 1 <= argument_count then
					separator_address := argument(i + 1)
					Result := 2
				end
			elseif eiffeldoc.flag_match(once "wiki_prefix", arg) then
				if i + 1 <= argument_count then
					wiki_prefix := argument(i + 1)
					Result := 2
				end
			elseif eiffeldoc.flag_match(once "depends", arg) then
				depends := True
				Result := 1
			end
		end

	set_default_arguments is
		do
			if title = Void then
				title := once "Generated Documentation"
			end
		end

	command_usage: STRING is
		do
			Result := once "[
								 {-title title} {-home_address address}
								 {-nav_css file} {-doc_css file} {-class_css file}
								 {-arrow_address address} {-separator_address address}
								 {-depends}

								 ]"
		end

	command_help: STRING is
		do
			Result := once "[
								 -title              the title of the generated documentation

								 -home_address       address of the 'go home' upper-left link

								 -home_image_address address of the 'go home' image

								 -nav_css            the stylesheet to use in the navigation bar
								                     (i.e. the left frames)

								 -doc_css            the stylesheet to use in the cluster documentation
								                     (i.e. on the right frames)

								 -class_css          the stylesheet to use in the class documentation
								                     (i.e. on the right frames)

								 -arrow_address      address of the arrow image

								 -separator_address  address of the separator image

								 -wiki_prefix        prefix of generated wiki URLs (the wiki word is added
								                     as a suffix to make the whole URL)

								 -depends            generate dependant classes even if their cluster is pruned


								 If -home_address is not given, the 'go home' upper link won't be generated.

								 If the -arrow_address is not given, links in the left navigation bar won't
								 have an arrow.

								 If the -separator_address is not given, there will be no separation before the
								 description of a cluster.

								 About the stylesheets: the used classes are

								   - A.home         for the home link
								   - A.title        for the titles
								   - A.cluster      for the clusters
								   - A.class        for the classes
								   - A.client       for the upper-right clients frame
								   - A.description  for the one-sentence description in the cluster summary
								   - TD.home        for the home link
								   - TD.title       for the titles
								   - TD.cluster     for the clusters
								   - TD.class       for the classes (except in the short proper)
								   - TD.client      for the upper-right clients frame
								   - TD.description for the one-sentence description in the cluster summary
								   - B.obsolete     for the obsolete header
								   - I.obsolete     for the obsolete message
								   - TD.comment     for the comment of a class
								   - PRE.comment    for the comment of a class
								   - B.comment      for the comment of a feature clause
								   - I.comment      for the comment of a feature
								   - LI.summary     for a feature in the summary
								   - A.summary      for the feature link in the summary
								   - TD.details     for a feature definition in the details section
								   - B.details      for the feature name in the details section
								   - B.assertion    for the assertion tags
								   - TT.assertion   for the assertion expressions
								   - I.assertion    for the assertion comments
								   - A.link         for a class link in the class documentation
								   - U.field        for a field link in a comment, not linked
								   - A.field        for a field link in a comment, linked
								   - TT.character   for a character in a comment
								   - TT.string      for a string in a comment

								 ]"
		end

feature {}
	generate_css (css, real_css: STRING) is
		require
			real_css /= Void
		local
			ft: FILE_TOOLS
		do
			if css = Void or else not ft.is_readable(css) then
				generate_default_css(real_css)
			else
				copy_file(css, real_css)
			end
		end

	copy_file (from_file, to_file: STRING) is
		local
			source: TEXT_FILE_READ; target: TEXT_FILE_WRITE
		do
			echo.put_string(once "Copying ")
			echo.put_string(from_file)
			echo.put_string(once " to ")
			echo.put_string(to_file)
			echo.put_new_line
			from
				create source.connect_to(from_file)
				create target.connect_to(to_file)
				source.read_line
			until
				source.end_of_input
			loop
				target.put_string(source.last_string)
				target.put_new_line
				source.read_line
			end
			source.disconnect
			target.disconnect
		end

	generate_default_css (real_css: STRING) is
		local
			f: TEXT_FILE_WRITE
		do
			create f.connect_to(real_css)
			f.put_string(once "A.cluster {%N")
			f.put_string(once "	  font-size: 12pt;%N")
			f.put_string(once "	  color: black%N")
			f.put_string(once "}%N")
			f.put_string(once "A.home {%N")
			f.put_string(once "	  font-size: 12pt;%N")
			f.put_string(once "	  color: blue%N")
			f.put_string(once "}%N")
			f.put_string(once "A.class {%N")
			f.put_string(once "	  font-size: 12pt;%N")
			f.put_string(once "	  color: red%N")
			f.put_string(once "}%N")
			f.put_string(once "A.title {%N")
			f.put_string(once "	  font-size: 16pt;%N")
			f.put_string(once "	  color: blue%N")
			f.put_string(once "}%N")
			f.flush
			f.disconnect
		end

feature {}
	make (e: EIFFELDOC) is
		do
			eiffeldoc := e
		end

	eiffeldoc: EIFFELDOC

end -- class EIFFELDOC_OPTIONS
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2005: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
