-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class EIFFELDOC_SHORTER
	--
	-- Writes all the files concerning a given class
	--

insert
	GLOBALS

creation {EIFFELDOC_CORE}
	make

feature {EIFFELDOC_CORE}
	short_for (ct: CLASS_TEXT; inherit_children, insert_children: FAST_ARRAY[CLASS_TEXT]): STRING is
			-- The filename of the class file is returned
		local
			fn: STRING; i: INTEGER; client: CLASS_NAME
		do
			depends.clear_count
			if not smart_eiffel.no_file_for(ct.name.to_string) then
				echo.put_character('%T')
				echo.put_string(ct.name.to_string)
				echo.put_new_line
				clients := find_clients.clients_of(ct)
				-- Now iterate over all the known clients
				from
					i := 1
				until
					i > clients.count
				loop
					client := clients.item(i)
					fn := filename_of(Void, ct, client)
					classdoc.generate(ct, client, fn, inherit_children, insert_children)
					depends.union(classdoc.depends)
					-- ... and on to the next client
					i := i + 1
				end
				fn := write_clients_file(ct)
				Result := write_class_file(ct, fn)
				--sourcedoc.generate(ct, filename_of(once "source", ct, Void))
			end
		end

	depends: HASHED_SET[CLASS_NAME] is
		once
			create Result.make
		end

feature {}
	write_clients_file (ct: CLASS_TEXT): STRING is
		local
			html: HTML_OUTPUT_STREAM; cn: CLASS_NAME; i: INTEGER
		do
			Result := once ""
			Result.copy(filename_of(once "client", ct, Void))
			html := options.doc_html(Result)
			html.body
			html.with_attribute(once "border", once "0")
			html.with_attribute(once "width", once "100%%")
			html.open_table
			html.open_table_row
			from
				i := 1
			until
				i > clients.count
			loop
				cn := clients.item(i)
				if i > 1 and then (i - 1) \\ 4 = 0 then
					html.close_table_row
					html.open_table_row
				end
				html.with_attribute(once "align", once "center")
				html.with_attribute(once "class", once "client")
				html.open_table_cell
				html.with_attribute(once "class", once "client")
				html.open_anchor_address(filename_of(Void, ct, cn), once "class")
				if cn.to_string = as_none then
					html.put_string(once "All")
					html.put_entity(once "nbsp")
					html.put_string(once "features")
				else
					html.put_string(cn.to_string)
				end
				html.close_anchor
				html.close_table_cell
				i := i + 1
			end
			html.close_table_row
			html.close_table
			html.close
			html.disconnect
		end

	write_class_file (ct: CLASS_TEXT; client_filename: STRING): STRING is
		local
			html: HTML_OUTPUT_STREAM
		do
			Result := once ""
			Result.copy(filename_of(once "class", ct, Void))
			html := options.doc_html(Result)
			html.frames
			inspect clients.count
			when 1..4 then
				html.with_attribute(once "rows", once "35px,*")
			when 5..8 then
				html.with_attribute(once "rows", once "70px,*")
			else
				html.with_attribute(once "rows", once "105px,*")
			end
			html.with_attribute(once "border", once "0")
			html.with_attribute(once "frameborder", "no")
			html.open_frameset
			html.with_attribute(once "scrolling", once "auto")
			html.put_frame(client_filename, once "clients")
			html.put_frame(filename_of(Void, ct, Void), once "class")
			html.close_frameset
			html.open_noframes
			html.put_string(once "This document is designed to be viewed exclusively using the frames feature.")
			html.close_noframes
			html.close
			html.disconnect
		end

feature {}
	make (a_context: EIFFELDOC_CONTEXT) is
		do
			options := a_context.options
			create find_clients.make
			create classdoc.make(a_context)
			create sourcedoc.make(options)
		end

	find_clients: EIFFELDOC_SHORTER_FIND_CLIENTS

	classdoc: EIFFELDOC_SHORTER_CLASSDOC

	sourcedoc: EIFFELDOC_SHORTER_SOURCEDOC

	options: EIFFELDOC_OPTIONS

	clients: CLASS_NAME_LIST

feature {}
	filename_of (a_prefix: STRING; a_class: CLASS_TEXT; a_client: CLASS_NAME): STRING is
		require
			a_class /= Void
		do
			Result := once ""
			if a_prefix = Void then
				Result.copy(a_class.name.to_string)
			else
				Result.copy(once "_")
				Result.append(a_prefix)
				Result.extend('@')
				Result.append(a_class.name.to_string)
			end
			if a_client /= Void and then not (a_client.to_string = as_any) then
				Result.extend('@')
				Result.append(a_client.to_string)
			end
			Result.append(once ".html")
		end

feature {ANY}
	command_name: STRING is ""

	command_line_help_summary: STRING is ""

feature {}
	valid_argument_for_ace_mode: STRING is ""

	parse_arguments is
		do
		end

	is_valid_argument_for_ace_mode (arg: STRING): BOOLEAN is
		do
			-- Always False
		end

end -- class EIFFELDOC_SHORTER
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2005: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
