-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class EIFFELDOC_SHORTER_CLASSDOC
	--
	-- Document a class for a given client
	--

inherit
	IN_OUT_VISITOR
		redefine enter_inherit_text, enter_class_text, enter_creation_clause_list, enter_creation_clause,
			exit_creation_clause, visit_feature_name, enter_feature_clause_list, enter_feature_clause,
			exit_feature_clause, enter_feature_text, visit_formal_generic_type_mark, visit_like_feature_type_mark,
			visit_like_argument_type_mark, visit_like_current_type_mark,
			visit_generic_type_mark, visit_agent_type_mark,
			visit_class_invariant
		end

insert
	GLOBALS
	ASSERTION_PRINT_TOOLS
	AGENT_TYPE_MARKS

creation {EIFFELDOC_SHORTER}
	make

feature {}
	invariant_phase:        INTEGER is 1
	summary_creation_phase: INTEGER is 2
	summary_features_phase: INTEGER is 3
	details_creation_phase: INTEGER is 4
	details_features_phase: INTEGER is 5

feature {EIFFELDOC_SHORTER}
	generate (a_class_text: CLASS_TEXT; a_client: like client; a_filename: STRING
		inherit_children, insert_children: FAST_ARRAY[CLASS_TEXT]) is
		local
			obs: MANIFEST_STRING; comment: COMMENT; client_class_text: CLASS_TEXT
		do
			context.set_class_text(a_class_text)
			context.set_client(a_client)
			context.set_html(options.class_html(a_filename))
			depends.clear_count
			html.body
			html.with_attribute(once "class", once "title")
			html.open_section
			if a_class_text.is_deferred then
				html.put_string(once "deferred ")
			elseif a_class_text.is_expanded then
				html.put_string(once "expanded ")
			elseif a_class_text.is_separate then
				html.put_string(once "separate ")
			end
			html.put_string(once "class ")
			put_class(a_class_text, True)
			html.close_section
			html.with_attribute(once "class", once "title")
			html.open_subsubsection
			if a_client.to_string = as_none then
				html.put_string(once "All features")
			else
				client_class_text := smart_eiffel.class_text(a_client, False)
				check
					client_class_text /= Void
				end
				html.put_string(once "Features exported to ")
				put_class(client_class_text, False)
			end
			html.close_subsubsection
			obs := a_class_text.obsolete_mark
			if obs /= Void then
				html.with_attribute(once "class", once "obsolete")
				html.open_bold
				html.put_string(once "This class is obsolete: ")
				html.close_bold
				html.with_attribute(once "class", once "obsolete")
				html.open_italics
				html.put_string(obs.to_string)
				html.close_italics
			else
				comment := a_class_text.heading_comment2
				if comment /= Void then
					html.open_table
					html.open_table_row
					html.with_attribute(once "class", once "comment")
					html.open_table_cell
					html.with_attribute(once "class", once "comment")
					context.write_comment(comment, False)
					html.close_table_cell
					html.close_table_row
					html.close_table
				end
				if a_class_text.inherit_text /= Void then
					html.with_attribute(once "class", once "title")
					html.open_subsection
					html.put_string(once "Direct parents")
					html.close_subsection
					if a_class_text.inherit_text.inherit_list /= Void then
						put_parent_classes(once "conformant parents", a_class_text.inherit_text.inherit_list)
					end
					if a_class_text.inherit_text.insert_list /= Void then
						put_parent_classes(once "non-conformant parents", a_class_text.inherit_text.insert_list)
					end
				end
				if inherit_children /= Void or else insert_children /= Void then
					html.with_attribute(once "class", once "title")
					html.open_subsection
					html.put_string(once "Known children")
					html.close_subsection
					if inherit_children /= Void then
						put_children_classes(once "conformant children", inherit_children)
					end
					if insert_children /= Void then
						put_children_classes(once "non-conformant children", insert_children)
					end
				end
				html.with_attribute(once "class", once "title")
				html.open_subsection
				html.put_string(once "Summary")
				html.close_subsection

				if a_class_text.creation_clause_list /= Void then
					html.with_attribute(once "class", once "title")
					html.open_subsubsection
					html.put_string(once "creation features")
					html.close_subsubsection
					phase := summary_creation_phase
					found_features.clear_count
					browse_class(a_class_text)
				end

				html.with_attribute(once "class", once "title")
				html.open_subsubsection
				html.put_string(once "exported features")
				html.close_subsubsection
				phase := summary_features_phase
				found_features.clear_count
				browse_class(a_class_text)

				html.with_attribute(once "class", once "title")
				html.open_subsection
				html.put_string(once "Details")
				html.close_subsection
				html.open_table

				if a_class_text.creation_clause_list /= Void then
					phase := details_creation_phase
					found_features.clear_count
					browse_class(a_class_text)
				end

				phase := details_features_phase
				found_features.clear_count
				browse_class(a_class_text)

				html.close_table

				put_class_invariant(a_class_text)
			end
			html.close
			html.disconnect
		end

	depends: HASHED_SET[CLASS_NAME] is
		once
			create Result.make
		end

feature {} -- Sorting and displaying lists of classes:
	put_parent_classes (title: STRING; parents: FAST_ARRAY[INHERIT_TEXT_ITEM]) is
		local
			i: INTEGER
		do
			list_of_classes.with_capacity(parents.count)
			from
				i := parents.lower
			until
				i > parents.upper
			loop
				list_of_classes.add_last(parents.item(i).class_text)
				i := i + 1
			end
			put_list_of_classes(title)
		end

	put_children_classes (title: STRING; children: FAST_ARRAY[CLASS_TEXT]) is
		require
			title /= Void
			children /= list_of_classes
			not children.is_empty
		do
			list_of_classes.with_capacity(children.count)
			list_of_classes.append_collection(children)
			put_list_of_classes(title)
		end

	put_list_of_classes (title: STRING) is
		require
			title /= Void
			not list_of_classes.is_empty
		local
			i: INTEGER
		do
			sort_list_of_classes
			html.with_attribute(once "class", once "title")
			html.open_subsubsection
			html.put_string(title)
			html.close_subsubsection
			html.open_paragraph
			from
				i := list_of_classes.lower
			until
				i > list_of_classes.upper
			loop
				if i > list_of_classes.lower then
					html.put_string(once ", ")
				end
				put_class(list_of_classes.item(i), False)
				i := i + 1
			end
			html.close_paragraph
		end

	sort_list_of_classes is
		local
			c: COMPARATOR_COLLECTION_SORTER[CLASS_TEXT]
		do
			c.set_comparator(class_texts_comparator)
			c.sort(list_of_classes)
		end

	compare_class_texts (x, y: CLASS_TEXT): BOOLEAN is
		do
			Result := x.name.to_string < y.name.to_string
		end

	class_texts_comparator: PREDICATE[TUPLE[CLASS_TEXT, CLASS_TEXT]] is
		once
			Result := agent compare_class_texts
		end

	list_of_classes: FAST_ARRAY[CLASS_TEXT] is
		once
			create Result.make(0)
		end

feature {}
	put_class_invariant (ct: CLASS_TEXT) is
		local
			phase_memory: like phase
		do
			phase_memory := phase
			phase := invariant_phase
			class_text := ct
			context.set_type(ct.declaration_type_of_like_current)

			if context.type.class_invariant /= Void then
				html.open_subsection
				html.put_string(once "Class invariant")
				html.close_subsection
				html.open_table
				html.open_table_row
				html.open_list
				print_assertion_list(context.type.class_invariant, Void)
				html.close_list
				html.close_table_row
				html.close_table
			end
			phase := phase_memory	
		end

	browse_class (ct: CLASS_TEXT) is
		local
			it: INHERIT_TEXT
		do
			check
				phase /= invariant_phase
			end
			class_text := ct
			context.set_type(ct.declaration_type_of_like_current)
			entered := False
			ct.accept(Current)
			it := ct.inherit_text
			if it /= Void then
				browse_inherit_items(it.inherit_list)
				browse_inherit_items(it.insert_list)
			end
			if ct /= class_any then
				browse_class(class_any)
			end
		end

	browse_inherit_items (items: FAST_ARRAY[INHERIT_TEXT_ITEM]) is
		local
			i: INTEGER; ct: CLASS_TEXT
		do
			if items /= Void then
				from
					i := items.lower
				until
					i > items.upper
				loop
					ct := items.item(i).class_text
					if ct /= class_any then
						browse_class(ct)
					end
					i := i + 1
				end
			end
		end

feature {CLASS_INVARIANT}
	visit_class_invariant (visited: CLASS_INVARIANT) is
		do
			if phase = invariant_phase then
				print_assertion_list(visited, Void)
			end
		end

feature {}
	enter_inherit_text (visited: INHERIT_TEXT): BOOLEAN is
		do
		ensure
			not Result
		end

	enter_class_text (visited: CLASS_TEXT): BOOLEAN is
		do
			Result := not entered
			entered := True
		end

	entered: BOOLEAN

	look_for_creation: FEATURE_NAME

	feature_clause_comment_to_print: COMMENT

	feature_clause_comment_printed: BOOLEAN

	enter_creation_clause_list (visited: CREATION_CLAUSE_LIST): BOOLEAN is
		do
			inspect
				phase
			when invariant_phase then
				check False end
			when summary_creation_phase then
				Result := True
			when details_creation_phase then
				Result := True
			when summary_features_phase then
			when details_features_phase then
			end
		end

	enter_creation_clause (visited: CREATION_CLAUSE): BOOLEAN is
		do
			Result := visited.clients.gives_permission_to(client)
			if Result then
				inspect
					phase
				when invariant_phase then
					check False end
				when summary_creation_phase then
					feature_clause_comment_to_print := visited.comment
					feature_clause_comment_printed := False
				when summary_features_phase then
				when details_creation_phase then
				when details_features_phase then
				end
			end
		end

	exit_creation_clause (visited: CREATION_CLAUSE) is
		do
			inspect
				phase
			when invariant_phase then
				check False end
			when summary_creation_phase then
				if feature_clause_comment_printed then
					html.close_list
				end
			when summary_features_phase then
			when details_creation_phase then
			when details_features_phase then
			end
		end

	enter_feature_clause_list (visited: FEATURE_CLAUSE_LIST): BOOLEAN is
		do
			inspect
				phase
			when invariant_phase then
				check False end
			when summary_creation_phase then
				Result := look_for_creation /= Void
			when summary_features_phase then
				Result := True
			when details_creation_phase then
				Result := look_for_creation /= Void
			when details_features_phase then
				Result := True
			end
		end

	enter_feature_clause (visited: FEATURE_CLAUSE): BOOLEAN is
		do
			inspect
				phase
			when invariant_phase then
				check False end
			when summary_creation_phase then
				Result := True
				check
					look_for_creation /= Void
				end
			when summary_features_phase then
				Result := visited.clients /= Void and then visited.clients.gives_permission_to(client)
				feature_clause_comment_to_print := visited.comment
				feature_clause_comment_printed := False
			when details_creation_phase then
				Result := True
				check
					look_for_creation /= Void
				end
			when details_features_phase then
				Result := visited.clients /= Void and then visited.clients.gives_permission_to(client)
			end
		end

	exit_feature_clause (visited: FEATURE_CLAUSE) is
		do
			inspect
				phase
			when invariant_phase then
				check False end
			when summary_creation_phase then
			when summary_features_phase then
				if feature_clause_comment_printed then
					html.close_list
				end
			when details_creation_phase then
			when details_features_phase then
			end
		end

	enter_feature_text (visited: FEATURE_TEXT): BOOLEAN is
		local
			do_print: BOOLEAN; fs: FEATURE_STAMP; fn: FEATURE_NAME; af: ANONYMOUS_FEATURE; i: INTEGER
		do
			from
				i := 1
			until
				i > visited.names.count
			loop
				fn := visited.names.item(i)
				fs := context.type.lookup(fn)
				if fs = Void then
					echo.put_string(once "*** Unknown feature ")
					echo.put_string(fn.to_string)
					echo.put_string(once " for class ")
					echo.put_string(context_class_text.name.to_string)
					echo.put_new_line
				else
					fs := fs.resolve_dynamic_binding_for(context.type)
					fn := context.type.get_feature_name(fs)
					do_print := not found_features.has(fs) and then (fn.start_position.class_text /= class_any or else context_class_text = class_any)
					if do_print then
						inspect
							phase
						when invariant_phase then
							check False end
						when summary_creation_phase then
							do_print := visited.names.has(look_for_creation)
						when summary_features_phase then
							check
								do_print
							end
						when details_creation_phase then
							do_print := visited.names.has(look_for_creation)
						when details_features_phase then
							check
								do_print
							end
						end
						if do_print then
							found_features.add_last(fs)
							af := fs.anonymous_feature(context.type)
							print_feature(fn, af)
						end
					end
				end
				i := i + 1
			end
		ensure
			not Result
		end

	found_features: FAST_ARRAY[FEATURE_STAMP] is
		once
			create Result.with_capacity(16)
		end

	print_feature (fn: FEATURE_NAME; af: ANONYMOUS_FEATURE) is
		do
			inspect
				phase
			when invariant_phase then
				check False end
			when summary_creation_phase, summary_features_phase then
				if not feature_clause_comment_printed then
					if feature_clause_comment_to_print /= Void then
						html.with_attribute(once "class", once "comment")
						html.open_bold
						context.write_comment(feature_clause_comment_to_print, False)
						html.close_bold
						feature_clause_comment_to_print := Void
					end
					feature_clause_comment_printed := True
					html.open_list
				end
				print_feature_summary(fn, af)
			when details_creation_phase, details_features_phase then
				print_feature_details(fn, af)
			end
		end

	print_feature_summary (fn: FEATURE_NAME; af: ANONYMOUS_FEATURE) is
		local
			ref: STRING; i: INTEGER
		do
			check
				phase = summary_features_phase implies af.permissions.gives_permission_to(client)
			end
			html.with_attribute(once "class", once "summary")
			html.open_list_item
			ref := once ""
			ref.copy(once "#")
			ref.append(fn.to_string)
			html.with_attribute(once "class", once "summary")
			html.open_anchor_address(ref, Void)
			html.with_attribute(once "class", once "summary")
			html.open_bold
			html.put_string(fn.to_string)
			html.close_bold
			html.close_anchor
			if af.arguments /= Void then
				html.put_entity(once "nbsp")
				html.put_character('(')
				from
					i := 1
				until
					i > af.arguments.count
				loop
					if i > 1 then
						html.put_string(once ", ")
					end
					html.put_string(af.arguments.name(i).to_string)
					html.put_string(once ": ")
					put_type_mark(af.arguments.type_mark(i))
					i := i + 1
				end
				html.put_character(')')
			end
			if af.result_type /= Void then
				html.put_string(once ": ")
				put_type_mark(af.result_type)
			end
			if af.header_comment /= Void then
				html.put_break
				html.with_attribute(once "class", once "comment")
				html.open_italics
				context.write_feature_comment(af.header_comment, af, True)
				html.close_italics
			end
			html.close_list_item
		end

	print_feature_details (fn: FEATURE_NAME; af: ANONYMOUS_FEATURE) is
		local
			ref: STRING; i: INTEGER
		do
			check
				phase = details_features_phase implies af.permissions.gives_permission_to(client)
			end
			html.with_attribute(once "border", once "1")
			html.open_table_row
			html.with_attribute(once "class", once "title")
			html.open_table_cell
			ref := once ""
			ref.copy(fn.to_string)
			html.open_anchor_name(ref)
			if af.is_deferred then
				html.put_string(once "deferred ")
			elseif fn.is_frozen then
				html.put_string(once "frozen ")
			end
			html.with_attribute(once "class", once "details")
			html.open_bold
			html.put_string(fn.to_string)
			html.close_bold
			if af.arguments /= Void then
				html.put_entity(once "nbsp")
				html.put_character('(')
				from
					i := 1
				until
					i > af.arguments.count
				loop
					if i > 1 then
						html.put_string(once ", ")
					end
					html.put_string(af.arguments.name(i).to_string)
					html.put_string(once ": ")
					put_type_mark(af.arguments.type_mark(i))
					i := i + 1
				end
				html.put_character(')')
			end
			if af.result_type /= Void then
				html.put_string(once ": ")
				put_type_mark(af.result_type)
			end
			html.close_anchor
			html.close_table_cell
			html.close_table_row
			html.open_table_row
			html.with_attribute(once "class", once "details")
			html.open_table_cell
			if af.obsolete_mark /= Void then
				html.open_blockquote
				html.with_attribute(once "class", once "obsolete")
				html.open_bold
				html.put_string(once "This feature is obsolete: ")
				html.close_bold
				html.with_attribute(once "class", once "obsolete")
				html.open_italics
				html.put_string(af.obsolete_mark.to_string)
				html.close_italics
				html.close_blockquote
			else
				if af.header_comment /= Void then
					html.open_blockquote
					html.with_attribute(once "class", once "comment")
					html.open_italics
					context.write_feature_comment(af.header_comment, af, False)
					html.close_italics
					html.close_blockquote
				end
				if af.require_assertion /= Void then
					html.open_paragraph
					html.with_attribute(once "class", once "assertion")
					html.open_bold
					html.put_string(once "require")
					html.close_bold
					html.open_list
					print_assertions(af.require_assertion.list, af)
					html.close_list
					html.close_paragraph
				end
				if af.ensure_assertion /= Void then
					html.open_paragraph
					html.with_attribute(once "class", once "assertion")
					html.open_bold
					html.put_string(once "ensure")
					html.close_bold
					html.open_list
					print_assertion_list(af.ensure_assertion, af)
					html.close_list
					html.close_paragraph
				end
			end
			html.close_table_cell
			html.close_table_row
		end

	print_assertions (assertions: FAST_ARRAY[ASSERTION_LIST]; for_feature: ANONYMOUS_FEATURE) is
		local
			i: INTEGER
		do
			from
				i := assertions.lower
			until
				i > assertions.upper
			loop
				print_assertion_list(assertions.item(i), for_feature)
				i := i + 1
			end
		end

	print_assertion_list (al: ASSERTION_LIST; for_feature: ANONYMOUS_FEATURE) is
		local
			i: INTEGER
		do
			if al.list /= Void then
				from
					i := al.list.lower
				until
					i > al.list.upper
				loop
					print_assertion(al.list.item(i), for_feature)
					i := i + 1
				end
			end
		end

	print_assertion (a: ASSERTION; for_feature: ANONYMOUS_FEATURE) is
		local
			s: STRING; ewc: EXPRESSION_WITH_COMMENT; e: EXPRESSION; c: COMMENT
		do
			if not hidden_expression_detector.visit(a, context.type, client, False) then
				html.open_list_item
				if a.tag /= Void then
					html.with_attribute(once "class", once "assertion")
					html.open_bold
					html.put_string(a.tag.to_string)
					html.put_string(once ": ")
					html.close_bold
				end
				e := a.expression
				c := a.comment
				if e /= Void then
					if ewc ?:= e then
						ewc ::= e
						check
							c = Void
						end
						e := ewc.expression
						c := ewc.comment
					end
					s := once ""
					s.clear_count
					pretty_printer.expression_in(s, e)
					html.with_attribute(once "class", once "assertion")
					html.open_typeset
					html.put_string(s)
					html.close_typeset
				end
				if c /= Void then
					html.with_attribute(once "class", once "assertion")
					html.open_italics
					context.write_feature_comment(c, for_feature, False)
					html.close_italics
				end
				html.close_list_item
			end
		end

feature {}
	visit_generic_type_mark (visited: GENERIC_TYPE_MARK) is
		local
			list: ARRAY[TYPE_MARK]; i: INTEGER
		do
			put_class(visited.resolve_in(context.type).class_text, False)
			html.put_character('[')
			from
				list := visited.generic_list
				i := list.lower
			until
				i > list.upper
			loop
				put_type_mark(list.item(i))
				i := i + 1
			end
			html.put_character(']')
		end

feature {AGENT_TYPE_MARK}
	visit_agent_type_mark (visited: AGENT_TYPE_MARK) is
		do
			inspect
				visited.pretty_code
			when routine_code then
				put_class_name_string(as_routine, True)
			when procedure_code then
				put_class_name_string(as_procedure, True)
			when function_code then
				put_class_name_string(as_function, True)
			when predicate_code then
				put_class_name_string(as_predicate, True)
			end
			html.put_character('[')
			put_class_name_string(as_tuple, True)
			html.put_character('[')
			visited.written_open.accept(Current)
			if visited.pretty_code = function_code then
				html.put_string(once ", ")
				visited.result_type.accept(Current)
			end
			html.put_character(']')
			html.put_character(']')
		end

feature {FEATURE_NAME}
	visit_feature_name (visited: FEATURE_NAME) is
		do
			inspect
				phase
			when invariant_phase then
				check False end
			when summary_creation_phase, details_creation_phase then
				if class_text.feature_clause_list /= Void then
					look_for_creation := visited
					class_text.feature_clause_list.accept(Current)
					look_for_creation := Void
				end
			when summary_features_phase, details_features_phase then
			end
		end

feature {FORMAL_GENERIC_TYPE_MARK}
	visit_formal_generic_type_mark (visited: FORMAL_GENERIC_TYPE_MARK) is
		do
			html.put_string(visited.written_mark)
		end

feature {LIKE_FEATURE_TYPE_MARK}
	visit_like_feature_type_mark (visited: LIKE_FEATURE_TYPE_MARK) is
		local
			fs: FEATURE_STAMP; ct: TYPE
		do
			ct := context.type
			fs := ct.lookup(ct.registered_name(visited.like_what)).resolve_dynamic_binding_for(ct)
			put_type_mark(fs.anonymous_feature(ct).result_type)
		end

feature {LIKE_ARGUMENT_TYPE_MARK}
	visit_like_argument_type_mark (visited: LIKE_ARGUMENT_TYPE_MARK) is
		do
			put_type_mark(visited.declaration_type)
		end

feature {LIKE_CURRENT_TYPE_MARK}
	visit_like_current_type_mark (visited: LIKE_CURRENT_TYPE_MARK) is
		do
			put_class(context_class_text, True)
		end

feature {}
	class_text: CLASS_TEXT

	client: CLASS_NAME is
		do
			Result := context.client
		end

	phase: INTEGER

	options: EIFFELDOC_OPTIONS is
		do
			Result := context.options
		end

	html: HTML_OUTPUT_STREAM is
		do
			Result := context.html
		end

	context_class_text: CLASS_TEXT is
		do
			Result := context.class_text
		end

	make (a_context: like context) is
		require
			a_context /= Void
		do
			context := a_context
		end

	put_type_mark (visited: TYPE_MARK) is
		local
			list: ARRAY[TYPE_MARK]; i: INTEGER
		do
			if visited.is_static then
				put_class(visited.resolve_in(context.type).class_text, True)
				if visited.is_generic then
					html.put_character('[')
					from
						list := visited.generic_list
						i := list.lower
					until
						i > list.upper
					loop
						put_type_mark(list.item(i))
						i := i + 1
					end
					html.put_character(']')
				end
			else
				visited.accept(Current)
			end
		end

	put_class (ct: CLASS_TEXT; show_generic: BOOLEAN) is
		local
			class_name: CLASS_NAME; i: INTEGER; arg: FORMAL_GENERIC_ARG
		do
			class_name := ct.name
			put_class_name(class_name, ct /= context_class_text and then not is_generic_name(class_name))
			if show_generic and then ct.is_generic then
				html.put_entity(once "nbsp")
				html.put_character('[')
				from
					i := 1
				until
					i > ct.formal_generic_list.count
				loop
					if i > 1 then
						html.put_string(once ", ")
					end
					arg := ct.formal_generic_list.item(i)
					class_name := arg.name
					put_class_name(class_name, not is_generic_name(class_name))
					if arg.constraint /= Void then
						html.put_entity(once "nbsp")
						html.put_string(once "->")
						html.put_entity(once "nbsp")
						put_class(arg.constraint.to_static(context.type).class_text, True)
					end
					i := i + 1
				end
				html.put_character(']')
			end
			if not depends.has(class_name) then
				depends.add(class_name)
			end
		end

	put_class_name (class_name: CLASS_NAME; anchored: BOOLEAN) is
		do
			put_class_name_string(class_name.to_string, anchored)
		end

	put_class_name_string (class_name_string: STRING; anchored: BOOLEAN) is
		local
			address: STRING
		do
			if not anchored then
				html.put_string(class_name_string)
			else
				address := once ""
				address.copy(once "_class@")
				address.append(class_name_string)
				address.append(once ".html")
				html.with_attribute(once "class", once "link")
				html.open_anchor_address(address, once "doc")
				html.put_string(class_name_string)
				html.close_anchor
			end
		end

	is_generic_name (class_name: CLASS_NAME): BOOLEAN is
		local
			i: INTEGER
		do
			if context_class_text.is_generic then
				from
					i := 1
				until
					Result or else i > context_class_text.formal_generic_list.count
				loop
					Result := context_class_text.formal_generic_list.item(i).name.is_equal(class_name)
					i := i + 1
				end
			end
		end

	context: EIFFELDOC_CONTEXT

end -- class EIFFELDOC_SHORTER_CLASSDOC
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2005: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
