-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class FEATURE_STAMP
	-- Identifies a feature in a unique way, whatever the renamings
	-- and the redefinitions.

inherit
	HASHABLE
	VISITABLE
insert
	GLOBALS

creation {CLASS_TEXT, FEATURE_ACCUMULATOR}
	make

creation {FEATURE_ACCUMULATOR}
	make_ncp, make_merge

feature {}
	make is
		require
			smart_eiffel.status.is_collecting
		do
			hash_code := next_hash
			create {HASHED_DICTIONARY[ANONYMOUS_FEATURE, TYPE]} anonymous_features.with_capacity(5)
		end

	make_ncp (a_feature_stamp: FEATURE_STAMP) is
		do
			hash_code := next_hash
			create {HASHED_DICTIONARY[ANONYMOUS_FEATURE, TYPE]} anonymous_features.with_capacity(5)
			non_conforming_parent := a_feature_stamp
			non_conforming_parent.add_dynamic_link(Current)
			debug
				debug_info := once "[" + smart_eiffel.accumulating_type.name.to_string
				debug_info.append(once ": ncp ")
				debug_info.append(a_feature_stamp.debug_info)
				debug_info.append(once "]")
			end
		ensure
			non_conforming_parent = a_feature_stamp
		end

	make_merge (fs1, fs2: FEATURE_STAMP) is
		require
			fs1 /= Void
			fs2 /= Void
		do
			hash_code := next_hash
			create {HASHED_DICTIONARY[ANONYMOUS_FEATURE, TYPE]} anonymous_features.with_capacity(5)
			create merge_parents.with_capacity(2)
			merge_parents.add_last(fs1)
			merge_parents.add_last(fs2)
			fs1.add_dynamic_link(Current)
			fs2.add_dynamic_link(Current)
			debug
				debug_info := once "[" + smart_eiffel.accumulating_type.name.to_string
				debug_info.append(once ": ")
				debug_info.append(fs1.debug_info)
				debug_info.append(once " / ")
				debug_info.append(fs2.debug_info)
				debug_info.append(once "]")
			end
		ensure
			fs1.has_dynamic_link(Current)
			fs2.has_dynamic_link(Current)
		end

feature {FEATURE_STAMP}
	debug_info: STRING

feature {CODE, CALL_SUPPORT, LIVE_TYPE, CECIL_ENTRY, MANIFEST_STRING_POOL, CODE_PRINTER}
	has_run_feature_for (t: TYPE): BOOLEAN is
		do
			if run_features /= Void then
				Result := run_features.fast_has(t)
			end
		end

	run_feature_for (t: TYPE): RUN_FEATURE is
		require
			has_run_feature_for(t) --|*** fn = t.get_feature_name(Current)
		do
			--|*** Utile pour le debug ...
--			if not has_run_feature_for(t) then
--				error_handler.append("Feature ")
--				if anonymous_features.is_empty then
--					error_handler.append(" ??? ")
--				else
--					error_handler.append(anonymous_features.item(1).first_name.to_string)					
--				end
--				error_handler.append(" non collectee dans ")
--				error_handler.append(t.name.to_string)
--				error_handler.append("... ca va planter.")
--				error_handler.print_as_warning
--			end
			--|*** 
			Result := run_features.fast_reference_at(t)
		ensure
			Result /= Void
		end

feature {LIVE_TYPE}
	make_run_feature (t: TYPE; fn: FEATURE_NAME) is
		require
			has_anonymous_feature_for(t)
			not has_run_feature_for(t)
		do
			if run_features = Void then
				create {HASHED_DICTIONARY[RUN_FEATURE, TYPE]} run_features.with_capacity(anonymous_features.count)
			end
			check
				not run_features.fast_has(t)
			end
			run_features.add(anonymous_feature(t).brand_new_run_feature_for(t, fn, False), t)
		ensure
			run_feature_for(t) /= Void
		end

	simplify (t: TYPE): ANONYMOUS_FEATURE is
			-- MUST NOT BE CALLED BY ANYONE EXCEPT LIVE_TYPE (because
			-- LIVE_TYPE keeps memory of the link between FS and ANONYMOUS_FEATURE)
		local
			af: ANONYMOUS_FEATURE
		do
			af := anonymous_features.fast_at(t)
			Result := af.simplify(t)
			if Result /= af then
				anonymous_features.fast_put(Result, t)
			end
		end

	contextual_simplify (t: TYPE): ANONYMOUS_FEATURE is
			-- MUST NOT BE CALLED BY ANYONE EXCEPT LIVE_TYPE (because
			-- LIVE_TYPE keeps memory of the link between FS and ANONYMOUS_FEATURE)
		local
			af: ANONYMOUS_FEATURE
		do
			af := anonymous_features.fast_at(t)
			Result := af.contextual_simplify(t)
			if Result /= af then
				anonymous_features.fast_put(Result, t)
			end
		end

feature {ANY}
	has_anonymous_feature_for (a_type: TYPE): BOOLEAN is
		do
			Result := anonymous_features.fast_has(a_type)
		end

	hash_code: INTEGER

	accept (visitor: FEATURE_STAMP_VISITOR) is
		do
			visitor.visit_feature_stamp(Current)
		end

feature {FEATURE_ACCUMULATOR, FEATURE_STAMP}
	add_dynamic_link (a_fs: FEATURE_STAMP) is
		require
			a_fs /= Void
		do
			if dynamic_links = Void then
				create dynamic_links.with_capacity(4)
			end
			if not has_dynamic_link(a_fs) then
				dynamic_links.add_last(a_fs)
			end
		ensure
			has_dynamic_link(a_fs)
		end

	add_seeds(feature_seeds: SET[FEATURE_STAMP]) is
		require
			feature_seeds /= Void
		local
			i: INTEGER
		do
			if merge_parents = Void then
				feature_seeds.fast_add(Current)
			else
				from
					i := merge_parents.upper
				until
					i < merge_parents.lower
				loop
					merge_parents.item(i).add_seeds(feature_seeds)
					i := i - 1
				end
			end
		end

feature {FEATURE_ACCUMULATOR}
	has_merge(parent_fs: FEATURE_STAMP): BOOLEAN is
		do
			Result := merge_parents.fast_has(parent_fs)
		end

	add_merge_ncp (parent_fs: FEATURE_STAMP) is
		require
			parent_fs /= Void
			--|*** (PH 09/09/04) I think dynamic_links should not 
			--|register non conforming children!
		do
			--|*** check non_conforming_parent /= Void end
			parent_fs.add_dynamic_link(Current)
			debug
				debug_info.remove_the_last
				debug_info.append(once " / ncp ")
				debug_info.append(parent_fs.debug_info)
				debug_info.append(once "]")
			end
		end

	add_merge (parent_fs: FEATURE_STAMP) is
		require
			parent_fs /= Void
		do
			parent_fs.add_dynamic_link(Current)
			if merge_parents = Void then --|*** PR (05/11/04) can be Void?
				create merge_parents.with_capacity(2)
				merge_parents.add_last(parent_fs)
			elseif not merge_parents.fast_has(parent_fs) then
				merge_parents.add_last(parent_fs)
			end
			debug
				debug_info.remove_the_last
				debug_info.append(once " / ")
				debug_info.append(parent_fs.debug_info)
				debug_info.append(once "]")
			end
		end

	add_anonymous_feature (an_af: ANONYMOUS_FEATURE; a_type: TYPE) is
		require
			an_af /= Void
			a_type.has_feature(Current)
			not has_anonymous_feature_for(a_type)
		do
			anonymous_features.add(an_af, a_type)
			debug
				if debug_info = Void then
					debug_info := a_type.name.to_string.twin
					debug_info.extend('.')
					an_af.names.first.complete_name_in(debug_info)
				end
			end
		ensure
			has_anonymous_feature_for(a_type)
		end

	has_non_conforming_parent (other: like Current): BOOLEAN is
		require
			other /= Void
		local
			ncp: like Current
		do
			from
				ncp := non_conforming_parent
			until
				Result or else ncp = Void
			loop
				Result := ncp = other
				ncp := ncp.non_conforming_parent
			end
		end

feature {ANY}
	anonymous_feature (a_type: TYPE): ANONYMOUS_FEATURE is
		require
			a_type /= Void
			has_anonymous_feature_for(a_type)
		do
			Result := anonymous_features.fast_reference_at(a_type)
		ensure
			Result /= Void
		end

	resolve_dynamic_binding_for (a_type: TYPE): FEATURE_STAMP is
			-- This function find the good feature_stamp for the same feature as the current one but in another (child) type.
			-- This does not work on not ready types, but it is quicker than fs_for_building_type.
		require
			a_type /= Void
		local
			i: INTEGER
		do
			if has_anonymous_feature_for(a_type) then
				Result := Current
			else
				from
					i := dynamic_links.upper
				variant
					i + 1
				until
					i < dynamic_links.lower or else dynamic_links.item(i).has_anonymous_feature_for(a_type)
				loop
					i := i - 1
				end
				if i >= dynamic_links.lower then
					Result := dynamic_links.item(i)
				else
					from
						i := dynamic_links.upper
					variant
						i + 1
					until
						Result /= Void
					loop
						Result := dynamic_links.item(i).internal_resolve_binding(a_type)
						i := i - 1
					end
					dynamic_links.add_last(Result) -- This optimization code has no impact on the result (it's not dangerous to add this element to dynamic_links). Another way to optimize this function is to save the result in a dictionnary.
				end
			end
		ensure
			Result.has_anonymous_feature_for(a_type)
			Result = fs_for_building_type(a_type)
			-- It may fail if it has been called with a wrong type or if
			-- there is a bug in dynamic_links building.
		end

	fs_for_building_type (a_type: TYPE): FEATURE_STAMP is
			-- This function find the good feature_stamp for the same feature as the current one but in another (child) type.
			-- It works on not yet ready TYPEs but is slower than resolve_dynamic_binding_for.
		require
			a_type /= Void
		local
			i: INTEGER
		do
			if a_type.has_feature(Current) then
				Result := Current
			else
				from
					i := dynamic_links.upper
				variant
					i + 1
				until
					Result /= Void
				loop
					Result := dynamic_links.item(i).internal_fs_for_building_type(a_type)
					i := i - 1
				end
			end
		ensure
			a_type.has_feature(Result)
			-- It may fail if it has been called with a wrong type or if
			-- there is a bug in dynamic_links building.
		end

	specialize_thru (parent_type: TYPE; inherit_item: INHERIT_TEXT_ITEM; new_type: TYPE): like Current is
			-- Assuming that `Current' is already specialized in `parent_type', now specialize `Current' for the `new_type' knowing
			-- that this source code is inherited thru `inherit_item'. (See also `specialize_in' and `has_been_specialized' as
			-- well.)
		require
			parent_type.has_feature(Current)
			new_type /= parent_type
			new_type /= Void
			inherit_item /= Void
			new_type.direct_thru_step(parent_type, inherit_item)
		do
			if not inherit_item.is_non_conforming and then anonymous_features.fast_has(new_type) then
				Result := Current
			else
				Result := new_type.lookup(inherit_item.do_rename(parent_type.get_feature_name(Current)))
			end

			--|*** (PH 09/09/04)Folowing code is faster because there it avoids using 
			--get_feature_name as much as possible. BUT it does not work 
			--because non-conforming children are registred in dynamic 
			--links as well as conforming ones (merge case). I think non 
			--conforming children should be registred another way!
-- 			If inherit_item.is_non_conforming then
-- 				Result := new_type.lookup(inherit_item.do_rename(parent_type.get_feature_name(Current)))
-- 			else
-- 				if anonymous_features.fast_has(new_type) then
-- 					Result := Current
-- 				else
-- 					if dynamic_links /= Void then
-- 						from
-- 							i := dynamic_links.upper
-- 						until
-- 							i < dynamic_links.lower or else dynamic_links.item(i).has_anonymous_feature_for(new_type)
-- 						loop
-- 							i := i - 1
-- 						end
-- 						if i >= dynamic_links.lower then
-- 							Result := dynamic_links.item(i)
-- 						end
-- 					end
-- 					if Result = Void then
-- 						Result := new_type.lookup(inherit_item.do_rename(parent_type.get_feature_name(Current)))
-- 					end
-- 				end
-- 			end
		ensure
			new_type.has_feature(Result)
			Result = new_type.lookup(inherit_item.do_rename(parent_type.get_feature_name(Current)))
		end

feature {TYPE, LIVE_TYPE, CREATION_CLAUSE}
	anonymous_feature_if_exist (a_type: TYPE): ANONYMOUS_FEATURE is
		require
			a_type /= Void
		do
			Result := anonymous_features.fast_reference_at(a_type)
		end

feature {FEATURE_STAMP}
	has_dynamic_link (a_fs: FEATURE_STAMP): BOOLEAN is
		do
			Result := dynamic_links.fast_has(a_fs)
		end

	internal_resolve_binding (a_type: TYPE): FEATURE_STAMP is
		require
			a_type /= Void
		local
			i: INTEGER
		do
			if has_anonymous_feature_for(a_type) then
				Result := Current
			elseif dynamic_links /= Void then
				from
					i := dynamic_links.upper
				variant
					i + 1
				until
					Result /= Void or else i < dynamic_links.lower
				loop
					Result := dynamic_links.item(i).internal_resolve_binding(a_type)
					i := i - 1
				end
			end
		end

	internal_fs_for_building_type (a_type: TYPE): FEATURE_STAMP is
		require
			a_type /= Void
		local
			i: INTEGER
		do
			--|*** should a mark be used to improve speed?
			if a_type.has_feature(Current) then
				Result := Current
			elseif dynamic_links /= Void then
				from
					i := dynamic_links.upper
				variant
					i + 1
				until
					Result /= Void or else i < dynamic_links.lower
				loop
					Result := dynamic_links.item(i).internal_fs_for_building_type(a_type)
					i := i - 1
				end
			end
		end

feature {TYPE, FEATURE_STAMP_VISITOR}
	try_to_resolve_dynamic_binding_for (a_type: TYPE): FEATURE_STAMP is
			-- This function returns Void if this feature stamp has no feature in `a_type'.
		do
			Result := internal_resolve_binding(a_type)
		ensure
			Result /= Void implies Result = resolve_dynamic_binding_for(a_type)
		end

feature {TYPE}
	specialize_2 (t: TYPE) is
		require
			t.has_feature(Current)
			has_anonymous_feature_for(t)
		local
			af1, af2: ANONYMOUS_FEATURE
		do
			af1 := anonymous_features.fast_reference_at(t)
			af2 := af1.specialize_2(t)
			if af1 /= af2 then
				anonymous_features.fast_put(af2, t)
			end
		ensure
			has_anonymous_feature_for(t)
		end

feature {FEATURE_STAMP}
	non_conforming_parent: like Current

feature {}
	anonymous_features: DICTIONARY[ANONYMOUS_FEATURE, TYPE]

	dynamic_links: FAST_ARRAY[like Current]

	merge_parents: FAST_ARRAY[like Current] -- Conforming ones only

	run_features: DICTIONARY[RUN_FEATURE, TYPE]

	hash_counter: COUNTER is
		once
			create Result
		end

	next_hash: INTEGER is
		do
			hash_counter.increment
			Result := hash_counter.value
		end

invariant
	anonymous_features /= Void

end -- class FEATURE_STAMP
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
