-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
expanded class SWITCH
	--
	-- Set of tools (no attributes) to handle one switching site.
	--

inherit
	VISITABLE
insert
	GLOBALS

feature {SWITCH_COLLECTION}
	c_define (desc: SWITCH_SITE) is
			-- Define the switching C function for the function named `rf'. All
			-- the possible calls are the `features'.
		require
			cpp.on_c
		local
			boost: BOOLEAN; arguments: FORMAL_ARG_LIST; ct, rt: TYPE; fn: FEATURE_NAME
			current_type, result_type, arg_type: TYPE_MARK; i: INTEGER
		do
			boost := ace.boost
			ct := desc.current_type
			rt := desc.result_type
			fn := desc.feature_name
			arguments := desc.arguments
			current_type := ct.canonical_type_mark
			if rt /= Void then
				result_type := rt.canonical_type_mark
			end
			c_code.clear_count
			if result_type = Void then
				c_code.append(fz_void)
			else
				result_type.c_type_for_result_in(c_code)
			end
			c_code.extend(' ')
			c_code.extend('X')
			current_type.id.append_in(c_code)
			fn.mapping_c_in(c_code)
			c_code.extend('(')
			if ace.profile then
				c_code.append(once "se_local_profile_t*parent_profile,")
			end
			if boost then
				c_code.append(once "T0*C")
			else
				c_code.append(once "se_dump_stack*caller,se_position position,T0*C")
			end
			if arguments /= Void then
				from
					i := 1
				until
					i > arguments.count
				loop
					c_code.extend(',')
					arg_type := arguments.type_mark(i).to_static(ct)
					arg_type.c_type_for_argument_in(c_code)
					c_code.append(once " a")
					i.append_in(c_code)
					i := i + 1
				end
			end
			c_code.extend(')')
			cpp.put_c_heading(c_code)
			cpp.swap_on_c
			if not boost then
				c_code.copy(once "static se_frame_descriptor fd={%"Dynamic dispatch {")
				c_code.append(ct.name.to_string)
				c_code.append(once "}.")
				fn.mapping_c_in(c_code)
				c_code.append(once "%",0,0,%"%",1};%N")
				cpp.put_string(c_code)
			end
			if smart_eiffel.scoop and then current_type.is_separate then
				cpp.put_string(once "se_subsystem_t* self=se_current_subsystem_thread();%N")
			end
			if ace.profile then
				local_profile
			end
			if result_type /= Void then
				c_code.clear_count
				result_type.c_type_for_result_in(c_code)
				c_code.append(once " R")
				if not boost then
					c_code.extend('=')
					result_type.c_initialize_in(c_code)
				end
				c_code.append(fz_00)
				cpp.put_string(c_code)
			end
			if not boost then
				cpp.put_string(once "[
               se_dump_stack ds;
               ds.fd=&fd;
               ds.current=NULL;
               ds.p=(caller->p);
               ds.caller=caller;
               ds.exception_origin=NULL;
               ds.locals=NULL;
               
               ]")
			end
			cpp.put_string(once "{Tid id=")
			if boost then
				cpp.put_string(once "((T0*)C)->id;%N")
			else
				cpp.put_string(once "vc(C,position)->id;%N")
			end
			if ace.profile then
				start_profile(desc)
			end
			if not boost then
				cpp.c_set_dump_stack_top_for(ct, once "&ds", fz_link)
			end
			if ace.all_check then
				new_c_switch(desc)
			else
				c_dicho(desc, 1, desc.count)
			end
			cpp.put_string(fz_12)
			if ace.profile then
				stop_profile
			end
			if not boost then
				cpp.c_set_dump_stack_top_for(ct, fz_caller, fz_unlink)
			end
			if result_type /= Void then
				cpp.put_string(fz_15)
			else
				cpp.put_string(fz_12)
			end
		ensure
			cpp.on_c
		end

feature {C_PRETTY_PRINTER}
	put_arguments (up_ct, ct: TYPE; up_fal, fal: FORMAL_ARG_LIST) is
			-- Produce C code for arguments of `fal' used inside the switching
			-- C function which arguments were `up_fal'.
		require
			cpp.on_c
			fal.count = up_fal.count
		local
			i, up: INTEGER
		do
			from
				i := 1
				up := fal.count
			until
				i > up
			loop
				if i > 1 then
					cpp.put_character(',')
				end
				put_ith_argument(up_ct, ct, up_fal, fal, i)
				i := i + 1
			end
		ensure
			cpp.on_c
		end

	put_ith_argument (up_ct, ct: TYPE; up_fal, fal: FORMAL_ARG_LIST; i: INTEGER) is
			-- Produce C code for argument at index `i' of `fal' used inside the switching C function.
		require
			cpp.on_c
			fal.count = up_fal.count
			1 <= i
			i <= fal.count
		local
			ft: TYPE
		do
			--|*** at := up_fal.type_mark(i).resolve_in(up_ct) -- ***
			ft := fal.type_mark(i).resolve_in(ct)
			if ft.is_separate then
				not_yet_implemented --|*** (Dom oct 1st 2004) ***
			end
			cpp.put_character('a')
			cpp.put_integer(i)
		ensure
			cpp.on_c
		end

feature {C_PRETTY_PRINTER, SWITCH, AGENT_CREATION}
	name (up_rf: RUN_FEATURE): STRING is
		do
			c_code.clear_count
			c_code.extend('X')
			up_rf.live_type.id.append_in(c_code)
			up_rf.name.mapping_c_in(c_code)
			Result := c_code
		end

feature {ANY}
	jvm_descriptor (up_rf: RUN_FEATURE): STRING is
		local
			arguments: FORMAL_ARG_LIST; rt: TYPE_MARK
		do
			arguments := up_rf.arguments
			tmp_jvmd.clear_count
			tmp_jvmd.extend('(')
			tmp_jvmd.append(jvm_root_descriptor)
			if arguments /= Void then
				arguments.jvm_descriptor_in(tmp_jvmd)
			end
			rt := up_rf.result_type
			if rt = Void then
				tmp_jvmd.append(fz_19)
			else
				tmp_jvmd.extend(')')
				if rt.is_reference then
					tmp_jvmd.append(jvm_root_descriptor)
				else
					rt.jvm_descriptor_in(tmp_jvmd)
				end
			end
			Result := tmp_jvmd
		end

feature {ANY}
	idx_methodref (up_rf: RUN_FEATURE): INTEGER is
		require
			up_rf /= Void
		do
			Result := constant_pool.idx_methodref3(jvm_root_class, name(up_rf), jvm_descriptor(up_rf))
		end

feature {JVM}
	jvm_mapping (up_rf: RUN_FEATURE; type: TYPE; target: EXPRESSION; eal: EFFECTIVE_ARG_LIST) is
		require
			target /= Void
			up_rf /= Void
		local
			idx, stack_level: INTEGER; fal: FORMAL_ARG_LIST; switch: SWITCH
		do
			target.compile_to_jvm(type)
			stack_level := 1
			if eal /= Void then
				fal := up_rf.arguments
				stack_level := stack_level + eal.compile_to_jvm(type, fal)
			end
			if up_rf.result_type /= Void then
				stack_level := stack_level - up_rf.result_type.jvm_stack_space
			end
			idx := switch.idx_methodref(up_rf)
			code_attribute.opcode_invokestatic(idx, - stack_level)
		end

feature {ANY}
	accept (visitor: SWITCH_VISITOR) is
		do
			visitor.visit_switch(Current)
		end

feature {SWITCH_SITE}
	jvm_define (up_rf: RUN_FEATURE) is
		local
			rt: TYPE_MARK; run_time_set: RUN_TIME_SET
		do
			-- Define the Java switching static method for `up_rf'.
			method_info.start(9, name(up_rf), jvm_descriptor(up_rf))
			rt := up_rf.result_type
			run_time_set := up_rf.live_type.run_time_set
			jvm_switch(up_rf, run_time_set, rt)
			method_info.finish
		end

feature {}
	c_code: STRING is
		once
			create Result.make(256)
		end

	jvm_switch (up_rf: RUN_FEATURE; run_time_set: RUN_TIME_SET; rt: TYPE_MARK) is
			-- Produce Java sequential switch code.
		require
			rt /= Void implies rt.is_static
		local
			space, point, i: INTEGER; dyn_lt: LIVE_TYPE; dyn_rf: RUN_FEATURE; boost: BOOLEAN
		do
			from
				boost := ace.boost
				i := run_time_set.count
			until
				i = 0
			loop
				dyn_lt := run_time_set.item(i)
				dyn_rf := dyn_lt.dynamic_feature(up_rf)
				if i = 1 and then boost then
				else
					code_attribute.opcode_aload_0
					dyn_rf.live_type.opcode_instanceof
					point := code_attribute.opcode_ifeq
				end
				jvm.push_switch(dyn_rf, up_rf)
				dyn_rf.mapping_jvm
				jvm.pop
				if rt = Void then
					code_attribute.opcode_return
				else
					rt.jvm_return_code
				end
				if i = 1 and then boost then
				else
					code_attribute.resolve_u2_branch(point)
				end
				i := i - 1
			end
			if not boost then
				code_attribute.opcode_aload_0
				code_attribute.runtime_error_bad_target(up_rf.start_position, up_rf.current_type,
																	 once "System-level Validity error detected inside virtual %
																			%switching code.%N%
																			%Bad target type for dynamic dispatch.")
				if rt = Void then
					code_attribute.opcode_return
				else
					space := rt.jvm_push_default
					rt.jvm_return_code
				end
			end
		end

	tmp_jvmd: STRING is
		once
			create Result.make(32)
		end

	c_dicho (desc: SWITCH_SITE; bi, bs: INTEGER) is
			-- Produce dichotomic inspection code for Current id.
		require
			bi <= bs
		local
			m: INTEGER; dyn_lt: LIVE_TYPE; dyn_rf: RUN_FEATURE
		do
			if bi = bs then
				dyn_rf := desc.item(bi)
				dyn_lt := dyn_rf.type_of_current.live_type
				cpp.new_inside_switch_call(dyn_rf, desc.arguments, desc.current_type, desc.result_type)
			else
				m := (bi + bs) #// 2
				dyn_rf := desc.item(m)
				dyn_lt := dyn_rf.type_of_current.live_type
				cpp.put_string(once "if(id<=")
				cpp.put_integer(dyn_lt.id)
				cpp.put_string(fz_62)
				c_dicho(desc, bi, m)
				cpp.put_string(once "}%Nelse{%N")
				c_dicho(desc, m + 1, bs)
				cpp.put_character('}')
			end
		end

	new_c_switch (desc: SWITCH_SITE) is
			-- Produce C switch inspection code for Current id.
		local
			i: INTEGER; dyn_lt: LIVE_TYPE; dyn_rf: RUN_FEATURE
		do
			cpp.put_string(once "switch(id){%N")
			from
				i := desc.count
			until
				i < 1
			loop
				dyn_rf := desc.item(i)
				dyn_lt := dyn_rf.type_of_current.live_type
				cpp.put_string(once "case ")
				cpp.put_integer(dyn_lt.id)
				cpp.put_character(':')
				cpp.new_inside_switch_call(dyn_rf, desc.arguments, desc.current_type, desc.result_type)
				cpp.put_string(once "%Nbreak;%N")
				i := i - 1
			end
			if ace.no_check then
				cpp.put_string(once "default: error2(C,position);%N")
			end
			cpp.put_string(fz_12)
		end

feature {}
	local_profile is
		require
			ace.profile
			cpp.on_c
		do
			smart_eiffel.local_profile
		end

	start_profile (desc: SWITCH_SITE) is
		require
			ace.profile
			cpp.on_c
		do
			cpp.put_string(once "local_profile.profile=switch_profile+")
			cpp.put_integer(desc.id)
			cpp.put_string(once ";%N")
			cpp.put_string(once "start_profile(parent_profile, &local_profile);%N")
		end

	stop_profile is
		require
			ace.profile
			cpp.on_c
		do
			smart_eiffel.stop_profile
		end

end -- class SWITCH
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
