-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class RUN_FEATURE_2

inherit
	RUN_FEATURE
		redefine base_feature, verify_scoop_expanded, prepare_introspection, prepare_introspection2
		end

creation {WRITABLE_ATTRIBUTE}
	for

feature {ANY}
	base_feature: WRITABLE_ATTRIBUTE

	result_type: TYPE_MARK

	is_deferred: BOOLEAN is False

	side_effect_free: BOOLEAN is True

	is_once_procedure: BOOLEAN is False

	is_once_function: BOOLEAN is False

	order: INTEGER
			-- Used for special sorting of LIVE_TYPE.writable_attributes

	arguments: FORMAL_ARG_LIST is
		do
		end

	require_assertion: E_REQUIRE is
		do
		end

	local_vars: LOCAL_VAR_LIST is
		do
		end

	routine_body: INSTRUCTION is
		do
		end

	rescue_compound: INSTRUCTION is
		do
		end

	ensure_assertion: E_ENSURE is
		do
		end

	put_c_field_name is
			--  Emit using `cpp' the corresponding field name with an extra C
			--  comment which include the offset of the corresponding C field.
			--  This extra C comment is mandatory to force C recompilation,
			--  because the field may move from one compilation to another.
		do
			cpp.put_character('_')
			cpp.put_string(name.to_string)
			cpp.put_recompilation_comment(live_type.offset_of(Current))
		end

	mapping_c is
		do
			cpp.put_string(once "(/*RF2*/")
			check
				-- No more attribute in kernel expanded classes.
				no_more_item: not type_of_current.is_kernel_expanded
			end
			if type_of_current.is_separate then
				cpp.put_character('(')
				cpp.put_target_as_target
				cpp.put_string(once ")->ref->")
				put_c_field_name
			elseif type_of_current.is_reference then
				cpp.put_character('(')
				cpp.put_target_as_target
				cpp.put_string(once ")->")
				put_c_field_name
			else
				check
					type_of_current.is_user_expanded
				end
				cpp.put_character('(')
				cpp.put_target_as_value
				cpp.put_string(once ").")
				put_c_field_name
			end
			cpp.put_character(')')
		end

	c_define is
		do
		end

feature {}
	do_adapt is
		do
		end

	set_result_type is
		do
			-- Adapt the result type:
			result_type := base_feature.result_type.resolve_in(type_of_current).canonical_type_mark --|*** CAD: need of resolve_in?
		ensure
			result_type /= Void
		end

feature {LIVE_TYPE}
	set_order (o: like order) is
		require
			o >= 0
		do
			order := o
		ensure
			order = o
		end

feature {ADDRESS_OF_POOL}
	address_of_c_define (caller: ADDRESS_OF) is
		do
		end

feature {ADDRESS_OF}
	address_of_c_mapping is
		do
			cpp.put_string(once "&(")
			cpp.print_current
			cpp.put_string(once "->_")
			cpp.put_string(name.to_string)
			cpp.put_character(')')
		end

feature {}
	compute_use_current is
		do
			use_current_state := True_state
		end

feature {LIVE_TYPE}
	jvm_field_or_method is
		do
			jvm.add_field(Current)
		end

	verify_scoop_expanded (p: POSITION) is
		do
			default_verify_scoop_expanded(p)
		end

feature {ANY}
	mapping_jvm is
		local
			idx: INTEGER; stack_level: INTEGER
		do
			jvm.push_target_as_target
			if current_type.is_kernel_expanded then
				check
					as_item = name.to_string
				end
			else
				stack_level := result_type.jvm_stack_space - 1
				idx := constant_pool.idx_fieldref(Current)
				code_attribute.opcode_getfield(idx, stack_level)
			end
		end

feature {LIVE_TYPE}
	prepare_introspection (put_else: BOOLEAN): BOOLEAN is
		local
			lt: LIVE_TYPE
		do
			lt := result_type.live_type
			if lt /= Void then
				cpp.push_introspect(type_of_current, Current)
				if put_else then
					cpp.put_string(once "else ")
				end
				cpp.put_string(once "if (!strcmp(attr,%"")
				cpp.put_string(name.to_string)
				cpp.put_string(once "%")) {%N")
				lt.c_return_introspect(Current)
				cpp.put_string(once "}%N")
				cpp.pop
				Result := True
			else
				Result := put_else
			end
		end

	prepare_introspection2 (put_coma: BOOLEAN): BOOLEAN is
		local
			lt: LIVE_TYPE
		do
			lt := result_type.live_type
			if lt /= Void then
				if put_coma then
					cpp.put_string(once ", ")
				else
					cpp.put_character('"')
				end
				cpp.put_string(name.to_string)
				Result := True
			else
				Result := put_coma
			end
		end

feature {JVM}
	jvm_define is
		local
			name_idx, descriptor: INTEGER; cp: like constant_pool; access_flags: INTEGER
		do
			cp := constant_pool
			name_idx := cp.idx_utf8(name.to_string)
			descriptor := cp.idx_utf8(jvm_descriptor)
			access_flags := field_info.field_modifier(name_idx)
			field_info.add(access_flags, name_idx, descriptor)
		end

feature {}
	update_tmp_jvm_descriptor is
		local
			rt: TYPE_MARK
		do
			rt := result_type
			if rt.is_reference then
				tmp_jvm_descriptor.append(jvm_root_descriptor)
			else
				rt.jvm_descriptor_in(tmp_jvm_descriptor)
			end
		end

	c_define_body is
		do
			check
				False
			end
		end

invariant
	order >= 0

end -- class RUN_FEATURE_2
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
