-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class INHERIT_TEXT
	--
	-- To store the parent list of a class.
	--

inherit
	VISITABLE
insert
	GLOBALS

creation {CLASS_TEXT}
	make

feature {ANY}
	class_text: CLASS_TEXT
			-- Where the parent list is written.

	inherit_comment: COMMENT
			-- Comment just after the "inherit" keyword.

	
	insert_comment: COMMENT
			-- Comment just after the "insert" keyword.

	default_insert_any_added_flag: BOOLEAN
			-- This flag indicates that ANY has been automatically added in the `insert_list' in order to reach 
			-- the canonical form (necessary for `pretty' and for error messages printing).
	
	class_text_name: CLASS_NAME is
		do
			Result := class_text.name
		end

 	has_redefine (fn: FEATURE_NAME): BOOLEAN is
 		require
 			fn /= Void
 		local
 			i: INTEGER
 		do
			if inherit_list /= Void then
				from
					i := inherit_list.upper
				until
					Result or else i < inherit_list.lower
				loop
					Result := inherit_list.item(i).has_redefine(fn)
					i := i - 1
				end
			end
			if insert_list /= Void then
				from
					i := insert_list.upper
				until
					Result or else i < insert_list.lower
				loop
					Result := insert_list.item(i).has_redefine(fn)
					i := i - 1
				end
			end
 		end

	accept (visitor: INHERIT_TEXT_VISITOR) is
		do
			visitor.visit_inherit_text(Current)
		end

feature {CLASS_TEXT, LIVE_TYPE}
 	inherit_count: INTEGER is
 		do
			if inherit_list /= Void then
				Result := inherit_list.count
			end
		end

feature {CLASS_TEXT}
 	undefine_position_of (fn: FEATURE_NAME): POSITION is
 			--|*** PH(31/03/04): I think it should be removed.
 			-- Search the POSITION of `fn' in one undefine lists.
 		require
 			fn /= Void
 		local
 			fn2: FEATURE_NAME; i: INTEGER
 		do
			if inherit_list /= Void then
				from
					i := inherit_list.upper
				until
					fn2 /= Void
				loop
					fn2 := inherit_list.item(i).get_undefine(fn)
					i := i - 1
				end
			end
			if insert_list /= Void then
				from
					i := insert_list.upper
				until
					fn2 /= Void
				loop
					fn2 := insert_list.item(i).get_undefine(fn)
					i := i - 1
				end
 			end
 			Result := fn2.start_position
 		ensure
 			not Result.is_unknown
 		end

 	id_extra_information (tfw: TEXT_FILE_WRITE) is
 		local
 			i: INTEGER
 		do
			if inherit_list /= Void then
				i := inherit_list.count
			end
			if insert_list /= Void then
				i := i + insert_list.count
			end
 			tfw.put_integer(i)
			if inherit_list /= Void then
				tfw.put_string(once " inherit: ")
				from
					i := inherit_list.lower
				until
					i > inherit_list.upper
				loop
					inherit_list.item(i).id_extra_information(tfw)
					i := i + 1
					if i <= inherit_list.upper then
						tfw.put_character(' ')
					end
				end
			end
			if insert_list /= Void then
				tfw.put_string(once " insert: ")
				from
					i := insert_list.lower
				until
					i > insert_list.upper
				loop
					insert_list.item(i).id_extra_information(tfw)
					tfw.put_character(' ')
					i := i + 1
				end
			end
 		end

 	has (fn: FEATURE_NAME): BOOLEAN is
			--|*** (PH 07/09/04) TYPEs should be used
		local
 			i: INTEGER; iti: INHERIT_TEXT_ITEM
 		do
			if inherit_list /= Void then
				from
					i := inherit_list.upper
				until
					Result or else i < inherit_list.lower
				loop
					Result := inherit_list.item(i).is_target_of_rename(fn)
					i := i - 1
				end
			end
			if insert_list /= Void then
				from
					i := insert_list.upper
				until
					Result or else i < insert_list.lower
				loop
					Result := insert_list.item(i).is_target_of_rename(fn)
					i := i - 1
				end
			end
			if inherit_list /= Void then
				from
					i := inherit_list.upper
				until
					Result or else i < inherit_list.lower
				loop
					iti := inherit_list.item(i)
					if not iti.is_source_of_rename(fn) then
						Result := iti.class_text.has(fn)
					end
					i := i - 1
				end
			end
			if insert_list /= Void then
				from
					i := insert_list.upper
				until
					Result or else i < insert_list.lower
				loop
					iti := insert_list.item(i)
					if not iti.is_source_of_rename(fn) then
						Result := iti.class_text.has(fn)
					end
					i := i - 1
				end
			end
 		end

 	has_parent (c: CLASS_TEXT): BOOLEAN is
		--|*** (PH 06/09/04) remove this function.
 		require
 			not c.is_any
 		local
 			i: INTEGER; bc: CLASS_TEXT
 		do
			if inherit_list /= Void then
				from
					i := inherit_list.upper
				until
					Result or else i < inherit_list.lower
				loop
					bc := inherit_list.item(i).class_text
					check
						bc /= Void
					end
					Result := c = bc or else bc.is_subclass_of(c)
					i := i - 1
				end
			end
			if insert_list /= Void then
				from
					i := insert_list.upper
				until
					Result or else i < insert_list.lower
				loop
					bc := insert_list.item(i).class_text
					check
						bc /= Void
					end
					Result := c = bc or else bc.is_subclass_of(c)
					i := i - 1
				end
 			end
 		end

 	header_comment_for (ci: CLASS_INVARIANT) is
 		local
 			i: INTEGER
 		do
			if inherit_list /= Void then
				from
					i := inherit_list.upper
				until
					i < inherit_list.lower or else ci.header_comment /= Void
				loop
					inherit_list.item(i).class_text.header_comment_for(ci)
					i := i - 1
				end
			end
			if insert_list /= Void then
				from
					i := insert_list.upper
				until
					i < insert_list.lower or else ci.header_comment /= Void
				loop
					insert_list.item(i).class_text.header_comment_for(ci)
					i := i - 1
				end
			end
 		end

 	get_started is
 		local
 			i: INTEGER
 		do
			if inherit_list /= Void then
				from
					i := inherit_list.upper
				until
					i < inherit_list.lower
				loop
					inherit_list.item(i).get_started(Current)
					i := i - 1
				end
			end
			if insert_list /= Void then
				from
					i := insert_list.upper
				until
					i < insert_list.lower
				loop
					insert_list.item(i).get_started(Current)
					i := i - 1
				end
			end
 		end

	pretty is
		local
			i: INTEGER
		do
			pretty_printer.set_indent_level(0)
			if not pretty_printer.zen_mode then
				pretty_printer.skip_one_line
			end
			if inherit_list /= Void then
				pretty_printer.set_indent_level(0)
				pretty_printer.keyword(once "inherit")
				pretty_printer.set_indent_level(1)
				if inherit_comment /= Void then
					inherit_comment.pretty(1)
				end
				from
					i := inherit_list.lower
				until
					i > inherit_list.upper
				loop
					inherit_list.item(i).pretty
					i := i + 1
				end
				pretty_printer.set_indent_level(0)
			end
			if insert_list /= Void and then not default_insert_any_added_flag then
				if inherit_list /= Void then
					pretty_printer.set_indent_level(0)
					pretty_printer.put_character('%N')
				end
				pretty_printer.set_indent_level(0)
				pretty_printer.keyword(once "insert")
				pretty_printer.set_indent_level(1)
				if insert_comment /= Void then
					insert_comment.pretty(1)
				end
				if not default_insert_any_added_flag then
					from
						i := insert_list.lower
					until
						i > insert_list.upper
					loop
						insert_list.item(i).pretty
						i := i + 1
					end
				end
			end
			pretty_printer.set_indent_level(0)
		end

feature {TYPE, INHERIT_TEXT_VISITOR, MAKE_VISITORS}
	inherit_list: FAST_ARRAY[INHERIT_TEXT_ITEM]
			-- When non Void, this `inherit_list' contains only conformant parents.
			-- (Items are in the same order as they are in the source text file.)

	insert_list: FAST_ARRAY[INHERIT_TEXT_ITEM]
			-- When non Void, this `inherit_list' contains only non-conformant parents.
			-- (Items are in the same order as they are in the source text file.)

feature {}
 	insert_count: INTEGER is
 		do
			if insert_list /= Void then
				Result := insert_list.count
			end
		end

	make (ct: like class_text;	hc1: like inherit_comment; l1: like inherit_list;
			hc2: like insert_comment; l2: like insert_list) is
		require
			not ct.is_any -- Because class ANY must have no parent to avoid cycles.
		local
			any_type_mark: ANY_TYPE_MARK
		do
			class_text := ct
			inherit_comment := hc1
			inherit_list := l1
			insert_comment := hc2
			insert_list := l2
			if insert_list = Void and then inherit_list = Void then
				default_insert_any_added_flag := True
				create insert_list.with_capacity(1)
				create any_type_mark.make(ct.name.start_position)
				insert_list.add_last(create {INHERIT_TEXT_ITEM}.make(True, any_type_mark))
				--|*** *
				--|*** Change the creation name ! *** (Dom Nov 11th 2004) ***
			end
		ensure
			class_text = ct
			inherit_comment = hc1
			inherit_list = l1
			insert_comment = hc2
         default_insert_any_added_flag xor insert_list = l2
		end

	em1: STRING is "Inheritance clash."

invariant
	class_text /= Void

	insert_list /= Void implies not insert_list.is_empty

	inherit_list /= Void implies not inherit_list.is_empty;

	(not class_text.is_any) implies (inherit_count + insert_count > 0)

end -- class INHERIT_TEXT
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
