-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class RENAME_PAIR
	--
	-- To store a rename pair of Eiffel construct :
	--    "inherit ... rename ... as ... end"
	--

inherit
	VISITABLE
insert
	GLOBALS

creation {ANY}
	make

feature {ANY}
	accept (visitor: RENAME_PAIR_VISITOR) is
		do
			visitor.visit_rename_pair(Current)
		end

feature {ANY}
	old_name, new_name: FEATURE_NAME

	make (on, nn: like old_name) is
		require
			on /= Void
			nn /= Void
		do
			if on.to_string = nn.to_string then
				error_handler.add_position(on.start_position)
				error_handler.add_position(nn.start_position)
				error_handler.append("New name and old name must be different.")
				error_handler.print_as_fatal_error
			end
			old_name := on
			new_name := nn
		ensure
			old_name = on
			new_name = nn
		end

	pretty (rank: INTEGER) is
		do
			if rank > 1 then
				pretty_printer.set_indent_level(3)
			end
			old_name.declaration_pretty_print
			pretty_printer.keyword(once "as")
			new_name.declaration_pretty_print
		end

end -- class RENAME_PAIR
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
